﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using Xamarin.Forms;

namespace TVMediaHub.Tizen.Controls
{
    /// <summary>
    /// A custom grid view for displaying the button of MediaHub application
    /// </summary>
    public partial class MediaHubButton : Grid
    {
        /// <summary>
        /// An Event Handler to notify if the button is clicked
        /// </summary>
        public event EventHandler Clicked;

        /// <summary>
        /// A value of effect duration
        /// </summary>
        int effectDuration = 200;

        /// <summary>
        /// Gets or sets MediaHubButtonIcon's image source
        /// </summary>
        public string MediaHubButtonIcon
        {
            get
            {
                return (BtnIcon.Source as FileImageSource).File;
            }

            set
            {
                BtnIcon.Source = value;
            }
        }

        /// <summary>
        /// Gets or sets MediaHubButtonIcon's button object
        /// </summary>
        public Button MediaHubButtonBtn
        {
            get
            {
                return Btn;
            }
        }

        /// <summary>
        /// Gets or sets MediaHubButtonIcon's normal image source
        /// </summary>
        public string NormalImage
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets MediaHubButtonIcon's pressed image source
        /// </summary>
        public string PressedImage
        {
            get;
            set;
        }

        /// <summary>
        /// A constructor of MediaHubBUtton
        /// Initializes the MediaHubButton and the button's BG image animation
        /// </summary>
        public MediaHubButton()
        {
            InitializeComponent();

            InitializeButton();
            InitializeFocusBG();
        }

        /// <summary>
        /// Initializes button's event handler
        /// </summary>
        private void InitializeButton()
        {
            Btn.Clicked += (s, e) =>
            {
                Clicked?.Invoke(this, EventArgs.Empty);
                var BGScale = BG.Scale;
                var BGOpacity = BG.Opacity;

                this.AbortAnimation("ClickAnimation");
                BtnIcon.Source = PressedImage;
                this.Animate("ClickAnimation", (v) =>
                 {
                     BG.Scale = BGScale - (BGScale - 0.1) * v;
                     BG.Opacity = BGOpacity - (BGOpacity - 0) * v;
                 },
                 length: Convert.ToUInt32(effectDuration),
                 easing: Easing.CubicInOut,
                 finished: (progress, isCanceled) =>
                 {
                     BtnIcon.Source = NormalImage;
                     if (!isCanceled)
                     {
                         this.Animate("ClickAnimation", (v) =>
                         {
                             BG.Scale = 0.1 + 0.9 * v;
                             BG.Opacity = v;
                         },
                         length: Convert.ToUInt32(effectDuration),
                         easing: Easing.CubicInOut,
                         finished: (progress2, isCanceled2) =>
                         {
                             BG.Scale = 1;
                             BGOpacity = 1;
                         });
                     }
                     else
                     {
                         BG.Scale = 1;
                         BGOpacity = 1;
                     }
                 });
            };
        }

        /// <summary>
        /// Initializes button's animation when the button is focused or unfocused
        /// </summary>
        private void InitializeFocusBG()
        {
            Btn.Focused += (s, e) =>
            {
                BG.ScaleTo(1, Convert.ToUInt32(effectDuration), Easing.CubicInOut);
                BG.FadeTo(1, Convert.ToUInt32(effectDuration), Easing.CubicInOut);
            };

            Btn.Unfocused += (s, e) =>
            {
                BG.ScaleTo(0.1, Convert.ToUInt32(effectDuration), Easing.CubicInOut);
                BG.FadeTo(0, Convert.ToUInt32(effectDuration), Easing.CubicInOut);
            };
        }
    }
}
