﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Tizen.Content.MediaContent;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.DataModels;

namespace TVMediaHub.Tizen.Models
{
    /// <summary>
    /// An enumeration for sort option of contents
    /// </summary>
    public enum SortOption
    {
        Date,   // Video, Image, Music
        Title,  // Video, Image, Music
        Album,  // Music
        Artist, // Music
        Genre,  // Video, Music
        Folder, // Music
        Type,   // Image
    }

    /// <summary>
    /// A class for providing media contents
    /// </summary>
    public abstract class ContentProvider
    {
        /// <summary>
        /// A method for getting condition for selection query
        /// </summary>
        /// <returns>A condition string</returns>
        abstract protected string GetConditionStringForSelection();

        /// <summary>
        /// An abstract method to determine the availability of the content.
        /// </summary>
        /// <param name="mediaContent">A media content to be checked the availability</param>
        abstract protected void CheckUnavailableContent(MediaInformationEx mediaContent);

        /// <summary>
        /// A method for handling ContentUpdatedEvent
        /// </summary>
        /// <param name="listener">A handling method</param>
        abstract public void SetContentUpdatedEventListener(EventHandler listener);

        /// <summary>
        /// An EventHandler be sent to Image, Video and MusicProvider when ContentUpdated event is triggered
        /// </summary>
        public EventHandler ContentUpdateFinished;

        /// <summary>
        /// A constructor
        /// Connect to the media database to search, insert, remove or modify media information.
        /// </summary>
        public ContentProvider()
        {
            try
            {
                MediaDatabase.MediaInfoUpdated += MediaDatabaseMediaUpdated;
            }
            catch (Exception e)
            {
                DbgPort.E(e.Message);
            }

        }

        /// <summary>
        /// Handles ContentUpdated event that is triggered when db is changed.
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A ContentUpdatedEvent arguments</param>
        private void MediaDatabaseMediaUpdated(object sender, MediaInfoUpdatedEventArgs e)
        {
            if (e.OperationType == OperationType.Update)
            {
                MediaHubImpl.GetInstance.StorageProviderInstance.CheckStorage();
                ContentUpdateFinished?.Invoke(this, null);
            }
        }

        /// <summary>
        /// Converts the value of the DateTime object to its equivalent string representation.
        /// </summary>
        /// <param name="date">A DateTime information</param>
        /// <returns>A string to be displayed</returns>
        private string GetDateString(DateTime date)
        {
            if (date.Equals(DateTime.Today))
            {
                return "Today";
            }
            else if (date.Equals(DateTime.Today.AddDays(-1)))
            {
                return "Yesterday";
            }

            return date.ToString("ddd. d MMMM");
        }

        /// <summary>
        /// A method for creating group item to be added to the proper group
        /// </summary>
        /// <param name="sortOption">A current sort option</param>
        /// <param name="lastGroupItem">A latest group item of current group</param>
        /// <param name="mediaInformationEx">A media information to be converted to group item</param>
        /// <returns>A group item to be added</returns>
        private GroupItem GetGroupItem(SortOption sortOption, GroupItem lastGroupItem, MediaInformationEx mediaInformationEx)
        {
            GroupItem currentGroupItem = lastGroupItem;
            string newTitle = null;
            bool newGroupFlag = false;

            switch (sortOption)
            {
                case SortOption.Title:
                    if (lastGroupItem == null || lastGroupItem.Title != mediaInformationEx.MediaContentInformation.DisplayName[0].ToString())
                    {
                        newGroupFlag = true;
                        newTitle = mediaInformationEx.MediaContentInformation.DisplayName[0].ToString();
                    }

                    break;
                case SortOption.Date:
                    if (lastGroupItem == null || lastGroupItem.Title != GetDateString(mediaInformationEx.MediaContentInformation.Timeline.DateTime))
                    {
                        newGroupFlag = true;
                        newTitle = GetDateString(mediaInformationEx.MediaContentInformation.Timeline.DateTime);
                    }

                    break;
                case SortOption.Genre:
                    if (lastGroupItem == null || lastGroupItem.Title != mediaInformationEx.MediaContentInformation.Category)
                    {
                        newGroupFlag = true;
                        newTitle = mediaInformationEx.MediaContentInformation.Category;
                    }

                    break;
                case SortOption.Type:
                    if (lastGroupItem == null || lastGroupItem.Title != mediaInformationEx.MediaContentInformation.MediaType.ToString())
                    {
                        newGroupFlag = true;
                        newTitle = mediaInformationEx.MediaContentInformation.MediaType.ToString();
                    }

                    break;
                case SortOption.Album:
                    if (lastGroupItem == null || lastGroupItem.Title != (mediaInformationEx.MediaContentInformation as AudioInfo)?.Album)
                    {
                        newGroupFlag = true;
                        newTitle = (mediaInformationEx.MediaContentInformation as AudioInfo)?.Album.ToString();
                    }

                    break;
                case SortOption.Artist:
                    if (lastGroupItem == null || lastGroupItem.Title != (mediaInformationEx.MediaContentInformation as AudioInfo)?.Artist)
                    {
                        newGroupFlag = true;
                        newTitle = (mediaInformationEx.MediaContentInformation as AudioInfo)?.Artist.ToString();
                    }

                    break;
                default:
                    throw new System.ArgumentException("Invalid sorting option.");
            }

            if (newGroupFlag == true)
            {
                currentGroupItem = new GroupItem();
                currentGroupItem.Title = newTitle;
                currentGroupItem.Contents = new List<MediaShortcutInfo>();
            }

            return currentGroupItem;
        }

        /// <summary>
        /// A method for making group to be displayed from MediaInformationEx list
        /// </summary>
        /// <param name="mediaInformationExList">A list of MediaInformationEx</param>
        /// <param name="sortOption">The current sort option</param>
        /// <returns>A list of group item</returns>
        private async Task<IEnumerable<GroupItem>> MakeGroupAsync(IEnumerable<MediaInformationEx> mediaInformationExList, SortOption sortOption)
        {
            return await Task.Run(() =>
            {
                List<GroupItem> result = new List<GroupItem>();
                GroupItem lastGroupItem = null;
                GroupItem currentGroupItem = null;

                if (mediaInformationExList == null)
                {
                    throw new System.ArgumentException("mediaInformationExList must not be null.");
                }

                foreach (MediaInformationEx mediaInformationEx in mediaInformationExList)
                {
                    var currentInformation = mediaInformationEx;
                    var shortcutInfo = new MediaShortcutInfo(currentInformation);

                    // TODO : The catch implementation should be checked once again.
                    try
                    {
                        currentGroupItem = GetGroupItem(sortOption, currentGroupItem, mediaInformationEx);
                    }
                    catch (Exception e)
                    {
                        DbgPort.E(e.Message);
                        return null;
                    }

                    if (lastGroupItem != currentGroupItem)
                    {
                        result.Add(currentGroupItem);
                        lastGroupItem = currentGroupItem;
                    }

                    if (currentGroupItem != null)
                    {
                        currentGroupItem.Contents.Add(shortcutInfo);
                    }
                }

                return result;
            });
        }

        /// <summary>
        /// A method for reading media informations from database without group information
        /// </summary>
        /// <param name="sortOption">The current sort option</param>
        /// <param name="storageId">The current storage id</param>
        /// <param name="offset">The start position of the given filter Starting from zero</param>
        /// <param name="count">The number of items to be searched with respect to the offset</param>
        /// <returns>A list of media informations</returns>
        public IEnumerable<MediaInformationEx> ReadWithoutGroup(SortOption sortOption, string storageId = null, int offset = -1, int count = -1)
        {
            // Makes Content Filter by arguments
            var selectArguments = new SelectArguments();

            switch (sortOption)
            {
                case SortOption.Title:
                    selectArguments.SortOrder = "MEDIA_DISPLAY_NAME";
                    break;
                case SortOption.Date:
                    selectArguments.SortOrder = "MEDIA_RECORDED_DATE";
                    break;
                case SortOption.Genre:
                    selectArguments.SortOrder = "MEDIA_GENRE";
                    break;
                case SortOption.Type:
                    selectArguments.SortOrder = "MEDIA_TYPE";
                    break;
                case SortOption.Album:
                    selectArguments.SortOrder = "MEDIA_ALBUM";
                    break;
                case SortOption.Artist:
                    selectArguments.SortOrder = "MEDIA_ARTIST";
                    break;
                default:
                    throw new System.ArgumentException("Invalid sorting option.");
            }

            if (offset >= 0)
            {
                selectArguments.StartRowIndex = offset;
            }

            if (count >= 0)
            {
                selectArguments.TotalRowCount = count;
            }

            if (storageId != null)
            {
                selectArguments.StorageId = storageId;
            }

            selectArguments.FilterExpression = GetConditionStringForSelection();
            // Executes the 'select' query
            List<MediaInfo> mediaInformationList = new List<MediaInfo>();
            List<MediaInformationEx> mediaInformationExList = new List<MediaInformationEx>();
            try
            {
                var reader = MediaHubImpl.GetInstance.MediaInfoCommand.SelectMedia(selectArguments);

                while (reader.Read())
                {
                    mediaInformationList.Add(reader.Current);
                }

                foreach (MediaInfo mediaInformation in mediaInformationList)
                {
                    var mediaInformationEx = new MediaInformationEx
                    {
                        MediaContentInformation = mediaInformation
                    };

                    CheckUnavailableContent(mediaInformationEx);
                    mediaInformationExList.Add(mediaInformationEx);
                }
            }
            catch (Exception exception)
            {
                DbgPort.E(exception.Message);
            }

            return mediaInformationExList;
        }

        /// <summary>
        /// A method for making thumbnail of Media contents.
        /// </summary>
        /// <param name="list">A list of media contents</param>
        /// <returns>A path of thumbnail</returns>
        public async Task CheckThumbnail(IEnumerable<MediaInformationEx> list)
        {
            foreach (var info in list)
            {
                if (string.IsNullOrEmpty(info.MediaContentInformation.ThumbnailPath))
                {
                    try
                    {
                        string path = await MediaHubImpl.GetInstance.MediaInfoCommand.CreateThumbnailAsync(info.MediaContentInformation.Id);
                    }
                    catch (Exception e)
                    {
                        DbgPort.D(e.Message);
                    }
                }
            }
        }

        /// <summary>
        /// A method for reading media informations from database without group information asynchronously
        /// </summary>
        /// <param name="sortOption">The current sort option</param>
        /// <param name="storageId">The current storage id</param>
        /// <param name="offset">The start position of the given filter Starting from zero</param>
        /// <param name="count">The number of items to be searched with respect to the offset</param>
        /// <returns>A list of media informations</returns>
        public async Task<IEnumerable<MediaInformationEx>> ReadWithoutGroupAsync(SortOption sortOption, string storageId = null, int offset = -1, int count = -1)
        {
            return await Task.Run(() => {
                // Makes Content Filter by arguments
                var selectArguments = new SelectArguments();

                switch (sortOption)
                {
                    case SortOption.Title:
                        selectArguments.SortOrder = "MEDIA_DISPLAY_NAME";
                        break;
                    case SortOption.Date:
                        selectArguments.SortOrder = "MEDIA_RECORDED_DATE";
                        break;
                    case SortOption.Genre:
                        selectArguments.SortOrder = "MEDIA_GENRE";
                        break;
                    case SortOption.Type:
                        selectArguments.SortOrder = "MEDIA_TYPE";
                        break;
                    case SortOption.Album:
                        selectArguments.SortOrder = "MEDIA_ALBUM";
                        break;
                    case SortOption.Artist:
                        selectArguments.SortOrder = "MEDIA_ARTIST";
                        break;
                    default:
                        throw new System.ArgumentException("Invalid sorting option.");
                }

                if (offset >= 0)
                {
                    selectArguments.StartRowIndex = offset;
                }

                if (count >= 0)
                {
                    selectArguments.TotalRowCount = count;
                }

                if (storageId != null)
                {
                    selectArguments.StorageId = storageId;
                }

                selectArguments.FilterExpression = GetConditionStringForSelection();

                // Executes the 'select' query
                List<MediaInfo> mediaInformationList = new List<MediaInfo>();
                List<MediaInformationEx> mediaInformationExList = new List<MediaInformationEx>();
                try
                {

                    var reader = MediaHubImpl.GetInstance.MediaInfoCommand.SelectMedia(selectArguments);

                    while (reader.Read())
                    {
                        mediaInformationList.Add(reader.Current);
                    }


                    foreach (MediaInfo mediaInformation in mediaInformationList)
                    {
                        var mediaInformationEx = new MediaInformationEx();

                        mediaInformationEx.MediaContentInformation = mediaInformation;
                        CheckUnavailableContent(mediaInformationEx);
                        mediaInformationExList.Add(mediaInformationEx);
                    }
                }
                catch (Exception exception)
                {
                    DbgPort.E(exception.Message);
                }

                return mediaInformationExList;
            });
        }

        /// <summary>
        /// A method for reading media contents from database and make a group
        /// </summary>
        /// <param name="sortOption">The current sort option</param>
        /// <param name="storageId">The current storage id</param>
        /// <param name="offset">The start position of the given filter Starting from zero</param>
        /// <param name="count">The number of items to be searched with respect to the offset</param>
        /// <returns>A list of group item</returns>
        public async Task<IEnumerable<GroupItem>> ReadAsync(SortOption sortOption, string storageId = null, int offset = -1, int count = -1)
        {
            IEnumerable<MediaInformationEx> list = await ReadWithoutGroupAsync(sortOption, storageId, offset, count);

            return await MakeGroupAsync(list, sortOption);
        }

        /// <summary>
        /// A method for reading media content with specific media id
        /// </summary>
        /// <param name="mediaID">A media Id to be read</param>
        /// <returns>A media content with specific media id</returns>
        public MediaInformationEx Read(String mediaID)
        {
            var mediaInformationEx = new MediaInformationEx();
            mediaInformationEx.MediaContentInformation = MediaHubImpl.GetInstance.MediaInfoCommand.SelectMedia(mediaID);
            return mediaInformationEx;
        }

        /// <summary>
        /// A method for deleting media content
        /// </summary>
        /// <param name="media">A media content to be deleted</param>
        /// <returns>Always returns true</returns>
        public bool Delete(MediaInformationEx media)
        {
            try
            {
                MediaHubImpl.GetInstance.MediaInfoCommand.Delete(media.MediaContentInformation.Id);
            }
            catch (Exception exception)
            {
                DbgPort.E(exception.Message);
            }

            return true;
        }

        /// <summary>
        /// A method for updating media content
        /// </summary>
        /// <param name="media">A media content to be updated</param>
        /// <returns>Always returns true</returns>
        public bool Update(MediaInformationEx media)
        {
            try
            {
                // @TODO : CHECK POINT 170824
                //MediaHubImpl.GetInstance.MediaInfoCommand.Update(media.MediaContentInformation.Id);
            }
            catch (Exception exception)
            {
                DbgPort.E(exception.Message);
            }

            return true;
        }
    }
}
