﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Threading;
using System.Threading.Tasks;
using Tizen.Content.MediaContent;
using Tizen.Multimedia;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Utils;

namespace TVMediaHub.Tizen.Models
{
    /// <summary>
    /// A custom EventArgs with string fields
    /// </summary>
    public class MusicPlayerInfoEventArgs : EventArgs
    {
        public string Title;
        public string Artist;
        public string AlbumCover;
        public string AlbumName;
    }

    /// <summary>
    /// A custom EventArgs with bool field
    /// </summary>
    public class MusicPlayerStateEventArgs : EventArgs
    {
        public bool IsPlaying;
    }

    /// <summary>
    /// A custom EventArgs with double field
    /// </summary>
    public class MusicPlayerProgressEventArgs : EventArgs
    {
        public double Progress;
    }

    /// <summary>
    /// An instance of the MusicPlayerModel
    /// </summary>
    public class MusicPlayerModel
    {
        /// <summary>
        /// A main instance class of the music player model instance
        /// </summary>
        private static readonly MusicPlayerModel instance = new MusicPlayerModel();

        /// <summary>
        /// An instance of the player
        /// </summary>
        private Player playerInstance;

        /// <summary>
        /// An timer for the progressbar
        /// </summary>
        private Timer progressbarTimer;

        /// <summary>
        /// The period of the timer
        /// </summary>
        private int timerPeriod = 1000;

        /// <summary>
        /// An event handler for handling the current music information
        /// </summary>
        private EventHandler<MusicPlayerInfoEventArgs> MusicPlayerInfoListener;

        /// <summary>
        /// An event handler for handling music player state change event
        /// </summary>
        private EventHandler<MusicPlayerStateEventArgs> MusicPlayerStateChanged;

        /// <summary>
        /// An event handler for handling the percentage of the progressbar
        /// </summary>
        private EventHandler<MusicPlayerProgressEventArgs> MusicPlayerProgressListener;

        /// <summary>
        /// The information of the current music
        /// </summary>
        public MediaInformationEx currentMusic;

        /// <summary>
        /// Gets or sets the current music
        /// </summary>
        public MediaInformationEx CurrentMusic
        {
            get
            {
                return currentMusic;
            }

            set
            {
                if (currentMusic != value)
                {
                    currentMusic = value;

                    /// 1. Stop the player
                    if (playerInstance.State == PlayerState.Playing || playerInstance.State == PlayerState.Paused)
                    {
                        StopPlayer();
                    }

                    /// 2. Play the current music
                    StartPlayerAsync((currentMusic.MediaContentInformation as AudioInfo).Path);

                    /// 3. Update the information of the current music
                    MusicPlayerInfoListener?.Invoke(this, new MusicPlayerInfoEventArgs()
                    {
                        Title = (currentMusic.MediaContentInformation as AudioInfo).Title,
                        Artist = (currentMusic.MediaContentInformation as AudioInfo).Artist,
                        AlbumCover = ((currentMusic.MediaContentInformation as AudioInfo).ThumbnailPath.Length != 0) ? (currentMusic.MediaContentInformation as AudioInfo).ThumbnailPath : "img_media_no_contents.png",
                        AlbumName = (currentMusic.MediaContentInformation as AudioInfo).Album
                    });

                    /// 4. Update the progressbar of the current music
                    MusicPlayerProgressListener?.Invoke(this, new MusicPlayerProgressEventArgs()
                    {
                        Progress = (playerInstance.State == PlayerState.Idle || playerInstance.State == PlayerState.Preparing) ? 0 : Convert.ToDouble(playerInstance.GetPlayPosition()) / (currentMusic.MediaContentInformation as AudioInfo).Duration,
                    });

                    MusicPlayerStateChanged?.Invoke(this, new MusicPlayerStateEventArgs()
                    {
                        IsPlaying = true
                    });
                }
            }
        }

        /// <summary>
        /// Gets an instance of the music player model
        /// </summary>
        public static MusicPlayerModel Instance
        {
            get
            {
                return instance;
            }
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public MusicPlayerModel()
        {
            InitializePlayer();
            InitializeProgressbarTimer();
        }

        /// <summary>
        /// Initialize the player instance
        /// Register event handler for the player
        /// </summary>
        private void InitializePlayer()
        {
            playerInstance = new Player();
            playerInstance.PlaybackCompleted += ((s, e) =>
            {
                StopPlayer();

                MusicPlayerStateChanged?.Invoke(this, new MusicPlayerStateEventArgs()
                {
                    IsPlaying = false
                });
            });

            playerInstance.PlaybackInterrupted += ((s, e) =>
            {
                DbgPort.E("Error : " + Enum.GetName(typeof(PlayerError), e.Reason));

                StopPlayer();

                MusicPlayerStateChanged?.Invoke(this, new MusicPlayerStateEventArgs()
                {
                    IsPlaying = false
                });
            });

            playerInstance.ErrorOccurred += ((s, e) =>
            {
                DbgPort.E("Error : " + Enum.GetName(typeof(PlayerError), e.Error));

                StopPlayer();

                MusicPlayerStateChanged?.Invoke(this, new MusicPlayerStateEventArgs()
                {
                    IsPlaying = false
                });
            });
        }

        /// <summary>
        /// Initialize the timer for the progressbar
        /// </summary>
        private void InitializeProgressbarTimer()
        {
            progressbarTimer = new Timer((object o) =>
            {
                MusicPlayerProgressListener?.Invoke(this, new MusicPlayerProgressEventArgs()
                {
                    Progress = (playerInstance.State == PlayerState.Idle || playerInstance.State == PlayerState.Preparing) ? 0 : Convert.ToDouble(playerInstance.GetPlayPosition()) / (currentMusic.MediaContentInformation as AudioInfo).Duration,
                });
            }, null, Timeout.Infinite, Timeout.Infinite);
        }

        /// <summary>
        /// Starts the timer for the progressbar
        /// </summary>
        private void StartProgressbarTimer()
        {
            progressbarTimer.Change(0, timerPeriod);
        }

        /// <summary>
        /// Stops the timer for the porgressbar
        /// </summary>
        private void StopProgressbarTimer()
        {
            progressbarTimer.Change(Timeout.Infinite, Timeout.Infinite);
        }

        /// <summary>
        /// Plays the current music
        /// </summary>
        /// <param name="filePath">The file path of the media source</param>
        public void StartPlayerAsync(string filePath)
        {
            MediaSource source = new MediaUriSource(filePath);
            playerInstance.SetSource(source);

            Task.Run(async () =>
            {
                await playerInstance.PrepareAsync();

                playerInstance.Start();

                StartProgressbarTimer();
            });
        }

        /// <summary>
        /// Toggles pause and play the current music
        /// </summary>
        public void TogglePausePlay()
        {
            if (playerInstance.State == PlayerState.Playing)
            {
                try
                {
                    playerInstance.Pause();

                    MusicPlayerStateChanged?.Invoke(this, new MusicPlayerStateEventArgs()
                    {
                        IsPlaying = false
                    });
                }
                catch (Exception e)
                {
                    DbgPort.E("Error : " + e.Message);
                }
                StopProgressbarTimer();
            }
            else if (playerInstance.State == PlayerState.Paused)
            {
                try
                {
                    playerInstance.Start();

                    MusicPlayerStateChanged?.Invoke(this, new MusicPlayerStateEventArgs()
                    {
                        IsPlaying = true
                    });
                }
                catch (Exception e)
                {
                    DbgPort.E("Error : " + e.Message);
                }
                StartProgressbarTimer();
            }
        }

        /// <summary>
        /// Stops the current music
        /// </summary>
        public void StopPlayer()
        {
            StopProgressbarTimer();

            try
            {
                playerInstance.Stop();
                playerInstance.Unprepare();
            }
            catch (Exception e)
            {
                DbgPort.E("Error : " + e.Message);
            }
        }

        /// <summary>
        /// A method adds EventHandler to SetCurrentMusicInfoListener
        /// </summary>
        /// <param name="listener">The EventHandler for adding</param>
        public void SetCurrentMusicInfoListener(EventHandler<MusicPlayerInfoEventArgs> listener)
        {
            MusicPlayerInfoListener += listener;
        }

        /// <summary>
        /// A method adds EventHandler to SetCurrentMusicProgressListener
        /// </summary>
        /// <param name="listener">The EventHandler for adding</param>
        public void SetCurrentMusicProgressListener(EventHandler<MusicPlayerProgressEventArgs> listener)
        {
            MusicPlayerProgressListener += listener;
        }

        /// <summary>
        /// A method adds EventHandler to SetCurrentMusicProgressListener
        /// </summary>
        /// <param name="listener">The EventHandler for adding</param>
        public void SetMusicPlayerStateChangedListener(EventHandler<MusicPlayerStateEventArgs> listener)
        {
            MusicPlayerStateChanged += listener;
        }

        /// <summary>
        /// A method adds EventHandler to SetCurrentMusicProgressListener
        /// </summary>
        /// <param name="listener">The EventHandler for adding</param>
        public void SetMusicPlayCompletedListener(EventHandler<EventArgs> listener)
        {
            playerInstance.PlaybackCompleted += listener;
        }
    }
}
