﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using Tizen.Content.MediaContent;

namespace TVMediaHub.Tizen.Models
{
    /// <summary>
    /// A custom EventArgs with string fields
    /// </summary>
    public class StorageInfoEventArgs : EventArgs
    {
        public string storageName;
    }

    /// <summary>
    /// A StorageProvider for the media hub
    /// </summary>
    public class StorageProvider
    {
        /// <summary>
        /// A list of external storage
        /// </summary>
        private List<Storage> storageList = new List<Storage>();

        /// <summary>
        /// A dictionary list of storage name and storage id
        /// Keeps the values when the media hub application is terminated.
        /// </summary>
        private Dictionary<string, string> storageTable = new Dictionary<string, string>();
        private static int index = 1;

        /// <summary>
        /// A dictionary list of storage name and storage id
        /// </summary>
        public Dictionary<string, string> StoragePairList { get; protected set; }

        /// <summary>
        /// An EventHandler that is occured when source is removed
        /// </summary>
        public EventHandler SourceRemovedEvent;

        /// <summary>
        /// A constructor
        /// </summary>
        public StorageProvider()
        {
            StoragePairList = new Dictionary<string, string>();
            CheckStorage();
        }

        /// <summary>
        /// Returns the list of the storage id of the media storage.
        /// </summary>
        public void CheckStorage()
        {
            var reader = MediaHubImpl.GetInstance.StorageCommand.Select();

            while (reader.Read())
            {
                storageList.Add(reader.Current);
            }

            if (storageList.Count() < storageTable.Count)
            {
                CheckRemoveStorage();
            }

            StoragePairList.Clear();
            foreach (var item in storageList)
            {
                if (!storageTable.ContainsKey(item.Id))
                {
                    storageTable.Add(item.Id, "USB " + index++);
                }

                StoragePairList.Add(item.Id, storageTable[item.Id]);
            }

            StoragePairList = StoragePairList.OrderBy(x => x.Value).ToDictionary(x => x.Key, x => x.Value);
        }

        /// <summary>
        /// Checks the removed storage name and invokes the SourceRemoveedEvent with storage name.
        /// </summary>
        private void CheckRemoveStorage()
        {
            foreach (var item in storageTable)
            {
                Storage storage = MediaHubImpl.GetInstance.StorageCommand.Select(item.Key);

                if (storage == null)
                {
                    StorageInfoEventArgs args = new StorageInfoEventArgs();
                    args.storageName = item.Value;

                    SourceRemovedEvent?.Invoke(this, args);
                }
            }
        }

        /// <summary>
        /// A method for handling SourceRemoveEvent
        /// </summary>
        /// <param name="listener">A handling method</param>
        public void SetSourceRemoveEventListener(EventHandler listener)
        {
            this.SourceRemovedEvent += listener;
        }
    }
}
