/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using ElmSharp;
using System;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.Views;
using Xamarin.Forms;

namespace TVMediaHub
{
    /// <summary>
    /// A class that represents TV MediaHub application
    /// </summary>
    public class App : Application
    {
        /// <summary>
        /// Gets or sets the window of TV MediaHub
        /// </summary>
        public static Window MainWindow { get; private set; }
        /// <summary>
        /// Gets or sets the NavigationPage of TV MediaHub
        /// </summary>
        public static NavigationPage AppMainPage { get; private set; }

        public static ElmSharp.EcoreEvent<ElmSharp.EcoreKeyEventArgs> KeyDownEvent;

        public static ElmSharp.EcoreEvent<ElmSharp.EcoreKeyEventArgs> KeyUpEvent;

        /// <summary>
        /// A constructor
        /// Sets main page to MainPage instance
        /// </summary>
        /// <see cref="MainPage"/>
        /// <param name="screenWidth">Screen Width</param>
        /// <param name="screenHeight">Screen Height</param>
        /// <param name="dpi">Screen DPI</param>
        /// <param name="scaleRatio">Scale ratio</param>
        /// <param name="window">Window</param>
        public App(int screenWidth, int screenHeight, int dpi, double scaleRatio, Window window)
        {
            SizeUtils.ScreenWidth = screenWidth;
            SizeUtils.ScreenHeight = screenHeight;
            SizeUtils.Dpi = dpi;
            SizeUtils.ScaleRatio = scaleRatio;

            try
            {
                string modelName;
                if (SystemSettingsPort.GetSystemModelName(out modelName))
                {
                    SizeUtils.SetModelName(modelName);
                }
            }
            catch (Exception e)
            {
                DbgPort.E("Cant get model name!!!, " + e.Message);
            }

            MainWindow = window;

            KeyDownEvent = new ElmSharp.EcoreEvent<ElmSharp.EcoreKeyEventArgs>(ElmSharp.EcoreEventType.KeyDown, (s, e, t) =>
            {
                return ElmSharp.EcoreKeyEventArgs.Create(s, e, t);
            });

            KeyUpEvent = new ElmSharp.EcoreEvent<ElmSharp.EcoreKeyEventArgs>(ElmSharp.EcoreEventType.KeyUp, (s, e, t) =>
            {
                return ElmSharp.EcoreKeyEventArgs.Create(s, e, t);
            });

            TransitionToMain();
        }

        /// <summary>
        /// A method for pushing a specific page to navigation stack.
        /// </summary>
        /// <param name="page">A page to be pushed</param>
        public void TransitionTo(ContentPage page)
        {
            //MainPage = page;
            AppMainPage.PushAsync(page);
        }

        /// <summary>
        /// A method for setting the root page as MainPage.
        /// </summary>
        /// <param name="tabIndex">An index of tab to be set as MainPage. Default value is 0</param>
        public void TransitionToMain(int tabIndex = 0)
        {
            /*
            MainPage = new NavigationPage(new MediaHubMainPage(tabIndex));
            MainPage.Opacity = 0;
            AppMainPage = MainPage as NavigationPage;
            */
            if (AppMainPage == null)
            {
                MainPage = new NavigationPage(new MediaHubMainPage(tabIndex));
                MainPage.Opacity = 0;
                AppMainPage = MainPage as NavigationPage;
            }
            else
            {
                AppMainPage.PopToRootAsync();
            }
        }

        /// <summary>
        /// A method for running hide animation
        /// </summary>
        public void RunHideAnimation()
        {
            MainPage.FadeTo(0.0, 667);
        }

        /// <summary>
        /// A method for running show animation
        /// </summary>
        public void RunShowAnimation()
        {
            MainPage.FadeTo(0.99, 667);
        }
    }
}
