﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;

namespace TVMediaHub.Tizen.Utils
{
    /// <summary>
    /// This class define custom easing function
    /// </summary>
    public class EasingFunction
    {
        public EasingFunction()
        {

        }
        /// <summary>
        /// CUBIC BEZIER P1=(0.33, 0.1), P2=(0.67, 1.0)
        /// </summary>
        public static Func<double, double> EasyEase = (x) =>
        {
            return CubicBezierCalc(CubicBezierTGet(x, 0.33, 0.67), 0.1, 1.0);
        };

        /// <summary>
        /// CUBIC BEZIER P1=(0.16, 0.02), P2=(0.0, 0.33)
        /// </summary>
        public static Func<double, double> EasyIn1 = (x) =>
        {
            return CubicBezierCalc(CubicBezierTGet(x, 0.16, 0.0), 0.02, 0.33);
        };

        /// <summary>
        /// CUBIC BEZIER P1=(0.33, 0.33), P2=(0.67, 0.16)
        /// </summary>
        public static Func<double, double> EasyIn2 = (x) =>
        {
            return CubicBezierCalc(CubicBezierTGet(x, 0.33, 0.67), 0.33, 0.16);
        };

        /// <summary>
        /// CUBIC BEZIER P1=(00.15, 0.0), P2=(0.30, 1.0)
        /// </summary>
        public static Func<double, double> GlideOut = (x) =>
        {
            return CubicBezierCalc(CubicBezierTGet(x, 0.15, 0.3), 0.0, 1.0);
        };

        /// <summary>
        /// Calculate Cubic Bezier curves
        /// </summary>
        /// <param name="a"></param>
        /// <param name="x1"></param>
        /// <param name="x2"></param>
        /// <returns></returns>
        private static double CubicBezierTGet(double a, double x1, double x2)
        {
            int limit = 100;
            double currentSlope;
            double change;
            double currentX;
            double guessT = a;

            for (int i = 0; i < limit; i++)
            {
                currentSlope = CubicBezierSlopeGet(guessT, x1, x2);
                if (currentSlope == 0.0)
                {
                    return guessT;
                }

                currentX = CubicBezierCalc(guessT, x1, x2) - a;
                change = currentX / currentSlope;
                guessT -= change;
                if ((change < 0.01) && (change > -0.01))
                {
                    break;
                }
            }

            return guessT;
        }

        /// <summary>
        /// Calculate Cubic Bezier curves
        /// </summary>
        /// <param name="t"></param>
        /// <param name="a1"></param>
        /// <param name="a2"></param>
        /// <returns></returns>
        private static double CubicBezierSlopeGet(double t, double a1, double a2)
        {
            return 3.0 * CubicBezierA(a1, a2) * t * t + 2.0 * CubicBezierB(a1, a2) * t + CubicBezierC(a1);
        }

        /// <summary>
        /// Calculate Cubic Bezier curves
        /// </summary>
        /// <param name="t"></param>
        /// <param name="a1"></param>
        /// <param name="a2"></param>
        /// <returns></returns>
        private static double CubicBezierCalc(double t, double a1, double a2)
        {
            return ((CubicBezierA(a1, a2) * t + CubicBezierB(a1, a2)) * t + CubicBezierC(a1)) * t;
        }

        /// <summary>
        /// Calculate Cubic Bezier curves
        /// </summary>
        /// <param name="a1"></param>
        /// <param name="a2"></param>
        /// <returns></returns>
        private static double CubicBezierA(double a1, double a2)
        {
            return 1.0 - 3.0 * a2 + 3.0 * a1;
        }

        /// <summary>
        /// Calculate Cubic Bezier curves
        /// </summary>
        /// <param name="a1"></param>
        /// <param name="a2"></param>
        /// <returns></returns>
        private static double CubicBezierB(double a1, double a2)
        {
            return 3.0 * a2 - 6.0 * a1;
        }

        /// <summary>
        /// Calculate Cubic Bezier curves
        /// </summary>
        /// <param name="a1"></param>
        /// <returns></returns>
        private static double CubicBezierC(double a1)
        {
            return 3.0 * a1;
        }
    }
}
