﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;

namespace TVMediaHub.Tizen.Utils
{
    /// <summary>
    /// A class provides Size metric related functions
    /// </summary>
    public class SizeUtils
    {
        /// <summary>
        /// Base Screen Height
        /// </summary>
        private static int baseScreenWidth = 1920;

        /// <summary>
        /// Base Screen Height
        /// </summary>
        public static int BaseScreenWidth
        {
            get
            {
                return baseScreenWidth;
            }
        }

        /// <summary>
        /// Base Screen Height
        /// </summary>
        private static int baseScreenHeight = 1080;

        /// <summary>
        /// Base Screen Height
        /// </summary>
        public static int BaseScreenHeight
        {
            get
            {
                return baseScreenHeight;
            }
        }


        /// <summary>
        /// Screen Height
        /// </summary>
        public static int ScreenHeight { set; get; }

        /// <summary>
        /// Screen Width
        /// </summary>
        public static int ScreenWidth { set; get; }

        /// <summary>
        /// Device DPI
        /// </summary>
        public static double Dpi { set; get; }

        /// <summary>
        /// Font Scale ratio
        /// </summary>
        public static double ScaleRatio { set; get; }

        /// <summary>
        /// Platform Model enumerations
        /// </summary>
        private enum PlatformModel
        {
            TV,
            Emulator,
            Other,
        }

        /// <summary>
        /// Platform Model Name
        /// </summary>
        private static PlatformModel ModelName = PlatformModel.TV;

        /// <summary>
        /// Set model name of running device.
        /// </summary>
        /// <param name="modelName">a model name</param>
        public static void SetModelName(string modelName)
        {
            if (modelName == null)
            {
                return;
            }

            DbgPort.D("ModelName is " + modelName);
            switch (modelName.ToLower()[0])
            {
                case 'e':
                    ModelName = PlatformModel.Emulator;
                    break;

                default:
                case 'x':
                    ModelName = PlatformModel.Other;
                    break;
            }
        }

        /// <summary>
        /// A method provides a converted height size.
        /// </summary>
        /// <param name="heightBaseSize">A height value in BaseScreen ratio</param>
        /// <returns>A date</returns>
        public static int GetHeightSize(int heightBaseSize)
        {
            return Convert.ToInt32((double)((double)heightBaseSize / (double)BaseScreenHeight) * (double)(ScreenHeight));
        }

        /// <summary>
        /// A method provides a converted width size.
        /// </summary>
        /// <param name="widthBaseSize">A width value in BaseScreen ratio</param>
        /// <returns>A date</returns>
        public static int GetWidthSize(int widthBaseSize)
        {
            return Convert.ToInt32((double)((double)widthBaseSize / (double)BaseScreenWidth) * (double)(ScreenWidth));
        }

        /// <summary>
        /// A method provides a converted font size.
        /// </summary>
        /// <param name="fontBaseSize">A base font size value in BaseScreen ratio</param>
        /// <returns>A date</returns>
        public static int GetFontSize(int fontBaseSize)
        {
            switch (ModelName)
            {
                case PlatformModel.Emulator:
                    //DbgPort.D("Emulator, Font size = " + fontBaseSize + " => " + ((double)((double)fontBaseSize / (double)BaseScreenHeight) * (double)ScreenHeight) * ScaleRatio);
                    return Convert.ToInt32(((double)((double)fontBaseSize / (double)BaseScreenHeight) * (double)ScreenHeight) * ScaleRatio);

                default:
                case PlatformModel.Other:
                case PlatformModel.TV:
                    // TODO : Remove this if the TV/Other device's dpi is correctly changed.
                    //DebuggingUtils.Dbg(String.Format("TV/Other, fontBaseSize = {0}, BaseScreenHeight = {1}, ScreenHeight = {2}, ScaleRatio = {3}", fontBaseSize, BaseScreenHeight, ScreenHeight, ScaleRatio));
                    double tempAdjustmentRatio = 1D;
                    return Convert.ToInt32(((double)((double)fontBaseSize / (double)BaseScreenHeight) * (double)ScreenHeight) * ScaleRatio * tempAdjustmentRatio);
            }
        }
    }
}
