﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows.Input;
using Xamarin.Forms;
using System.Linq;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.DataModels;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using System.Threading;
using TVMediaHub.Tizen.Utils;
using System.IO;
using System;

namespace TVMediaHub.Tizen.ViewModels
{
    /// <summary>
    /// A class for ViewModel of Image tab
    /// </summary>
    public class ImageTabViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Gets or sets list of image GroupItem
        /// </summary>
        public ObservableCollection<GroupItem> ImageList { get; set; }

        /// <summary>
        /// A dictionary list of the source and storage id
        /// </summary>
        private Dictionary<string, string> sourcePairList;

        /// <summary>
        /// An index of displaying image
        /// </summary>
        private int displayingImageIndex;

        /// <summary>
        /// A sort option
        /// </summary>
        private SortOption option = SortOption.Date;

        /// <summary>
        /// A storage id
        /// </summary>
        private string storageId = null;

        /// <summary>
        /// Gets or sets the displayingImageIndex
        /// </summary>
        private int DisplayingImageIndex
        {
            get
            {
                return displayingImageIndex;
            }

            set
            {
                if (displayingImageIndex < 0 ||
                    displayingImageIndex > imagePlayList.Count())
                {
                    return;
                }

                displayingImageIndex = value;
                OnPropertyChanged("CurrentImage");
            }
        }

        /// <summary>
        /// Gets the ImageViewerInformation of displaying image
        /// </summary>
        public ImageViewerInfomation CurrentImage
        {
            get
            {
                return imagePlayList[DisplayingImageIndex];
            }
        }

        /// <summary>
        /// A list of ImageViewerInformation
        /// </summary>
        private List<ImageViewerInfomation> imagePlayList;

        /// <summary>
        /// Gets the ImageViewerInformation list
        /// </summary>
        public IEnumerable<ImageViewerInfomation> ImagePlayList
        {
            get
            {
                return imagePlayList;
            }
        }

        /// <summary>
        /// Gets or sets SlideShowSpeed enumeration value
        /// </summary>
        public SlideShowSpeed Speed { get; set; }

        /// <summary>
        /// Gets or sets SlideShowEffect enumeration value
        /// </summary>
        public SlideShowEffect Effect { get; set; }

        /// <summary>
        /// Gets or sets the SourceList
        /// </summary>
        public IEnumerable<string> SourceList { get; set; }

        /// <summary>
        /// Gets or sets the SortOption list
        /// </summary>
        public IEnumerable<string> SortOptions { get; set; }

        /// <summary>
        /// Gets or sets the Option list
        /// </summary>
        public IEnumerable<string> OptionList { get; set; }

        /// <summary>
        /// A command for executing when SortOption is changed
        /// </summary>
        public ICommand ChangeSortOptionCommand { get; set; }

        /// <summary>
        /// A command for changing Source
        /// </summary>
        public ICommand ChangeSourceCommand { get; set; }

        /// <summary>
        /// A command for deleting a image content
        /// </summary>
        public ICommand DeleteContentCommand { get; set; }

        /// <summary>
        /// A command for selecting all video contents
        /// </summary>
        public ICommand SelectAllContentCommand { get; set; }

        /// <summary>
        /// An enumeration value indicates current image tab's status
        /// </summary>
        private TabStatus imageTabStatus = TabStatus.Default;

        /// <summary>
        /// Gets or sets the ImageTabStatus
        /// </summary>
        public TabStatus ImageTabStatus
        {
            get
            {
                return imageTabStatus;
            }

            set
            {
                if (imageTabStatus != value)
                {
                    imageTabStatus = value;
                    OnPropertyChanged("ImageTabStatus");
                }
            }
        }

        /// <summary>
        /// A flag that whether the BottomButtonList of Tab is needed to delete or not
        /// </summary>
        private bool isNeededClearList;

        /// <summary>
        /// Gets or sets the isNeededClearList
        /// </summary>
        public bool IsNeededClearList
        {
            get
            {
                return isNeededClearList;
            }

            set
            {
                if (isNeededClearList != value)
                {
                    isNeededClearList = value;
                    OnPropertyChanged("IsNeededClearList");
                }
            }
        }

        /// <summary>
        /// A command for executing when CurrentImageInfo is changed
        /// </summary>
        public ICommand SetCurrentImageInfo
        {
            protected set;
            get;
        }

        /// <summary>
        /// A command for executing when Navigation button is pressed
        /// </summary>
        public ICommand NavigationCommand
        {
            protected set;
            get;
        }

        /// <summary>
        /// A command for executing when SlideShowSpeed value is changed
        /// </summary>
        public ICommand SetSlideShowSpeedCommand
        {
            protected set;
            get;
        }

        /// <summary>
        /// A command for executing when SlideShowEffect value is changed
        /// </summary>
        public ICommand SetSlideShowEffectCommand
        {
            protected set;
            get;
        }

        /// <summary>
        /// A command for executing to save rotation value
        /// </summary>
        public ICommand SavedRotationCommand { get; set; }

        /// <summary>
        /// A command for executing when DeleteMode is changed
        /// </summary>
        public ICommand DeleteModeChangeCommand { get; set; }

        /// <summary>
        /// A command for executing to get informations
        /// </summary>
        public ICommand GetInformationsCommand { get; set; }

        /// <summary>
        /// Gets or sets the SelectedList
        /// </summary>
        public List<MediaInformationEx> SelectedList { get; set; }

        /// <summary>
        /// Gets the SelectedList's count
        /// </summary>
        public int SelectedCount
        {
            get
            {
                return SelectedList.Count;
            }
        }

        /// <summary>
        /// An event that is occurred when property of ViewModel is changed
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// A method for invoking PropertyChanged event
        /// </summary>
        /// <param name="propertyName">The name of property</param>
        protected void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        /// <summary>
        /// A constructor
        /// Initializes the footer's item source
        /// Initializes the commands that are used in Image tab
        /// Initializes several lists that are used in Image tab
        /// </summary>
        public ImageTabViewModel()
        {
            ImageList = new ObservableCollection<GroupItem>();
            SelectedList = new List<MediaInformationEx>();
            sourcePairList = new Dictionary<string, string>();
            InitializeFooterItemsSource();
            InitializeCommands();
            CreateImageInfoList();
            OnPropertyChanged("ImageList");
            MediaHubImpl.GetInstance.ImageProviderInstance.SetContentUpdatedEventListener((sender, arg) =>
            {
                AddSourceListItem();
            });
        }

        /// <summary>
        /// A method for initializing commands that are used in Image tab
        /// </summary>
        private void InitializeCommands()
        {
            DeleteModeChangeCommand = new Command(() =>
            {
                SelectedList.Clear();
                OnPropertyChanged("SelectedCount");

                if (ImageTabStatus == TabStatus.Default)
                {
                    ImageTabStatus = TabStatus.Delete;
                }
                else
                {
                    ImageTabStatus = TabStatus.Default;
                }

                foreach (var group in ImageList)
                {
                    foreach (var info in group.Contents)
                    {
                        info.CurrentTabStatus = ImageTabStatus;
                    }
                }
            });

            NavigationCommand = new Command<string>((direction) =>
            {
                if (direction.Equals("Left"))
                {
                    if (DisplayingImageIndex > 0)
                    {
                        DisplayingImageIndex -= 1;
                    }
                }
                else if (direction.Equals("Right"))
                {
                    if (DisplayingImageIndex < imagePlayList.Count - 1)
                    {
                        DisplayingImageIndex += 1;
                    }
                }
            });

            SetSlideShowSpeedCommand = new Command<SlideShowSpeed>((speed) =>
            {
                Speed = speed;
                OnPropertyChanged("Speed");
            });

            SetSlideShowEffectCommand = new Command<SlideShowEffect>((effect) =>
            {
                Effect = effect;
                OnPropertyChanged("Effect");
            });

            SetCurrentImageInfo = new Command<MediaInformationEx>((info) =>
            {
                if (ImageTabStatus == TabStatus.Delete)
                {
                    if (SelectedList.Contains(info))
                    {
                        SelectedList.Remove(info);
                    }
                    else
                    {
                        SelectedList.Add(info);
                    }

                    OnPropertyChanged("SelectedCount");
                }
                else
                {
                    SetCurrentImage(info);
                }
            });

            ChangeSourceCommand = new Command<string>((name) =>
            {
                if (name == null)
                {
                    return;
                }

                storageId = sourcePairList[name];
                SynchronizationContext.Current.Post((o) =>
                {
                    ReadImageList(option, storageId);
                }, "");
            });

            SelectAllContentCommand = new Command<bool>((IsSelectedAll) =>
            {
                if (IsSelectedAll)
                {
                    foreach (var group in ImageList)
                    {
                        foreach (var info in group.Contents)
                        {
                            if (!SelectedList.Contains(info.Information))
                            {
                                SelectedList.Add(info.Information);
                            }
                        }
                    }
                }
                else
                {
                    SelectedList.Clear();
                }

                OnPropertyChanged("SelectedCount");
            });

            SavedRotationCommand = new Command<double>((rotation) =>
            {
                if (rotation == -1D)
                {
                    /// Initialize the saved rotation
                    foreach (var item in imagePlayList)
                    {
                        item.SavedRotation = 0;
                    }
                }
                else
                {
                    CurrentImage.SavedRotation = rotation;
                }
            });

            ChangeSortOptionCommand = new Command<string>((opt) =>
            {
                var optionString = opt.ToLower();
                switch (optionString)
                {
                    case "name":
                        ReadImageList(SortOption.Title);
                        option = SortOption.Title;
                        break;
                    case "event":
                        ReadImageList(SortOption.Date);
                        option = SortOption.Date;
                        break;
                    case "file format":
                        ReadImageList(SortOption.Type);
                        option = SortOption.Type;
                        break;
                }
            });

            GetInformationsCommand = new Command(() =>
            {
                SynchronizationContext.Current.Post((o) =>
                {
                    ReadImageList(option, storageId);
                }, "");
            });

            DeleteContentCommand = new Command<MediaInformationEx>((SelectedItem) =>
            {
                if (SelectedItem != null)
                {
                    SelectedList.Clear();
                    SelectedList.Add(SelectedItem);
                }

                foreach (var info in SelectedList)
                {
                    try
                    {
                        File.Delete(info.MediaContentInformation.Path);
                    }
                    catch (Exception exception)
                    {
                        // TODO: Handling exceptions
                        DbgPort.E("Exception - " + exception.Message);
                        return;
                    }

                    MediaHubImpl.GetInstance.ImageProviderInstance.Delete(info);
                }
            });

            OnPropertyChanged("DeleteModeChangeCommand");
            OnPropertyChanged("ChangeSortOptionCommand");
        }

        /// <summary>
        /// A method for making the source list
        /// </summary>
        private void AddSourceListItem()
        {
            sourcePairList.Clear();
            List<string> tempList = new List<string>();

            sourcePairList.Add("ALL SOURCE", null);
            sourcePairList.Add("DEVICE", "media");

            tempList.Add("ALL SOURCE");
            tempList.Add("DEVICE");

            foreach (var item in MediaHubImpl.GetInstance.StorageProviderInstance.StoragePairList)
            {
                sourcePairList.Add(item.Value, item.Key);
                tempList.Add(item.Value);
            }

            SourceList = tempList;
            OnPropertyChanged("SourceList");
        }

        /// <summary>
        /// Finds specific Image in list and sets the DisplayingImageIndex
        /// </summary>
        /// <param name="info">A MediaInformationEx to be found</param>
        private void FindImageInfoInList(MediaInformationEx info)
        {
            for (int i = 0; i < imagePlayList.Count; i++)
            {
                if (imagePlayList[i].FilePath.Equals(info.MediaContentInformation.Path))
                {
                    DisplayingImageIndex = i;
                    return;
                }
            }
        }

        /// <summary>
        /// A method for setting the current image
        /// </summary>
        /// <param name="info">An information of current image</param>
        private void SetCurrentImage(MediaInformationEx info)
        {
            FindImageInfoInList(info);
        }

        /// <summary>
        /// A method for creating ImageViewrInformation with specific MediaInformationEx
        /// </summary>
        /// <param name="info">A specific MediaInformationEx</param>
        /// <returns>A created ImageViewerInformation</returns>
        private ImageViewerInfomation CreateImageViewerInfomation(MediaInformationEx info)
        {
            ImageViewerInfomation ivInfo = new ImageViewerInfomation();

            ivInfo.Title = info.MediaContentInformation.Title;
            ivInfo.Date = info.MediaContentInformation.DateModified.ToString("ddd. d MMMM");
            ivInfo.FilePath = info.MediaContentInformation.Path;
            ivInfo.ThumbnailPath = info.MediaContentInformation.ThumbnailPath;
            ivInfo.SavedRotation = 0;

            return ivInfo;
        }

        /// <summary>
        /// A method for reading image contents through ImageProvider and updating ImageList
        /// </summary>
        /// <param name="option">A current sort option</param>
        /// <param name="storageID">The given storage ID</param>
        private async void ReadImageList(SortOption option, string storageID = null)
        {
            ImageList.Clear();

            IEnumerable<MediaInformationEx> list = await MediaHubImpl.GetInstance.ImageProviderInstance.ReadWithoutGroupAsync(SortOption.Title);
            await MediaHubImpl.GetInstance.ImageProviderInstance.CheckThumbnail(list);

            IEnumerable<GroupItem> tempList = await MediaHubImpl.GetInstance.ImageProviderInstance.ReadAsync(option, storageID);

            IsNeededClearList = true;

            foreach (var group in tempList)
            {
                // TODO : remove delay
                await Task.Delay(1);
                ImageList.Add(group);
            }

            IsNeededClearList = false;
        }

        /// <summary>
        /// A method for creating list of ImageViewerInformation
        /// </summary>
        private void CreateImageInfoList()
        {
            IEnumerable<MediaInformationEx> MediaInfoList = MediaHubImpl.GetInstance.ImageProviderInstance.ReadWithoutGroup(option);
            imagePlayList = new List<ImageViewerInfomation>();

            foreach (var item in MediaInfoList)
            {
                imagePlayList.Add(CreateImageViewerInfomation(item));
            }

            OnPropertyChanged("ImagePlayList");
        }

        /// <summary>
        /// A method for initializing the SourceList, SortOptionList and OptionList
        /// </summary>
        private void InitializeFooterItemsSource()
        {
            AddSourceListItem();

            var list = new List<string>
            {
                "EVENT",
                "FILE FORMAT",
                "NAME",
            };
            SortOptions = list;
            OnPropertyChanged("SortOptions");

            list = new List<string>
            {
                "          SLIDE SHOW          ",
                "DISK INFO",
                "DELETE"
            };
            OptionList = list;
            OnPropertyChanged("OptionList");
        }
    }

}
