﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System.ComponentModel;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.Utils;
using Xamarin.Forms;

namespace TVMediaHub.Tizen.ViewModels
{
    /// <summary>
    /// A class for ViewModel of the music player
    /// </summary>
    public class MusicPlayerViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// The percentage of the progressbar to be shown
        /// </summary>
        public double ProgressBarPercentage { get; private set; }

        /// <summary>
        /// The album title to be shown
        /// </summary>
        public string AlbumTitle { get; private set; }

        /// <summary>
        /// The artist to be shown
        /// </summary>
        public string AlbumArtist { get; private set; }

        /// <summary>
        /// The album name to be shown
        /// </summary>
        public string AlbumName { get; private set; }

        /// <summary>
        /// The album cover to be shown
        /// </summary>
        public string AlbumCover { get; private set; }

        public Command MusicPlayerButtonClickCommand { get; private set; }

        public string IconPath { get; private set; }

        /// <summary>
        /// An event that is occurred when property of ViewModel is changed
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// A method for invoking PropertyChanged event
        /// </summary>
        /// <param name="name">The name of property</param>
        public void OnPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public MusicPlayerViewModel()
        {
            ProgressBarPercentage = 0.0;
            AlbumTitle = "Title";
            AlbumArtist = "Artist";
            AlbumCover = "img_media_no_contents.png";
            AlbumName = "Album";

            MusicPlayerButtonClickCommand = new Command(() =>
            {
                MusicPlayerModel.Instance.TogglePausePlay();
            });

            IconPath = "btn_music_info_play.png";

            MusicPlayerModel.Instance.SetCurrentMusicInfoListener((e, args) =>
            {
                if (args == null)
                {
                    return;
                }

                AlbumTitle = args.Title;
                OnPropertyChanged("AlbumTitle");

                AlbumArtist = args.Artist;
                OnPropertyChanged("AlbumArtist");

                AlbumCover = args.AlbumCover;
                OnPropertyChanged("AlbumCover");

                AlbumName = args.AlbumName;
                OnPropertyChanged("AlbumName");
            });

            MusicPlayerModel.Instance.SetCurrentMusicProgressListener((e, arg) =>
            {
                if (arg == null)
                {
                    return;
                }

                ProgressBarPercentage = arg.Progress;
                OnPropertyChanged("ProgressBarPercentage");
            });

            MusicPlayerModel.Instance.SetMusicPlayerStateChangedListener((e, arg) => {
               if(arg == null)
                {
                    return;
                }

                if (arg.IsPlaying)
                {
                    IconPath = "btn_music_info_pause.png";
                }
                else
                {
                    IconPath = "btn_music_info_play.png";
                }
                OnPropertyChanged("IconPath");
            });
        }
    }
}
