﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.Utils;
using Xamarin.Forms;

namespace TVMediaHub.Tizen.ViewModels
{
    /// <summary>
    /// A class for ViewModel of Music tab
    /// </summary>
    public class MusicTabViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Gets or sets the list of music group item
        /// </summary>
        public ObservableCollection<GroupItem> MusicList { get; set; }

        /// <summary>
        /// Gets or sets the list of music items
        /// </summary>
        public List<MediaInformationEx> MusicPlayList { get; set; }

        /// <summary>
        /// Gets or sets the SourceList
        /// </summary>
        public IEnumerable<string> SourceList { get; set; }

        /// <summary>
        /// Gets or sets the SortOption list
        /// </summary>
        public IEnumerable<string> SortOptions { get; set; }

        /// <summary>
        /// Gets or sets the Option list
        /// </summary>
        public IEnumerable<string> OptionList { get; set; }

        /// <summary>
        /// A dictionary list of the source and storage id
        /// </summary>
        private Dictionary<string, string> sourcePairList;

        /// <summary>
        /// A command for executing when SortOption is changed
        /// </summary>
        public ICommand ChangeSortOptionCommand { get; set; }

        /// <summary>
        /// A command for executing to get informations
        /// </summary>
        public ICommand GetInformationsCommand { get; set; }

        /// <summary>
        /// A command for deleting a video content
        /// </summary>
        public ICommand DeleteContentCommand { get; set; }

        /// <summary>
        /// A command for selecting all video contents
        /// </summary>
        public ICommand SelectAllContentCommand { get; set; }

        /// <summary>
        /// A command to set current music information
        /// </summary>
        public ICommand SelectMusicItem { get; set; }

        /// <summary>
        /// A command for changing Source
        /// </summary>
        public ICommand ChangeSourceCommand { get; set; }

        /// <summary>
        /// A command for changing tab's status
        /// </summary>
        public ICommand ChangeTabStatusCommand { get; set; }

        /// <summary>
        /// The information of the current music
        /// </summary>
        public MediaInformationEx CurrentMusic { get; set; }

        /// <summary>
        /// A value indicates current Music tab's status
        /// </summary>
        private TabStatus musicTabStatus = TabStatus.Default;

        /// <summary>
        /// Gets or sets the MusicTabStatus
        /// </summary>
        public TabStatus MusicTabStatus
        {
            get
            {
                return musicTabStatus;
            }

            set
            {
                if (musicTabStatus != value)
                {
                    musicTabStatus = value;
                    OnPropertyChanged("MusicTabStatus");
                }
            }
        }

        /// <summary>
        /// A list of video contents to be displayed
        /// </summary>
        public List<MediaInformationEx> SelectedList { get; set; }

        /// <summary>
        /// A count of SelectedList
        /// </summary>
        public int SelectedCount
        {
            get
            {
                return SelectedList.Count;
            }
        }

        /// <summary>
        /// The index of the current music
        /// </summary>
        public int CurrentMusicIndex { get; set; }

        /// <summary>
        /// The sort option to display items of the music tab
        /// </summary>
        private SortOption option = SortOption.Title;

        /// <summary>
        /// A storage id
        /// </summary>
        private string storageId = null;

        /// <summary>
        /// An event that is occurred when property of ViewModel is changed
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// A method for invoking PropertyChanged event
        /// </summary>
        /// <param name="name">The name of property</param>
        public void OnPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public MusicTabViewModel()
        {
            MusicList = new ObservableCollection<GroupItem>();
            MusicPlayList = new List<MediaInformationEx>();

            sourcePairList = new Dictionary<string, string>();
            SelectedList = new List<MediaInformationEx>();
            InitializeFooterItemsSource();
            InitializeCommands();

            OnPropertyChanged("MusicList");
            MediaHubImpl.GetInstance.MusicProviderInstance.SetContentUpdatedEventListener((sender, arg) =>
            {
                AddSourceListItem();
            });

            MusicPlayerModel.Instance.SetMusicPlayCompletedListener((e, v) =>
            {
                if(CurrentMusicIndex < MusicPlayList.Count - 1)
                {
                    CurrentMusic = MusicPlayerModel.Instance.CurrentMusic = MusicPlayList[++CurrentMusicIndex];
                    OnPropertyChanged("CurrentMusic");
                }
            });
        }

        /// <summary>
        /// A method for reading music contents through MusicProvider and updating MusicList
        /// </summary>
        /// <param name="option">A current sort option</param>
        private async void ReadMusicList(SortOption option, string storageID = null)
        {
            MusicList.Clear();
            IEnumerable<MediaInformationEx> list = await MediaHubImpl.GetInstance.MusicProviderInstance.ReadWithoutGroupAsync(SortOption.Title);
            await MediaHubImpl.GetInstance.MusicProviderInstance.CheckThumbnail(list);

            IEnumerable<GroupItem> templist = await MediaHubImpl.GetInstance.MusicProviderInstance.ReadAsync(option, storageID);

            foreach (var group in templist)
            {
                await Task.Delay(1);
                MusicList.Add(group);
                foreach(var item in group.Contents)
                {
                    MusicPlayList.Add(item.Information);
                }
            }
        }

        /// <summary>
        /// A method for initializing the SourceList, SortOptionList and OptionList
        /// </summary>
        private void InitializeFooterItemsSource()
        {
            AddSourceListItem();

            var list = new List<string>
            {
                "SONG",
                "ALBUM",
                "ARTIST",
                "GENRE",
            };

            SortOptions = list;
            OnPropertyChanged("SortOptions");

            list = new List<string>
            {
                "        DISK INFO        ",
                "DELETE"
            };

            OptionList = list;
            OnPropertyChanged("OptionList");
        }

        /// <summary>
        /// A method for making the source list
        /// </summary>
        private void AddSourceListItem()
        {
            sourcePairList.Clear();
            List<string> tempList = new List<string>();

            sourcePairList.Add("ALL SOURCE", null);
            sourcePairList.Add("DEVICE", "media");

            tempList.Add("ALL SOURCE");
            tempList.Add("DEVICE");

            foreach (var item in MediaHubImpl.GetInstance.StorageProviderInstance.StoragePairList)
            {
                sourcePairList.Add(item.Value, item.Key);
                tempList.Add(item.Value);
            }

            SourceList = tempList;
            OnPropertyChanged("SourceList");
        }

        /// <summary>
        /// A method for initializing commands that are used in Music tab
        /// </summary>
        private void InitializeCommands()
        {
            ChangeTabStatusCommand = new Command(() =>
            {
                SelectedList.Clear();
                OnPropertyChanged("SelectedCount");

                if (MusicTabStatus.Equals(TabStatus.Default))
                {
                    MusicTabStatus = TabStatus.Delete;
                }
                else
                {
                    MusicTabStatus = TabStatus.Default;
                }
            });

            SelectAllContentCommand = new Command<bool>((IsSelectedAll) =>
            {
                if (IsSelectedAll)
                {
                    foreach (var group in MusicList)
                    {
                        foreach (var info in group.Contents)
                        {
                            if (!SelectedList.Contains(info.Information))
                            {
                                SelectedList.Add(info.Information);
                            }
                        }
                    }
                }
                else
                {
                    SelectedList.Clear();
                }

                OnPropertyChanged("SelectedCount");
            });

            DeleteContentCommand = new Command<MediaInformationEx>((SelectedItem) =>
            {
                if (SelectedItem != null)
                {
                    SelectedList.Clear();
                    SelectedList.Add(SelectedItem);
                }

                foreach (var info in SelectedList)
                {
                    try
                    {
                        File.Delete(info.MediaContentInformation.Path);
                    }
                    catch (Exception exception)
                    {
                        // TODO: Handling exceptions
                        DbgPort.E("Exception - " + exception.Message);
                        return;
                    }

                    MediaHubImpl.GetInstance.MusicProviderInstance.Delete(info);
                }
            });

            SelectMusicItem = new Command<MediaInformationEx>((info) =>
            {
                if (MusicTabStatus == TabStatus.Delete)
                {
                    if (SelectedList.Contains(info))
                    {
                        SelectedList.Remove(info);
                    }
                    else
                    {
                        SelectedList.Add(info);
                    }

                    OnPropertyChanged("SelectedCount");
                }
                else
                {
                    SetCurrentMusic(info);
                }
            });

            ChangeSortOptionCommand = new Command<string>((opt) =>
            {
                var optionString = opt.ToLower();
                switch (optionString)
                {
                    case "song":
                        ReadMusicList(SortOption.Title);
                        option = SortOption.Title;
                        break;
                    case "album":
                        ReadMusicList(SortOption.Album);
                        option = SortOption.Album;
                        break;
                    case "artist":
                        ReadMusicList(SortOption.Artist);
                        option = SortOption.Artist;
                        break;
                    case "genre":
                        ReadMusicList(SortOption.Genre);
                        option = SortOption.Genre;
                        break;
                }
            });

            GetInformationsCommand = new Command(() =>
            {
                SynchronizationContext.Current.Post((o) =>
                {
                    ReadMusicList(option);
                }, "");
            });

            ChangeSourceCommand = new Command<string>((name) =>
            {
                if (name == null)
                {
                    return;
                }

                storageId = sourcePairList[name];
                SynchronizationContext.Current.Post((o) =>
                {
                    ReadMusicList(option, storageId);
                }, "");
            });

            OnPropertyChanged("ChangeTabStatusCommand");
            OnPropertyChanged("ChangeSortOptionCommand");
        }

        /// <summary>
        /// Sets the current music
        /// </summary>
        /// <param name="musicContent">The given content to be set</param>
        public void SetCurrentMusic(MediaInformationEx musicContent)
        {
            CurrentMusic = MusicPlayerModel.Instance.CurrentMusic = musicContent;
            CurrentMusicIndex = MusicPlayList.IndexOf(musicContent);
            OnPropertyChanged("CurrentMusic");
        }
    }
}
