﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Threading;
using System.Windows.Input;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.DataModels;
using Xamarin.Forms;
using Tizen.Xamarin.Forms.Extension;
using System.Threading.Tasks;

namespace TVMediaHub.Tizen.ViewModels
{
    /// <summary>
    /// A class for ViewModel of Video tab
    /// </summary>
    public class VideoTabViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// An event that is occurred when property of ViewModel is changed
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        private Dictionary<string, string> sourcePairList;

        /// <summary>
        /// A method for invoking PropertyChanged event
        /// </summary>
        /// <param name="name">The name of property</param>
        public void OnPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        /// <summary>
        /// Gets or sets the list of video group item
        /// </summary>
        public ObservableCollection<GroupItem> VideoList { get; set; }

        /// <summary>
        /// Gets or sets the SourceList
        /// </summary>
        public IEnumerable<string> SourceList { get; set; }

        /// <summary>
        /// Gets or sets the sort option list
        /// </summary>
        public IEnumerable<string> SortOptions { get; set; }

        /// <summary>
        /// Gets or sets the option list
        /// </summary>
        public IEnumerable<string> OptionList { get; set; }

        /// <summary>
        /// A command for changing tab's status
        /// </summary>
        public ICommand ChangeTabStatusCommand { get; set; }

        /// <summary>
        /// A command for changing sort option
        /// </summary>
        public ICommand ChangeSortOptionCommand { get; set; }

        /// <summary>
        /// A command for changing Source
        /// </summary>
        public ICommand ChangeSourceCommand { get; set; }

        /// <summary>
        /// A command for getting the list of video contents
        /// </summary>
        public ICommand GetInformationsCommand { get; set; }

        /// <summary>
        /// A command for deleting a video content
        /// </summary>
        public ICommand DeleteContentCommand { get; set; }

        /// <summary>
        /// A command for selecting all video contents
        /// </summary>
        public ICommand SelectAllContentCommand { get; set; }

        /// <summary>
        /// A command for changing current video to next
        /// </summary>
        public ICommand SetNextVideoCommand { get; set; }

        /// <summary>
        /// A command for changing current video to previous
        /// </summary>
        public ICommand SetPrevVideoCommand { get; set; }

        /// <summary>
        /// A command to set current video information
        /// </summary>
        public ICommand SetCurrentVideoInfo
        {
            protected set;
            get;
        }

        /// <summary>
        /// A sort option
        /// </summary>
        private SortOption option = SortOption.Title;

        /// <summary>
        /// A storage id
        /// </summary>
        private string storageId = null;

        /// <summary>
        /// Gets or sets the MediaInformationEx of current video
        /// </summary>
        public MediaInformationEx CurrentVideo { get; set; }
        public int CurrentVideoIndex { get; set; }
        public List<MediaInformationEx> PlayList { get; set; }

        public bool IsLastVideo
        {
            get
            {
                return (PlayList.Count - 1) == CurrentVideoIndex;
            }
        }

        /// <summary>
        /// An enumeration value indicates current video tab's status
        /// </summary>
        private TabStatus videoTabStatus = TabStatus.Default;

        /// <summary>
        /// Gets or sets the VideoTabStatus
        /// </summary>
        public TabStatus VideoTabStatus
        {
            get
            {
                return videoTabStatus;
            }

            set
            {
                if (videoTabStatus != value)
                {
                    videoTabStatus = value;
                    OnPropertyChanged("VideoTabStatus");
                }
            }
        }

        /// <summary>
        /// A flag that whether the BottomButtonList of Tab is needed to delete or not
        /// </summary>
        private bool isNeededClearList;

        /// <summary>
        /// Gets or sets the isNeededClearList
        /// </summary>
        public bool IsNeededClearList
        {
            get
            {
                return isNeededClearList;
            }

            set
            {
                if (isNeededClearList != value)
                {
                    isNeededClearList = value;
                    OnPropertyChanged("IsNeededClearList");
                }
            }
        }

        /// <summary>
        /// A list of video contents to be displayed
        /// </summary>
        public List<MediaInformationEx> SelectedList { get; set; }

        /// <summary>
        /// A count of SelectedList
        /// </summary>
        public int SelectedCount
        {
            get
            {
                return SelectedList.Count;
            }
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public VideoTabViewModel()
        {
            VideoList = new ObservableCollection<GroupItem>();
            SelectedList = new List<MediaInformationEx>();
            PlayList = new List<MediaInformationEx>();
            sourcePairList = new Dictionary<string, string>();
            InitializeFooterItemsSource();
            InitializeCommands();
            OnPropertyChanged("VideoList");
            MediaHubImpl.GetInstance.VideoProviderInstance.SetContentUpdatedEventListener((sender, arg) =>
            {
                AddSourceListItem();
            });
        }

        /// <summary>
        /// A method for initializing commands that are used in Video tab
        /// </summary>
        private void InitializeCommands()
        {
            ChangeTabStatusCommand = new Command(() =>
            {
                SelectedList.Clear();
                OnPropertyChanged("SelectedCount");
                if (VideoTabStatus == TabStatus.Default)
                {
                    VideoTabStatus = TabStatus.Delete;
                }
                else
                {
                    VideoTabStatus = TabStatus.Default;
                }

                foreach (var group in VideoList)
                {
                    foreach (var info in group.Contents)
                    {
                        info.CurrentTabStatus = VideoTabStatus;
                    }
                }
            });
            DeleteContentCommand = new Command<MediaInformationEx>((SelectedItem) =>
            {
                if (SelectedItem != null)
                {
                    SelectedList.Clear();
                    SelectedList.Add(SelectedItem);
                }

                foreach (var info in SelectedList)
                {
                    try
                    {
                        File.Delete(info.MediaContentInformation.Path);
                    }
                    catch (Exception exception)
                    {
                        // TODO: Handling exceptions
                        DbgPort.E("Exception - " + exception.Message);
                        return;
                    }

                    MediaHubImpl.GetInstance.VideoProviderInstance.Delete(info);
                }
            });
            ChangeSortOptionCommand = new Command<string>((opt) =>
            {
                var optionString = opt.ToLower();
                switch (optionString)
                {
                    case "date":
                        ReadVideoList(SortOption.Date);
                        option = SortOption.Date;
                        break;
                    case "genre":
                        ReadVideoList(SortOption.Genre);
                        option = SortOption.Genre;
                        break;
                    case "name":
                        ReadVideoList(SortOption.Title);
                        option = SortOption.Title;
                        break;
                }
            });
            GetInformationsCommand = new Command(() =>
            {
                SynchronizationContext.Current.Post((o) =>
                {
                    ReadVideoList(option, storageId);
                }, "");
            });
            ChangeSourceCommand = new Command<string>((name) =>
            {
                if (name == null)
                {
                    return;
                }

                storageId = sourcePairList[name];
                SynchronizationContext.Current.Post((o) =>
                {
                    ReadVideoList(option, storageId);
                }, "");
            });
            SetCurrentVideoInfo = new Command<MediaInformationEx>((info) =>
            {
                if (VideoTabStatus == TabStatus.Delete)
                {
                    if (SelectedList.Contains(info))
                    {
                        SelectedList.Remove(info);
                    }
                    else
                    {
                        SelectedList.Add(info);
                    }

                    OnPropertyChanged("SelectedCount");
                }
                else
                {
                    SetCurrentVideo(info);
                }
            });

            SelectAllContentCommand = new Command<bool>((IsSelectedAll) =>
            {
                if (IsSelectedAll)
                {
                    foreach (var group in VideoList)
                    {
                        foreach (var info in group.Contents)
                        {
                            if (!SelectedList.Contains(info.Information))
                            {
                                SelectedList.Add(info.Information);
                            }
                        }
                    }
                }
                else
                {
                    SelectedList.Clear();
                }

                OnPropertyChanged("SelectedCount");
            });

            SetNextVideoCommand = new Command(() =>
            {
                SetNextVideo();
            });

            SetPrevVideoCommand = new Command(() =>
            {
                SetPrevVideo();
            });
            OnPropertyChanged("SetNextVideoCommand");
            OnPropertyChanged("SetPrevVideoCommand");

            OnPropertyChanged("ChangeTabStatusCommand");
            OnPropertyChanged("ChangeSortOptionCommand");
        }

        /// <summary>
        /// A method for initializing the SourceList, SortOptionList and OptionList
        /// </summary>
        private void InitializeFooterItemsSource()
        {
            AddSourceListItem();

            var list = new List<string>
            {
                "NAME",
                "GENRE",
                "DATE"
            };
            SortOptions = list;
            OnPropertyChanged("SortOptions");

            list = new List<string>
            {
                "            DISK INFO            ",
                "DELETE"
            };
            OptionList = list;
            OnPropertyChanged("OptionList");
        }

        /// <summary>
        /// A method for making the source list
        /// </summary>
        private void AddSourceListItem()
        {
            sourcePairList.Clear();
            List<string> tempList = new List<string>();

            sourcePairList.Add("ALL SOURCE", null);
            sourcePairList.Add("DEVICE", "media");

            tempList.Add("ALL SOURCE");
            tempList.Add("DEVICE");

            foreach (var item in MediaHubImpl.GetInstance.StorageProviderInstance.StoragePairList)
            {
                sourcePairList.Add(item.Value, item.Key);
                tempList.Add(item.Value);
            }

            SourceList = tempList;
            OnPropertyChanged("SourceList");
        }

        /// <summary>
        /// A method for reading video contents through VideoProvider and updating VideoList
        /// </summary>
        /// <param name="option">A current sort option</param>
        /// <param name="storageId">A storage id</param>
        private async void ReadVideoList(SortOption option, string storageId = null)
        {
            VideoList.Clear();

            IEnumerable<MediaInformationEx> list = await MediaHubImpl.GetInstance.VideoProviderInstance.ReadWithoutGroupAsync(SortOption.Title);
            await MediaHubImpl.GetInstance.VideoProviderInstance.CheckThumbnail(list);

            IEnumerable<GroupItem> tempList = await MediaHubImpl.GetInstance.VideoProviderInstance.ReadAsync(option, storageId);

            IsNeededClearList = true;

            foreach (var group in tempList)
            {
               VideoList.Add(group);
            }

            IsNeededClearList = false;

            foreach (var group in VideoList)
            {
                foreach (var item in group.Contents)
                {
                    PlayList.Add(item.Information);
                }
            }
        }

        /// <summary>
        /// A method for setting the current video
        /// </summary>
        /// <param name="videoContent">An information of current video</param>
        private void SetCurrentVideo(MediaInformationEx videoContent)
        {
            CurrentVideo = videoContent;
            CurrentVideoIndex = PlayList.IndexOf(CurrentVideo);
            OnPropertyChanged("CurrentVideo");
        }

        /// <summary>
        /// A method for setting the next video
        /// </summary>
        private void SetPrevVideo()
        {
            if (CurrentVideoIndex != 0)
            {
                CurrentVideoIndex--;
                CurrentVideo = PlayList[CurrentVideoIndex];
                OnPropertyChanged("CurrentVideo");
            }
        }

        /// <summary>
        /// A method for setting the next video
        /// </summary>
        private void SetNextVideo()
        {
            if (CurrentVideoIndex != (PlayList.Count - 1))
            {
                CurrentVideoIndex++;
                CurrentVideo = PlayList[CurrentVideoIndex];
                OnPropertyChanged("CurrentVideo");
            }
        }

        /// <summary>
        /// A method for setting the current video
        /// </summary>
        /// <param name="mediaId">A mediaId to be current video</param>
        public void SetCurrentVideo(string mediaId)
        {
            CurrentVideo = MediaHubImpl.GetInstance.VideoProviderInstance.Read(mediaId);
            OnPropertyChanged("CurrentVideo");
        }
    }
}
