﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;
using TVMediaHub.Tizen.Utils;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;
using TizenVisualElement = Xamarin.Forms.PlatformConfiguration.TizenSpecific.VisualElement;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom view for displaying footer when the status of page is select mode
    /// </summary>
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class FooterDeleteStatus : RelativeLayout
    {
        public EventHandler SelecteAllButtonEvent;
        public EventHandler OKButtonEvent;
        public EventHandler CancelButtonEvent;

        /// <summary>
        /// The flag that whether all contents are selected or not
        /// </summary>
        public bool IsSelectedAll { get; set; }

        /// <summary>
        /// Identifies the SelectedCount bindable property
        /// </summary>
        public static readonly BindableProperty SelectedCountProperty = BindableProperty.Create("SelectedCount", typeof(int), typeof(FooterDeleteStatus), 0);

        /// <summary>
        /// Gets or sets count of selected item
        /// </summary>
        public int SelectedCount
        {
            get { return (int)GetValue(SelectedCountProperty); }
            set { SetValue(SelectedCountProperty, value); }
        }

        /// <summary>
        /// Identifies the ContentType bindable property
        /// </summary>
        public static readonly BindableProperty ContentTypeProperty = BindableProperty.Create("ContentType", typeof(string), typeof(FooterDeleteStatus), "Movie");

        /// <summary>
        /// Gets or sets ContentType
        /// </summary>
        public string ContentType
        {
            get { return (string)GetValue(ContentTypeProperty); }
            set { SetValue(ContentTypeProperty, value); }
        }

        /// <summary>
        /// A method for getting SelectAllButton's button object
        /// </summary>
        /// <returns>A button object</returns>
        public Button GetSelectAllButton()
        {
            return SelectAllButton;
        }

        /// <summary>
        /// A method for getting OkButton's button object
        /// </summary>
        /// <returns>A button object</returns>
        public Button GetOkButton()
        {
            return OkButton;
        }

        /// <summary>
        /// A method for getting CancelButton's button object
        /// </summary>
        /// <returns>A button object</returns>
        public Button GetCancelButton()
        {
            return CancelButton;
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public FooterDeleteStatus()
        {
            InitializeComponent();
            IsSelectedAll = false;

            SelectAllButton.Clicked += (s, e) =>
            {
                IsSelectedAll = !IsSelectedAll;

                if (IsSelectedAll)
                {
                    SelectAllButton.Text = "Unselect All";
                }
                else
                {
                    SelectAllButton.Text = "Select All";
                }

                SelecteAllButtonEvent?.Invoke(s, e);
            };
            OkButton.Clicked += (s, e) =>
            {
                OKButtonEvent?.Invoke(s, e);
                IsSelectedAll = false;
                SelectAllButton.Text = "Select All";

            };
            CancelButton.Clicked += (s, e) =>
            {
                CancelButtonEvent?.Invoke(s, e);
                IsSelectedAll = false;
                SelectAllButton.Text = "Select All";
            };
            InitializeSize();
            InitializeLabel();

            PropertyChanged += SelectedCountChanged;
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void SelectedCountChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("ContentType") || e.PropertyName.Equals("SelectedCount"))
            {
                if (SelectedCount > 0)
                {
                    OkButton.IsEnabled = true;
                    SetFocusChain();
                }
                else
                {
                    OkButton.IsEnabled = false;
                    SetFocusChain();
                }
                if (SelectedCount > 1)
                {
                    LabelCount.Text = SelectedCount + " " + ContentType + "s";
                    LabelText.Text = "are Selected";
                }
                else
                {
                    LabelCount.Text = SelectedCount + " " + ContentType;
                    LabelText.Text = "is Selected";
                }
            }
        }

        /// <summary>
        /// A method for initializing sizes
        /// </summary>
        private void InitializeSize()
        {
            int font28 = SizeUtils.GetFontSize(28);
            int font32 = SizeUtils.GetFontSize(32);

            LabelStack.Spacing = SizeUtils.GetWidthSize(8);
            LabelCount.FontSize = font32;
            LabelText.FontSize = font32;

            SelectAllButton.FontSize = font28;
            OkButton.FontSize = font28;
            CancelButton.FontSize = font28;
        }

        /// <summary>
        /// A method for initializing label to display
        /// </summary>
        private void InitializeLabel()
        {
            LabelCount.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Medium);
            LabelText.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Light);

            LabelCount.Text = "0 " + ContentType;
            LabelText.Text = "is Selected";
        }

        public void SetFocusChain()
        {
            TizenVisualElement.SetNextFocusLeftView(SelectAllButton, SelectAllButton);
            TizenVisualElement.SetNextFocusDownView(SelectAllButton, SelectAllButton);

            TizenVisualElement.SetNextFocusDownView(OkButton, OkButton);

            TizenVisualElement.SetNextFocusRightView(CancelButton, CancelButton);
            TizenVisualElement.SetNextFocusDownView(CancelButton, CancelButton);

            if (OkButton.IsEnabled)
            {
                TizenVisualElement.SetNextFocusRightView(SelectAllButton, OkButton);

                TizenVisualElement.SetNextFocusLeftView(OkButton, SelectAllButton);
                TizenVisualElement.SetNextFocusRightView(OkButton, CancelButton);

                TizenVisualElement.SetNextFocusLeftView(CancelButton, OkButton);
            }
            else
            {
                TizenVisualElement.SetNextFocusRightView(SelectAllButton, CancelButton);
                TizenVisualElement.SetNextFocusLeftView(CancelButton, SelectAllButton);
            }
        }
    }
}
