﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;

using Xamarin.Forms;
using Xamarin.Forms.Xaml;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.DataModels;
using TizenVisualElement = Xamarin.Forms.PlatformConfiguration.TizenSpecific.VisualElement;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom view for displaying footer when the status of page is normal
    /// </summary>
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class FooterNormalStatus : RelativeLayout
    {
        public static readonly BindableProperty IsFooterEnabledProperty = BindableProperty.Create("IsFooterEnabled", typeof(bool), typeof(FooterNormalStatus), true);

        public bool IsFooterEnabled
        {
            get { return (bool)GetValue(IsFooterEnabledProperty); }
            set { SetValue(IsFooterEnabledProperty, value); }
        }

        /// <summary>
        /// Identifies the SourceList bindable property
        /// </summary
        public static readonly BindableProperty SourceListProperty = BindableProperty.Create("SourceList", typeof(IEnumerable<string>), typeof(FooterNormalStatus), default(IEnumerable<string>));

        /// <summary>
        /// Gets or sets list about source of contents
        /// </summary>
        public IEnumerable<string> SourceList
        {
            get { return (IEnumerable<string>)GetValue(SourceListProperty); }
            set { SetValue(SourceListProperty, value); }
        }

        /// <summary>
        /// Identifies the SortOptions bindable property
        /// </summary
        public static readonly BindableProperty SortOptionsProperty = BindableProperty.Create("SortOptions", typeof(IEnumerable<string>), typeof(FooterNormalStatus), default(IEnumerable<string>));

        /// <summary>
        /// Gets or sets sort options
        /// </summary>
        public IEnumerable<string> SortOptions
        {
            get { return (IEnumerable<string>)GetValue(SortOptionsProperty); }
            set { SetValue(SortOptionsProperty, value); }
        }

        /// <summary>
        /// Identifies the OptionList bindable property
        /// </summary
        public static readonly BindableProperty OptionListProperty = BindableProperty.Create("OptionList", typeof(IEnumerable<string>), typeof(FooterNormalStatus), default(IEnumerable<string>));

        /// <summary>
        /// Gets or sets list about options
        /// </summary>
        public IEnumerable<string> OptionList
        {
            get { return (IEnumerable<string>)GetValue(OptionListProperty); }
            set { SetValue(OptionListProperty, value); }
        }

        /// <summary>
        /// An EventHandler for DropdownSource ItemSelected event
        /// </summary>
        public EventHandler<SelectedItemChangedEventArgs> OnDropdownSourceItemSelected;

        /// <summary>
        /// An EventHandler for DropdownSort ItemSelected event
        /// </summary>
        public EventHandler<SelectedItemChangedEventArgs> OnDropdownSortItemSelected;

        /// <summary>
        /// An EventHandler for selected item's IndexChanged event of Context popup
        /// </summary>
        public EventHandler<ContextPopupSelectedEventArgs> OnSelectedOptionIndexChanged;

        /// <summary>
        /// A DropdownList for displaying sources of contents
        /// </summary>
        public DropdownList DropdownSource;

        /// <summary>
        /// A DropdownList for displaying sort options
        /// </summary>
        public DropdownList DropdownSort;

        /// <summary>
        /// A Button for displaying page options
        /// </summary>
        public Button ButtonOption;
        private bool isPopupShowing;

        /// <summary>
        /// A Constructor
        /// </summary>
        public FooterNormalStatus()
        {
            InitializeComponent();
            InitializeFooterItems();
            PropertyChanged += FooterNormalStatusPropertyChanged;
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void FooterNormalStatusPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("SourceList"))
            {
                DropdownSource.ItemsSource = SourceList;
                var list = SourceList as List<string>;
                DropdownSource.SelectedItem = list[0];
            }
            else if (e.PropertyName.Equals("SortOptions"))
            {
                DropdownSort.ItemsSource = SortOptions;
                var list = SortOptions as List<string>;
                DropdownSort.SelectedItem = list[0];
            }
            else if (e.PropertyName.Equals("IsFooterEnabled"))
            {
                DropdownSort.IsEnabled = IsFooterEnabled;
                ButtonOption.IsEnabled = IsFooterEnabled;
            }
        }

        private void KeyDownHandler(object sender, EventArgs e)
        {
            ElmSharp.EcoreKeyEventArgs arg = e as ElmSharp.EcoreKeyEventArgs;

            if (arg != null && arg.KeyName.Contains("Back"))
            {
                if (DropdownSort.IsExpanded == true)
                {
                    DropdownSort.Collapse();
                }
            }
        }

        /// <summary>
        /// A method for initializing footer items
        /// </summary>
        private void InitializeFooterItems()
        {
            DropdownSource = new DropdownList();
            DropdownSort = new DropdownList();
            ButtonOption = new Button();

            ButtonOption.Text = "OPTION";

            DropdownSource.ItemsSource = SourceList;
            DropdownSort.ItemsSource = SortOptions;

            DropdownSort.Collapsed += (s, e) =>
            {
                if (App.AppMainPage != null && App.AppMainPage.RootPage != null)
                {
                    (App.AppMainPage.RootPage as MediaHubMainPage).DisableBackKeyHandler = false;

                    App.KeyDownEvent.On -= KeyDownHandler;
                }
            };

            DropdownSort.Expanded += (s, e) =>
            {
                if (App.AppMainPage != null && App.AppMainPage.RootPage != null)
                {
                    (App.AppMainPage.RootPage as MediaHubMainPage).DisableBackKeyHandler = true;

                    App.KeyDownEvent.On += KeyDownHandler;
                }
            };

            DropdownSource.ItemSelected += (s, e) =>
            {
                OnDropdownSourceItemSelected?.Invoke(s, e);
            };

            DropdownSort.ItemSelected += (s, e) =>
            {
                OnDropdownSortItemSelected?.Invoke(s, e);
            };

            ButtonOption.Command = new Command(() =>
            {
                ShowContextPopup();
            });

            // TODO : Change yConstraint.
            // Tab Size - 128

            Children.Add(DropdownSource,
                heightConstraint: Constraint.Constant(SizeUtils.GetHeightSize(80)),
                widthConstraint: Constraint.Constant(SizeUtils.GetWidthSize(300)),
                yConstraint: Constraint.Constant(SizeUtils.GetHeightSize(752)),
                xConstraint: Constraint.Constant(SizeUtils.GetWidthSize(96)));

            Children.Add(DropdownSort,
                heightConstraint: Constraint.Constant(SizeUtils.GetHeightSize(80)),
                widthConstraint: Constraint.Constant(SizeUtils.GetWidthSize(300)),
                yConstraint: Constraint.Constant(SizeUtils.GetHeightSize(752)),
                xConstraint: Constraint.Constant(SizeUtils.GetWidthSize(1220)));

            Children.Add(ButtonOption,
                heightConstraint: Constraint.Constant(SizeUtils.GetHeightSize(80)),
                widthConstraint: Constraint.Constant(SizeUtils.GetWidthSize(300)),
                yConstraint: Constraint.Constant(SizeUtils.GetHeightSize(752)),
                xConstraint: Constraint.Constant(SizeUtils.GetWidthSize(1524)));

            ButtonOption.FontSize = SizeUtils.GetFontSize(28);
        }

        /// <summary>
        /// A method for showing context popup
        /// </summary>
        private void ShowContextPopup()
        {
            if (isPopupShowing)
            {
                return;
            }

            ContextPopup popup = new ContextPopup
            {
                IsAutoHidingEnabled = true,
                DirectionPriorities = new ContextPopupDirectionPriorities(ContextPopupDirection.Up, ContextPopupDirection.Right, ContextPopupDirection.Left, ContextPopupDirection.Down),
            };

            if (OptionList != null)
            {
                foreach (var item in OptionList)
                {
                    popup.Items.Add(new ContextPopupItem(item));
                }
            }

            //TODO: need to change the event callback
            popup.SelectedIndexChanged += (s, args) =>
            {
                var index = popup.SelectedIndex;
                var item = popup.SelectedItem as ContextPopupItem;
                OnSelectedOptionIndexChanged.Invoke(s, new ContextPopupSelectedEventArgs(index, item));
                popup.Dismiss();
            };

            popup.Dismissed += (s, args) =>
            {
                isPopupShowing = false;
            };

            popup.Show(ButtonOption, (ButtonOption.Width / 2), 0);
            isPopupShowing = true;
        }

        public void SetFocusChain()
        {
            TizenVisualElement.SetNextFocusLeftView(DropdownSource, DropdownSource);
            TizenVisualElement.SetNextFocusRightView(DropdownSource, DropdownSort);
            TizenVisualElement.SetNextFocusDownView(DropdownSource, DropdownSource);

            TizenVisualElement.SetNextFocusLeftView(DropdownSort, DropdownSource);
            TizenVisualElement.SetNextFocusRightView(DropdownSort, ButtonOption);
            TizenVisualElement.SetNextFocusDownView(DropdownSort, DropdownSort);

            TizenVisualElement.SetNextFocusLeftView(ButtonOption, DropdownSort);
            TizenVisualElement.SetNextFocusRightView(ButtonOption, ButtonOption);
            TizenVisualElement.SetNextFocusDownView(ButtonOption, ButtonOption);
        }
    }
}
