﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Input;
using Tizen.Content.MediaContent;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.Utils;
using Xamarin.Forms;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;
using Xamarin.Forms.Xaml;
using static TVMediaHub.Tizen.Views.ImageItem;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom view for displaying image group
    /// </summary>
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class ImageGroup : RelativeLayout
    {
        /// <summary>
        /// Identifies the ItemsSource bindable property
        /// </summary>
        public static readonly BindableProperty ItemsSourceProperty = BindableProperty.Create("ItemsSource", typeof(IEnumerable<MediaShortcutInfo>), typeof(ImageTab), default(IEnumerable<MediaShortcutInfo>));

        /// <summary>
        /// A command will be executed when item is clicked
        /// </summary>
        private ICommand ItemClickCommand;

        public delegate void ContextPopupItemSelectedEventHandler(MediaInformationEx info, ImageContextPopupItem item);

        public ContextPopupItemSelectedEventHandler ContextPopupItemSelectedHandler;

        /// <summary>
        /// Gets or set ItemSource to display item
        /// </summary>
        public IEnumerable<MediaShortcutInfo> ItemsSource
        {
            get { return (IEnumerable<MediaShortcutInfo>)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        /// <summary>
        /// A method for getting Title area's button object
        /// </summary>
        /// <returns>A button object</returns>
        public Button GetTitleFocusArea()
        {
            return TitleFocusArea;
        }

        private List<Button> ChildrenFocusList;
        public List<Button> LeftFocusList { get; protected set; }
        public List<Button> RightFocusList { get; protected set; }
        public List<KeyValuePair<double, Button>> BottomFocusList { get; protected set; }
        public List<ImageItem> ImageItemList { get; set; }

        public EventHandler<GroupItemFocusEventArgs> GroupItemFocused;

        public ImageItem focusedItem;

        /// <summary>
        /// A constructor
        /// </summary>
        public ImageGroup()
        {
            InitializeComponent();
            Init();
            InitializeSize();
            PropertyChanged += ImageGroupPropertyChanged;
        }

        /// <summary>
        /// A method for initializing member fields
        /// </summary>
        private void Init()
        {
            TitleFocusArea.Clicked += (e, v) =>
            {
                TitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().MoveFocusDown();
            };

            ChildrenFocusList = new List<Button>();
            LeftFocusList = new List<Button>();
            RightFocusList = new List<Button>();
            BottomFocusList = new List<KeyValuePair<double, Button>>();
            ImageItemList = new List<ImageItem>();

            GroupTitle.TextColor = Color.FromRgba(255, 255, 255, 31);
            GroupTitle.FontFamily = "BreezSans";
            GroupTitle.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Normal);

            TitleFocusArea.Focused += TitleFocused;
            TitleFocusArea.Unfocused += TitleUnfocused;
        }

        /// <summary>
        /// This method is called when the group's title is unfocused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private void TitleUnfocused(object sender, FocusEventArgs e)
        {
            GroupTitle.AbortAnimation("FocusAnimation");
            GroupTitle.Animate("FocusAnimation",
                (v) =>
                {
                    var R = (int)(255 * v);
                    var G = (int)(129 + 126 * v);
                    var B = (int)(198 + 57 * v);
                    var A = (int)(255 - 224 * v);
                    GroupTitle.TextColor = Color.FromRgba(R, G, B, A);
                },
                length: 167,
                finished: (v, IsCanceled) =>
                {
                    if (!IsCanceled)
                    {
                        GroupTitle.TextColor = Color.FromRgba(255, 255, 255, 31);
                    }
                });
        }

        /// <summary>
        /// This method is called when the group's title is focused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private void TitleFocused(object sender, FocusEventArgs e)
        {
            GroupTitle.AbortAnimation("FocusAnimation");
            GroupTitle.Animate("FocusAnimation",
                (v) => { },
                length: 167,
                finished: (v, IsCanceled) =>
                {
                    if (!IsCanceled)
                    {

                        GroupTitle.Animate("FocusAnimation",
                            (v2) =>
                            {
                                var R = (int)(255 * (1 - v));
                                var G = (int)(255 - 126 * v);
                                var B = (int)(255 - 57 * v);
                                var A = (int)(31 + 224 * v);
                                GroupTitle.TextColor = Color.FromRgba(R, G, B, A);
                            },
                            length: 167,
                            finished: (v2, IsCanceled2) =>
                            {
                                if (!IsCanceled2)
                                {
                                    GroupTitle.TextColor = Color.FromRgba(0, 129, 198, 255);
                                }
                            });
                    }
                });
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void ImageGroupPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("ItemsSource"))
            {
                var index = 0;
                ChildrenFocusList.Clear();
                ImageItemList.Clear();
                foreach (var item in ItemsSource)
                {
                    var view = new ImageItem();
                    ChildrenFocusList.Add(view.GetFocusArea());
                    view.BindingContext = item;
                    ImageItemList.Add(view);
                    view.OnFocusedEventHandler += (se, ev) =>
                    {
                        GroupItemFocused?.Invoke(view, new GroupItemFocusEventArgs(X + view.X));
                        GroupContentArea.RaiseChild(view);
                        focusedItem = view;
                    };
                    view.OnUnfocusedEventHandler += (se, ev) =>
                    {
                        if (focusedItem != null && focusedItem.Equals(view as ImageItem))
                        {
                            focusedItem = null;
                        }
                    };
                    view.OnItemClickedHandler += (info) =>
                    {
                        ItemClickCommand?.Execute(info);
                    };
                    view.ContextPopupItemSelectedHandler += (info, selItem) =>
                    {
                        ContextPopupItemSelectedHandler?.Invoke(info, selItem);
                    };
                    GroupContentArea.Children.Add(view, index / 3, index % 3);
                    index++;
                }

                SetFocusChain();
            }
        }

        /// <summary>
        /// A method for initializing sizes
        /// </summary>
        private void InitializeSize()
        {
            GroupTitle.FontSize = Utils.SizeUtils.GetFontSize(28);
            GroupTitle.HeightRequest = Utils.SizeUtils.GetHeightSize(50);
            GroupTitle.Margin = new Thickness(SizeUtils.GetWidthSize(12), 0, 0, 0);

            TitleFocusArea.WidthRequest = SizeUtils.GetWidthSize(180);
            GroupContentArea.RowSpacing = Utils.SizeUtils.GetHeightSize(-150);
            GroupContentArea.ColumnSpacing = Utils.SizeUtils.GetWidthSize(-150);
            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(332) });
            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(332) });
            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(332) });
            GroupContentArea.Margin = new Thickness(SizeUtils.GetWidthSize(-64), SizeUtils.GetHeightSize(-64), SizeUtils.GetWidthSize(-64), SizeUtils.GetHeightSize(-64));
        }

        /// <summary>
        /// A method for making focus chain among items
        /// </summary>
        private void SetFocusChain()
        {
            if (ChildrenFocusList.Count < 1)
            {
                return;
            }

            var lastColumn = (ChildrenFocusList.Count - 1) / 3;

            SetFocusList();

            TitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusDownView(ChildrenFocusList[0]);
            TitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(TitleFocusArea);
            TitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(TitleFocusArea);

            for (var i = 0; i < ChildrenFocusList.Count; i++)
            {
                var target = ChildrenFocusList[i];
                var row = i % 3;
                var column = i / 3;

                // Left
                if (column != 0)
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(ChildrenFocusList[i - 3]);
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(target);
                }

                // Right
                if (column != lastColumn)
                {
                    if ((i + 3) < ChildrenFocusList.Count)
                    {
                        target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(ChildrenFocusList[i + 3]);
                    }
                    else
                    {
                        target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(ChildrenFocusList[ChildrenFocusList.Count - 1]);
                    }
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(target);
                }

                // Up
                if (row != 0)
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusUpView(ChildrenFocusList[i - 1]);
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusUpView(TitleFocusArea);
                }

                // Down
                if (row != 2 && (i + 1) < ChildrenFocusList.Count)
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusDownView(ChildrenFocusList[i + 1]);
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusDownView(target);
                    var targetX = column * SizeUtils.GetWidthSize(204);
                    BottomFocusList.Add(new KeyValuePair<double, Button>(targetX, target));
                }
            }
        }

        /// <summary>
        /// A method for making focus list
        /// </summary>
        private void SetFocusList()
        {
            var totalCount = ChildrenFocusList.Count - 1;
            var lastItemColumn = totalCount / 3;
            var lastItemRow = totalCount % 3;

            var leftFocusFrom = 0;
            var leftFocusTo = lastItemColumn > 0 ? 2 : lastItemRow;

            var rightFocusFrom = lastItemColumn * 3;
            var rightFocusTo = rightFocusFrom + lastItemRow;

            for (var i = leftFocusFrom; i <= leftFocusTo; i++)
            {
                LeftFocusList.Add(ChildrenFocusList[i]);
            }

            for (var i = rightFocusFrom; i <= rightFocusTo; i++)
            {
                RightFocusList.Add(ChildrenFocusList[i]);
            }
        }

        /// <summary>
        /// A method for setting the command to ItemClickCommand
        /// </summary>
        /// <param name="command">A command to be set</param>
        public void SetClickCommand(ICommand command)
        {
            ItemClickCommand = command;
        }

        /// <summary>
        /// If there is focused image item, notify the menu key pressed event to focused video item for show context popup
        /// </summary>
        /// <returns>If group has a focused image item, true; otherwise, false</returns>
        public bool NotifyMenuKeyPressedToFocusedItem()
        {
            if (focusedItem != null)
            {
                focusedItem.ShowContextPopup();
                return true;
            }

            return false;
        }
    }
}
