﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.ComponentModel;
using Tizen.Content.MediaContent;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.DataModels;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;
using TVMediaHub.Tizen.ViewModels;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom view for displaying image item
    /// </summary>
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class ImageItem : RelativeLayout
    {
        /// <summary>
        /// An enumeration for ItemStatus
        /// </summary>
        public enum ItemStatus
        {
            Normal = 0,
            Selected,
        };
        public enum ImageContextPopupItem
        {
            FileInfo = 0,
            Delete,
        }

        /// <summary>
        /// A flag that whether the item is loaded or not
        /// </summary>
        private bool IsLoaded = false;

        /// <summary>
        /// Bounds for ImageArea(Image Container) when item is unfocused
        /// </summary>
        private Xamarin.Forms.Rectangle ImageAreaNormalBounds;

        /// <summary>
        /// Bounds for ImageArea(Image Container) when item is focused
        /// </summary>
        private Xamarin.Forms.Rectangle ImageAreaFocusedBounds;

        /// <summary>
        /// Bounds for Content Image when item is unfocused
        /// </summary>
        private Xamarin.Forms.Rectangle ImageBounds;

        /// <summary>
        /// Bounds for Content Image when item is focused
        /// </summary>
        private Xamarin.Forms.Rectangle ImageFocusedBounds;

        /// <summary>
        /// Bounds for Shadow Image when item is unfocused
        /// </summary>
        private Xamarin.Forms.Rectangle ShadowNormalBounds;

        /// <summary>
        /// Bounds for Shadow Image when item is focused
        /// </summary>
        private Xamarin.Forms.Rectangle ShadowFocusedBounds;

        /// <summary>
        /// An EventHandler for focus event of the item
        /// </summary>
        public EventHandler OnFocusedEventHandler;
        /// <summary>
        /// An EventHandler for unfocus event of the item
        /// </summary>
        public EventHandler OnUnfocusedEventHandler;
        /// <summary>
        /// A delegate will be executed when the item is clicked
        /// </summary>
        /// <param name="info">A clicked item's MediaInformation</param>
        public delegate void ItemEventHandler(MediaInformationEx info);
        /// <summary>
        /// A ClickEventHandler for click event of the item
        /// </summary>
        public ItemEventHandler OnItemClickedHandler;

        public delegate void ContextPopupItemSelectedEventHandler(MediaInformationEx info, ImageContextPopupItem item);

        public ContextPopupItemSelectedEventHandler ContextPopupItemSelectedHandler;

        /// <summary>
        /// Identifies the ImageTabStatus bindable property
        /// </summary>
        public static readonly BindableProperty ImageTabStatusProperty = BindableProperty.Create("ImageTabStatus", typeof(TabStatus), typeof(ImageItem), TabStatus.Default);

        /// <summary>
        /// Gets or sets ImageTabStatus
        /// </summary>
        public TabStatus ImageTabStatus
        {
            set
            {
                SetValue(ImageTabStatusProperty, value);
            }

            get
            {
                return (TabStatus)GetValue(ImageTabStatusProperty);
            }
        }

        /// <summary>
        /// Identifies the SelectedImage bindable property
        /// </summary>
        public static readonly BindableProperty ImageInfoProperty = BindableProperty.Create("SelectedImage", typeof(MediaInformationEx), typeof(ImageItem), null);

        /// <summary>
        /// Gets or sets the MediaInformationEx of selected image
        /// </summary>
        public MediaInformationEx SelectedImage
        {
            set
            {
                SetValue(ImageInfoProperty, value);
            }

            get
            {
                return (MediaInformationEx)GetValue(ImageInfoProperty);
            }
        }

        /// <summary>
        /// A value of current ItemStatus
        /// </summary>
        private ItemStatus CurStatus;

        /// <summary>
        /// A constructor
        /// Initializes the size of the items that are used in this class
        /// </summary>
        public ImageItem()
        {
            InitializeComponent();
            InitializeSize();
            PropertyChanged += ImageTabPropertyChanged;
        }

        /// <summary>
        /// A method for initializing size of the items that are used in this class
        /// </summary>
        private void InitializeSize()
        {
            int h332 = SizeUtils.GetHeightSize(332);
            int w332 = SizeUtils.GetWidthSize(332);
            int h76 = SizeUtils.GetHeightSize(76);
            int w76 = SizeUtils.GetWidthSize(76);
            int h64 = SizeUtils.GetHeightSize(64);
            int w64 = SizeUtils.GetWidthSize(64);
            int h20 = SizeUtils.GetHeightSize(20);
            int w20 = SizeUtils.GetWidthSize(20);
            int h5 = SizeUtils.GetHeightSize(5);
            int w5 = SizeUtils.GetWidthSize(5);


            WidthRequest = h332;
            HeightRequest = h332;

            ImageBounds = new Xamarin.Forms.Rectangle(-w76, -h76, w332, h332);
            ImageFocusedBounds = new Xamarin.Forms.Rectangle(-w64, -h64, w332, h332);

            ImageAreaNormalBounds = new Xamarin.Forms.Rectangle(w76, h76, SizeUtils.GetWidthSize(180), SizeUtils.GetHeightSize(180));
            ImageAreaFocusedBounds = new Xamarin.Forms.Rectangle(w64, h64, SizeUtils.GetWidthSize(204), SizeUtils.GetHeightSize(204));

            ShadowNormalBounds = new Xamarin.Forms.Rectangle(SizeUtils.GetWidthSize(12), SizeUtils.GetWidthSize(12), SizeUtils.GetWidthSize(308), SizeUtils.GetHeightSize(308));
            ShadowFocusedBounds = new Xamarin.Forms.Rectangle(0, 0, w332, h332);

            ImgFocused.BorderLeft = w20;
            ImgFocused.BorderRight = w20;
            ImgFocused.BorderTop = h20;
            ImgFocused.BorderBottom = h20;

            ImgDimmed.BorderLeft = w20;
            ImgDimmed.BorderRight = w20;
            ImgDimmed.BorderTop = h20;
            ImgDimmed.BorderBottom = h20;

            ImgCheckDimmed.BorderLeft = w5;
            ImgCheckDimmed.BorderRight = w5;
            ImgCheckDimmed.BorderTop = h5;
            ImgCheckDimmed.BorderBottom = h5;

            ImgCheck.BorderLeft = w5;
            ImgCheck.BorderRight = w5;
            ImgCheck.BorderTop = h5;
            ImgCheck.BorderBottom = h5;

            ImgCheck.ImageColor = Color.White;
        }

        /// <summary>
        /// A method for getting item's button object
        /// </summary>
        /// <returns>A button object</returns>
        public Button GetFocusArea()
        {
            return FocusArea;
        }

        /// <summary>
        /// This method is called when item is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event argument</param>
        private async void OnItemClicked(object sender, EventArgs e)
        {
            if (ImgDimmed.AnimationIsRunning("LayoutTo"))
            {
                return;
            }
#pragma warning disable CS4014
            ContentImage.LayoutTo(ImageBounds, 167);
            ImgFocused.LayoutTo(ShadowNormalBounds, 167);
            ImgDimmed.LayoutTo(ImageAreaNormalBounds, 167);
            ImgCheckDimmed.LayoutTo(ImageAreaNormalBounds, 167);
            ImgCheck.LayoutTo(ImageAreaNormalBounds, 167);
            await ImageArea.LayoutTo(ImageAreaNormalBounds, 167);

            ImgDimmed.LayoutTo(ImageAreaFocusedBounds, 167);
            ImgFocused.LayoutTo(ShadowFocusedBounds, 167);
            ContentImage.LayoutTo(ImageFocusedBounds, 167);
            ImgCheckDimmed.LayoutTo(ImageAreaFocusedBounds, 167);
            ImgCheck.LayoutTo(ImageAreaFocusedBounds, 167);
            ImageArea.LayoutTo(ImageAreaFocusedBounds, 167);
#pragma warning restore CS4014
            OnItemClickedHandler?.Invoke(SelectedImage);
            if (ImageTabStatus == TabStatus.Delete)
            {
                if (CurStatus == ItemStatus.Normal)
                {
                    CurStatus = ItemStatus.Selected;
                }
                else
                {
                    CurStatus = ItemStatus.Normal;
                }

                UpdateView();
            }
            else
            {
                Program.TransitionTo(new SimpleImageViewer());
            }
        }

        public void UpdateViewSelectAll(bool isSelectedAll)
        {
            if (!isSelectedAll)
            {
                ImgCheckDimmed.FadeTo(0.0, 167);
                ImgCheck.FadeTo(0.0, 167);
                CurStatus = ItemStatus.Normal;
            }
            else
            {
                ImgCheckDimmed.FadeTo(0.75, 167);
                ImgCheck.FadeTo(0.99, 167);
                CurStatus = ItemStatus.Selected;
            }
        }

        /// <summary>
        /// A method for updating view according to current status
        /// </summary>
        private void UpdateView()
        {
            switch (CurStatus)
            {
                case ItemStatus.Normal:
                    ImgCheckDimmed.FadeTo(0.0, 167);
                    ImgCheck.FadeTo(0.0, 167);
                    break;
                case ItemStatus.Selected:
                    ImgCheckDimmed.FadeTo(0.75, 167);
                    ImgCheck.FadeTo(0.99, 167);
                    break;
            }
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void ImageTabPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName.CompareTo("ImageTabStatus") == 0)
            {
                if (ImageTabStatus == TabStatus.Default)
                {
                    CurStatus = ItemStatus.Normal;
                    UpdateView();
                }
            }
            else if (e.PropertyName.CompareTo("SelectedImage") == 0)
            {
                if (SelectedImage.IsAvailable == false)
                {
                    ContentImage.IsVisible = false;
                    UnavailableIcon.IsVisible = true;
                }
            }
        }

        /// <summary>
        /// A method for getting absolute position of item
        /// </summary>
        /// <returns>An absolute position</returns>
        private Point GetAbsolutePosition()
        {
            // TODO : Find proper way to get absolute location of this element.
            var result = new Point();
            var x = this.X;
            var y = this.Y;
            var parent = (VisualElement)this.Parent;
            while (parent != null && parent.Parent?.GetType() == typeof(VisualElement))
            {
                x += parent.X;
                y += parent.Y;
                parent = (VisualElement)parent.Parent;
            }

            result.X = x;
            result.Y = y;

            return result;
        }

        /// <summary>
        /// This method is called when item is focused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private void OnItemFocused(object sender, FocusEventArgs e)
        {
            Easing easing = new Easing(EasingFunction.EasyIn1);
            this.Animate("FocusAnimation",
                (v) => { },
                length: 167,
                finished: (v, IsCanceled) =>
                {
                    if (!IsCanceled)
                    {
                        ImgFocused.Opacity = 0.99;
                        OnFocusedEventHandler?.Invoke(sender, e);
                        ContentImage.ScaleTo(1.3, 167, easing);
                        ImgDimmed.LayoutTo(ImageAreaFocusedBounds, 500, easing);
                        ImgFocused.LayoutTo(ShadowFocusedBounds, 500, easing);
                        ContentImage.LayoutTo(ImageFocusedBounds, 500, easing);

                        if (SelectedImage.IsAvailable == false)
                        {
                            UnavailableIcon.ScaleTo(1.3, 167, easing);
                            UnavailableIcon.LayoutTo(ImageFocusedBounds, 500, easing);
                        }

                        if (ImageTabStatus == TabStatus.Delete)
                        {
                            ImgCheckDimmed.LayoutTo(ImageAreaFocusedBounds, 500, easing);
                            ImgCheck.LayoutTo(ImageAreaFocusedBounds, 500, easing);
                        }

                        ImageArea.LayoutTo(ImageAreaFocusedBounds, 500, easing);
                    }
                });
        }

        /// <summary>
        /// This method is called when item is unfocused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private void OnItemUnfocused(object sender, FocusEventArgs e)
        {
            OnUnfocusedEventHandler?.Invoke(sender, e);
            Easing easing = new Easing(EasingFunction.EasyIn2);
            this.AbortAnimation("FocusAnimation");
            ImgFocused.Opacity = 0.0;
            ContentImage.ScaleTo(1.0, 167, easing);
            ContentImage.LayoutTo(ImageBounds, 167, easing);

            if (SelectedImage.IsAvailable == false)
            {
                UnavailableIcon.ScaleTo(1.0, 167, easing);
                UnavailableIcon.Layout(ImageBounds);
            }

            ImgFocused.LayoutTo(ShadowNormalBounds, 167, easing);
            ImgDimmed.LayoutTo(ImageAreaNormalBounds, 167, easing);
            if (ImageTabStatus == TabStatus.Delete)
            {
                ImgCheckDimmed.LayoutTo(ImageAreaNormalBounds, 167, easing);
                ImgCheck.LayoutTo(ImageAreaNormalBounds, 167, easing);
            }

            ImageArea.LayoutTo(ImageAreaNormalBounds, 167, easing);
        }

        /// <summary>
        /// Positions and sizes the children of a Layout.
        /// </summary>
        /// <param name="x">A value representing the x coordinate of the child region bounding box.</param>
        /// <param name="y">A value representing the y coordinate of the child region bounding box.</param>
        /// <param name="width">A value representing the width of the child region bounding box.</param>
        /// <param name="height">A value representing the height of the child region bounding box.</param>
        protected override void LayoutChildren(double x, double y, double width, double height)
        {
            if (IsLoaded == false)
            {
                FocusArea.Layout(ImageAreaNormalBounds);
                ImgFocused.Layout(ShadowNormalBounds);
                ImageArea.Layout(ImageAreaNormalBounds);
                ContentImage.Layout(ImageBounds);
                ImgDimmed.Layout(ImageBounds);
                ImgCheckDimmed.Layout(ImageAreaNormalBounds);
                ImgCheck.Layout(ImageAreaNormalBounds);
                UnavailableIcon.Layout(ImageAreaNormalBounds);
                IsLoaded = true;
            }
        }

        public void ShowContextPopup()
        {
            if (!FocusArea.IsFocused)
            {
                return;
            }

            ContextPopup popup = new ContextPopup();
            popup.Items.Add(new ContextPopupItem("FILE INFO"));
            popup.Items.Add(new ContextPopupItem("DELETE"));
            popup.SelectedIndexChanged += SelectedIndexChanged;
            popup.Show(this, this.Width / 2, this.Height + SizeUtils.GetHeightSize(72));

        }

        private void SelectedIndexChanged(object sender, EventArgs e)
        {
            var ctxPopup = sender as ContextPopup;
            switch (ctxPopup.SelectedIndex)
            {
                case 0:
                    ContextPopupItemSelectedHandler?.Invoke(SelectedImage, ImageContextPopupItem.FileInfo);
                    break;
                case 1:
                    ContextPopupItemSelectedHandler?.Invoke(SelectedImage, ImageContextPopupItem.Delete);
                    break;
            }

            ctxPopup.Dismiss();
        }

    }
}
