﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading;
using System.Windows.Input;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.Controls;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.ViewModels;
using Xamarin.Forms;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;
using static TVMediaHub.Tizen.Views.ImageItem;
using TizenVisualElement = Xamarin.Forms.PlatformConfiguration.TizenSpecific.VisualElement;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom ContentPage for displaying the image tab
    /// </summary>
    public partial class ImageTab : ContentPageEx
    {
        /// <summary>
        /// The flag whether content is ready
        /// </summary>
        private bool IsContentReady = false;

        /// <summary>
        /// The flag that whether all contents are selected or not
        /// </summary>
        private bool IsSelectedAll { get; set; }

        /// <summary>
        /// Identifies the ItemsSource bindable property
        /// </summary>
        public static readonly BindableProperty ItemsSourceProperty = BindableProperty.Create("ItemsSource", typeof(ObservableCollection<GroupItem>), typeof(ImageTab), default(ObservableCollection<GroupItem>));

        /// <summary>
        /// Gets or set ItemSource to display image tab
        /// </summary>
        public ObservableCollection<GroupItem> ItemsSource
        {
            get { return (ObservableCollection<GroupItem>)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        /// <summary>
        /// Identifies the ChangeSortOptionCommand bindable property
        /// </summary>
        public static readonly BindableProperty ChangeSortOptionCommandProperty = BindableProperty.Create("ChangeSortOptionCommand", typeof(ICommand), typeof(ImageTab), default(ICommand));

        /// <summary>
        /// Gets or sets ChangeSortOptionCommand
        /// </summary>
        public ICommand ChangeSortOptionCommand
        {
            get { return (ICommand)GetValue(ChangeSortOptionCommandProperty); }
            set { SetValue(ChangeSortOptionCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the ChangeSourceCommand bindable property
        /// </summary>
        public static readonly BindableProperty ChangeSourceCommandProperty = BindableProperty.Create("ChangeSourceCommand", typeof(ICommand), typeof(ImageTab), default(ICommand));

        /// <summary>
        /// Gets or sets ChangeSourceCommand Command
        /// </summary>
        public ICommand ChangeSourceCommand
        {
            get { return (ICommand)GetValue(ChangeSourceCommandProperty); }
            set { SetValue(ChangeSourceCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the DeleteModeChangeCommand bindable property
        /// </summary>
        public static readonly BindableProperty DeleteModeChangeCommandProperty = BindableProperty.Create("DeleteModeChangeCommand", typeof(ICommand), typeof(ImageTab), default(ICommand));

        /// <summary>
        /// Gets or sets DeleteModeChangeCommand
        /// </summary>
        public ICommand DeleteModeChangeCommand
        {
            get { return (ICommand)GetValue(DeleteModeChangeCommandProperty); }
            set { SetValue(DeleteModeChangeCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the GetInformationsCommand bindable property
        /// </summary>
        public static readonly BindableProperty GetInformationsCommandProperty = BindableProperty.Create("GetInformationsCommand", typeof(ICommand), typeof(ImageTab), default(ICommand));

        /// <summary>
        /// Gets or sets GetInformationsCommand
        /// </summary>
        public ICommand GetInformationsCommand
        {
            get { return (ICommand)GetValue(GetInformationsCommandProperty); }
            set { SetValue(GetInformationsCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the OnClickCommand bindable property
        /// </summary>
        public static readonly BindableProperty OnClickCommandProperty = BindableProperty.Create("OnClickCommand", typeof(ICommand), typeof(ImageTab), default(ICommand));

        /// <summary>
        /// Gets or sets OnClickCommand
        /// </summary>
        public ICommand OnClickCommand
        {
            get
            {
                return (ICommand)GetValue(OnClickCommandProperty);
            }

            set
            {
                SetValue(OnClickCommandProperty, value);
            }
        }

        /// <summary>
        /// Identifies the DeleteContentCommand bindable property
        /// </summary>
        public static readonly BindableProperty DeleteContentCommandProperty = BindableProperty.Create("DeleteContentCommand", typeof(ICommand), typeof(ImageTab), default(ICommand));

        /// <summary>
        /// Gets or sets DeleteContentCommand
        /// </summary>
        public ICommand DeleteContentCommand
        {
            get
            {
                return (ICommand)GetValue(DeleteContentCommandProperty);
            }

            set
            {
                SetValue(DeleteContentCommandProperty, value);
            }
        }

        /// <summary>
        /// Identifies the DeleteContentCommand bindable property
        /// </summary>
        public static readonly BindableProperty SelectAllContentCommandProperty = BindableProperty.Create("SelectAllContentCommand", typeof(ICommand), typeof(ImageTab), default(ICommand));

        /// <summary>
        /// Gets or sets SelectAllContent Command
        /// </summary>
        public ICommand SelectAllContentCommand
        {
            get { return (ICommand)GetValue(SelectAllContentCommandProperty); }
            set { SetValue(SelectAllContentCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the ImageTabStatus bindable property
        /// </summary>
        public static readonly BindableProperty ImageTabStatusProperty = BindableProperty.Create("ImageTabStatus", typeof(TabStatus), typeof(ImageTab), TabStatus.Default);

        /// <summary>
        /// Gets or sets ImageTabStatus
        /// </summary>
        public TabStatus ImageTabStatus
        {
            get { return (TabStatus)GetValue(ImageTabStatusProperty); }
            set { SetValue(ImageTabStatusProperty, value); }
        }

        /// <summary>
        /// Identifies the IsNeededClearList bindable property
        /// </summary>
        public static readonly BindableProperty IsNeededClearListProperty = BindableProperty.Create("IsNeededClearList", typeof(bool), typeof(ImageTab), false);

        /// <summary>
        /// Gets or sets IsNeededClearList
        /// </summary>
        public bool IsNeededClearList
        {
            get { return (bool)GetValue(IsNeededClearListProperty); }
            set { SetValue(IsNeededClearListProperty, value); }
        }

        /// <summary>
        /// A list of bottom buttons
        /// </summary>
        private List<KeyValuePair<double, Button>> BottomButtonList;

        public List<ImageGroup> ImageGroupList { get; set; }

        /// <summary>
        /// Identifies the SelectedCount bindable property
        /// </summary>
        public static readonly BindableProperty SelectedCountProperty = BindableProperty.Create("SelectedCount", typeof(int), typeof(ImageTab), 0);

        /// <summary>
        /// Gets or sets count of selected item
        /// </summary>
        public int SelectedCount
        {
            get { return (int)GetValue(SelectedCountProperty); }
            set { SetValue(SelectedCountProperty, value); }
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public ImageTab()
        {
            BindingContext = ImageTabViewModelLocator.ViewModel;
            InitializeComponent();

            InitializeData();
            InitializeSize();
            InitializeFonts();
            InitializeFooter();
            ItemsSource.CollectionChanged += ItemsSourceCollectionChanged;
            PropertyChanged += ImageTabPropertyChanged;
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void ImageTabPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("IsNeededClearList"))
            {
                if (IsNeededClearList)
                {
                    BottomButtonList.Clear();
                }
            }
            else if (e.PropertyName.Equals("ImageTabStatus"))
            {
                SetFooterFocusChain(ImageTabScrollView.ScrollX);
            }
        }

        /// <summary>
        /// A method for initializing list
        /// </summary>
        private void InitializeData()
        {
            BottomButtonList = new List<KeyValuePair<double, Button>>();
            ImageGroupList = new List<ImageGroup>();
        }

        /// <summary>
        /// A method for initializing several sizes
        /// </summary>
        private void InitializeSize()
        {
            GalleryContentView.Padding = new Thickness(SizeUtils.GetWidthSize(96), SizeUtils.GetWidthSize(74));
            GalleryContentView.Spacing = SizeUtils.GetWidthSize(36);
            ImgNoContents.Margin = new Thickness(0, Utils.SizeUtils.GetHeightSize(266), 0, Utils.SizeUtils.GetHeightSize(28));
            ImgNoContents.HeightRequest = Utils.SizeUtils.GetHeightSize(88);
            ImgNoContents.WidthRequest = Utils.SizeUtils.GetWidthSize(68);
            LabelNoContents.HeightRequest = Utils.SizeUtils.GetHeightSize(32);
            LabelNoContents.FontSize = Utils.SizeUtils.GetFontSize(28);
        }

        /// <summary>
        /// A method for initializing several fonts that are used in this class
        /// </summary>
        private void InitializeFonts()
        {
            LabelNoContents.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Light);
            LabelNoContents.TextColor = Color.FromRgba(255, 255, 255, 20);
        }

        /// <summary>
        /// A method for initializing footer
        /// </summary>
        private void InitializeFooter()
        {
            FooterNormal.OnDropdownSourceItemSelected += OnSourceChanged;
            FooterNormal.OnDropdownSortItemSelected += OnSortOptionChanged;
            FooterNormal.OnSelectedOptionIndexChanged += OnOptionSelected;

            FooterDelete.SelecteAllButtonEvent += OnSelectAllClicked;
            FooterDelete.OKButtonEvent += OnOKClicked;
            FooterDelete.CancelButtonEvent += OnCancelClicked;
        }

        /// <summary>
        /// A method for initializing page
        /// </summary>
        protected override void InitializePage()
        {
            if (IsContentReady == false)
            {
                GetInformationsCommand?.Execute("");
                IsContentReady = true;
            }
        }

        /// <summary>
        /// A method for finalizing page
        /// </summary>
        protected override void FinalizePage()
        {
            /*
            ItemsSource.Clear();
            GalleryContentView.Children.Clear();
            BottomButtonList.Clear();
            if (ImageTabStatus == TabStatus.Delete)
            {
                DeleteModeChangeCommand?.Execute("");
                FooterNormal.IsVisible = true;
                FooterDelete.IsVisible = false;
            }
            IsContentReady = false;
            */
        }

        /// <summary>
        /// This method is called when the collection changes
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A NotifyCollectionChanged event's argument</param>
        private void ItemsSourceCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action.ToString().Equals("Add"))
            {
                if (GalleryNoContents.IsVisible)
                {
                    GalleryNoContents.IsVisible = false;
                    ImageTabScrollView.IsVisible = true;
                    FooterNormal.IsFooterEnabled = true;
                }

                var group = e.NewItems[0] as GroupItem;
                ImageGroup galleryGroup = new ImageGroup();

                galleryGroup.PropertyChanged += (se, ev) =>
                {
                    if (ev.PropertyName.Equals("ItemsSource"))
                    {
                        if (galleryGroup.X > 0)
                        {
                            galleryGroup.BottomFocusList.ForEach((item) =>
                            {
                                var key = galleryGroup.X + item.Key;
                                BottomButtonList.Add(new KeyValuePair<double, Button>(key, item.Value));
                            });

                            SetFooterFocusChain(0);
                        }
                    }
                };

                galleryGroup.SetClickCommand(OnClickCommand);
                galleryGroup.ContextPopupItemSelectedHandler += (info, item) =>
                {
                    ShowPopup(info, item);
                };
                GalleryContentView.Children.Add(galleryGroup);
                galleryGroup.BindingContext = group;
                ImageGroupList.Add(galleryGroup);

                galleryGroup.GetTitleFocusArea().Focused += (se, ev) =>
                {
                    ScrollToTarget(galleryGroup.X, SizeUtils.GetWidthSize(300), SizeUtils.GetWidthSize(96));
                };

                galleryGroup.GroupItemFocused += (se, ev) =>
                {
                    ScrollToTarget(ev.PositionX, SizeUtils.GetWidthSize(180), SizeUtils.GetWidthSize(96));
                };

                if (GalleryContentView.Children.Count > 1)
                {
                    var leftGroup = GalleryContentView.Children[GalleryContentView.Children.Count - 2] as ImageGroup;

                    leftGroup.GetTitleFocusArea().On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(galleryGroup.GetTitleFocusArea());
                    galleryGroup.GetTitleFocusArea().On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(leftGroup.GetTitleFocusArea());

                    var leftList = leftGroup.RightFocusList;
                    var rightList = galleryGroup.LeftFocusList;
                    var minCount = leftList.Count > rightList.Count ? rightList.Count : leftList.Count;

                    for (var i = 0; i < minCount; i++)
                    {
                        leftList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(rightList[i]);
                        rightList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(leftList[i]);
                    }
                    if (leftList.Count != rightList.Count)
                    {

                        if (leftList.Count > rightList.Count)
                        {
                            for (var i = rightList.Count; i < leftList.Count; i++)
                            {
                                leftList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(rightList[rightList.Count - 1]);
                            }
                        }
                        else
                        {
                            for (var i = leftList.Count; i < rightList.Count; i++)
                            {
                                rightList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(leftList[leftList.Count - 1]);
                            }
                        }
                    }
                }
            }
            else if (e.Action.ToString().Equals("Reset"))
            {
                GalleryContentView.Children.Clear();
                ImageGroupList.Clear();
                if (!GalleryNoContents.IsVisible)
                {
                    ImageTabScrollView.IsVisible = false;
                    GalleryNoContents.IsVisible = true;
                    FooterNormal.IsFooterEnabled = false;
                }
            }
        }

        /// <summary>
        /// A method for setting footer's focus chain
        /// </summary>
        /// <param name="scrollX">The current X position of the scroll</param>
        private void SetFooterFocusChain(double scrollX)
        {
            FooterNormal.SetFocusChain();
            FooterDelete.SetFocusChain();
            var Padding = SizeUtils.GetWidthSize(96);
            var ItemWidth = SizeUtils.GetWidthSize(180);

            var FocusableBoundFrom = scrollX;
            var FocusableBoundTo = scrollX + SizeUtils.BaseScreenWidth - ItemWidth;
            var list = BottomButtonList.FindAll((pair) =>
            {
                Console.WriteLine("Pair.key " + pair.Key);
                return (pair.Key >= FocusableBoundFrom && pair.Key <= FocusableBoundTo);
            });

            Console.WriteLine("{0} - {1}", FocusableBoundFrom, FocusableBoundTo);

            for (var buttonIndex = 0; buttonIndex < list.Count; buttonIndex++)
            {
                var btn = list[buttonIndex].Value;
                if (buttonIndex == 0)
                {
                    if (ImageTabStatus == TabStatus.Delete)
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterDelete.GetSelectAllButton());
                        TizenVisualElement.SetNextFocusUpView(FooterDelete.GetSelectAllButton(), btn);
                    }
                    else
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterNormal.DropdownSource);
                        TizenVisualElement.SetNextFocusUpView(FooterNormal.DropdownSource, btn);
                    }
                }
                else if (buttonIndex == list.Count - 1)
                {
                    if (ImageTabStatus == TabStatus.Delete)
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterDelete.GetCancelButton());
                        TizenVisualElement.SetNextFocusUpView(FooterDelete.GetCancelButton(), btn);
                    }
                    else
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterNormal.ButtonOption);
                        TizenVisualElement.SetNextFocusUpView(FooterNormal.ButtonOption, btn);
                    }
                }
                else
                {
                    if (ImageTabStatus == TabStatus.Delete)
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterDelete.GetOkButton());
                        TizenVisualElement.SetNextFocusUpView(FooterDelete.GetOkButton(), btn);
                    }
                    else
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterNormal.DropdownSort);
                        TizenVisualElement.SetNextFocusUpView(FooterNormal.DropdownSort, btn);
                    }
                }
            }
        }

        /// <summary>
        /// Moves the scroller to the given target
        /// </summary>
        /// <param name="targetX">The given target</param>
        /// <param name="targetWidth">The width of the given target</param>
        /// <param name="padding">The padding of the given target</param>
        private async void ScrollToTarget(double targetX, int targetWidth, int padding)
        {
            var ScrollX = ImageTabScrollView.ScrollX;

            var ScrollBoundFrom = targetX - (double)(SizeUtils.BaseScreenWidth - targetWidth - padding);
            ScrollBoundFrom = ScrollBoundFrom > 0 ? ScrollBoundFrom : 0;
            var ScrollBoundTo = targetX - (double)padding;
            ScrollBoundTo = ScrollBoundTo > 0 ? ScrollBoundTo : 0;

            // Is Focusable.
            if (ScrollX >= ScrollBoundFrom && ScrollX <= ScrollBoundTo)
            {
                return;
            }

            if (Math.Abs(ScrollX - ScrollBoundFrom) > Math.Abs(ScrollX - ScrollBoundTo))
            {
                SetFooterFocusChain(ScrollBoundTo);
                await ImageTabScrollView.ScrollToAsync(ScrollBoundTo, 0, true);
            }
            else
            {
                SetFooterFocusChain(ScrollBoundFrom);
                await ImageTabScrollView.ScrollToAsync(ScrollBoundFrom, 0, true);
            }
        }

        /// <summary>
        /// This method is called when cancel button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void OnCancelClicked(object sender, EventArgs e)
        {
            DeleteModeChangeCommand?.Execute("");
            FooterNormal.IsVisible = true;
            FooterDelete.IsVisible = false;
            SetFooterFocusChain(ImageTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when SelectAll button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void OnSelectAllClicked(object sender, EventArgs e)
        {
            IsSelectedAll = FooterDelete.IsSelectedAll;
            SelectAllContentCommand?.Execute(IsSelectedAll);

            foreach (var group in ImageGroupList)
            {
                foreach (var item in group.ImageItemList)
                {
                    item.UpdateViewSelectAll(IsSelectedAll);
                }
            }
        }

        /// <summary>
        /// This method is called when OK button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private async void OnOKClicked(object sender, EventArgs e)
        {
            string title = "Delete";

            if (SelectedCount < 1)
            {
                return;
            }
            else if (SelectedCount > 1)
            {
                title = "Delete Selected";

                if (IsSelectedAll)
                {
                    title = "Delete All";
                }
            }

            bool answer = await DisplayAlert(title, "Do you want to delete?", "YES", "NO");

            if (answer)
            {
                DeleteContentCommand?.Execute(null);
                GetInformationsCommand?.Execute("");
            }

            DeleteModeChangeCommand?.Execute("");
            FooterNormal.IsVisible = true;
            FooterDelete.IsVisible = false;
            SetFooterFocusChain(ImageTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when Source option is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A SelectedItemChanged event's argument</param>
        private void OnSourceChanged(object sender, SelectedItemChangedEventArgs e)
        {
            String storageName = e.SelectedItem as String;

            BottomButtonList.Clear();
            ChangeSourceCommand?.Execute(storageName);
            SetFooterFocusChain(ImageTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when Sort option is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A SelectedItemChanged event's argument</param>
        private void OnSortOptionChanged(object sender, SelectedItemChangedEventArgs e)
        {
            BottomButtonList.Clear();
            ChangeSortOptionCommand?.Execute(e.SelectedItem);
            SetFooterFocusChain(ImageTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when option menu is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A SelectedItemChanged event's argument</param>
        private void OnOptionSelected(object sender, ContextPopupSelectedEventArgs e)
        {
            var label = e.Item.Label.Trim().ToLower();
            if (label.Equals("delete"))
            {
                DeleteModeChangeCommand?.Execute("");
                FooterNormal.IsVisible = false;
                FooterDelete.IsVisible = true;
                SetFooterFocusChain(ImageTabScrollView.ScrollX);
            }
            else if (label.Equals("slide show"))
            {
                Program.TransitionTo(new ImageViewerSlideShow());
            }
            else if (label.Equals("disk info"))
            {
                ShowDiskInfoPopup();
            }
        }

        /// <summary>
        /// A method for running hide animation
        /// </summary>
#pragma warning disable CS1998 // Async method lacks 'await' operators and will run synchronously
        public override async void RunHideAnimation()
#pragma warning restore CS1998 // Async method lacks 'await' operators and will run synchronously
        {
            var h12 = SizeUtils.GetHeightSize(12);
#pragma warning disable CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
            GalleryContentView.TranslateTo(0, h12, 667);
#pragma warning restore CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
        }

        /// <summary>
        /// A method for running show animation
        /// </summary>
#pragma warning disable CS1998 // Async method lacks 'await' operators and will run synchronously
        public override async void RunShowAnimation()
#pragma warning restore CS1998 // Async method lacks 'await' operators and will run synchronously
        {
            var h0 = SizeUtils.GetHeightSize(0);
            var h12 = SizeUtils.GetHeightSize(12);

            GalleryContentView.TranslationY = h12;
            GalleryContentView.Opacity = 0;
#pragma warning disable CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
            GalleryContentView.FadeTo(0.99, 667);
            GalleryContentView.TranslateTo(0, h0, 667);
#pragma warning restore CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
        }

        protected override void OnAppearing()
        {
            base.OnAppearing();
            // add Listener

            App.KeyUpEvent.On += MenuKeyListener;
        }

        protected override void OnDisappearing()
        {
            base.OnDisappearing();
            // remove listener
            App.KeyUpEvent.On -= MenuKeyListener;
        }

        private void MenuKeyListener(object sender, EventArgs e)
        {
            ElmSharp.EcoreKeyEventArgs args = e as ElmSharp.EcoreKeyEventArgs;
            if (args.KeyName.Contains(ElmSharp.EvasKeyEventArgs.PlatformMenuButtonName))
            {
                NotifyMenuKeyPressed();
            }
        }

        private void NotifyMenuKeyPressed()
        {
            if (GalleryContentView.Children.Count > 0)
            {
                foreach (ImageGroup group in GalleryContentView.Children)
                {
                    if (group.NotifyMenuKeyPressedToFocusedItem())
                    {
                        break;
                    }
                }

            }
        }

        private async void ShowPopup(MediaInformationEx info, ImageContextPopupItem item)
        {
            switch (item)
            {
                case ImageContextPopupItem.FileInfo:
                    string message = "File name  : " + info.MediaContentInformation.Title + "\n";
                    message += "File size : " + info.MediaContentInformation.FileSize + "\n";
                    message += "File format : " + info.MediaContentInformation.MimeType + "\n";
                    message += "File size : " + info.MediaContentInformation.FileSize + "\n";
                    message += "File path : " + info.MediaContentInformation.Path;

                    await DisplayAlert("File Information", message, "Close");
                    Dialog dialog = new Dialog();
                    break;
                case ImageContextPopupItem.Delete:
                    bool answer = await DisplayAlert("Delete", "Delete '" + info.MediaContentInformation.Title + "'?", "Yes", "No");
                    if (answer)
                    {
                        DeleteContentCommand?.Execute(info);
                        GetInformationsCommand?.Execute("");
                    }

                    break;
            }

        }

        /// <summary>
        /// A method for showing Disk Information Popup
        /// </summary>
        private async void ShowDiskInfoPopup()
        {
            int numberOfFiles = 0;

            foreach (var item in ImageGroupList)
            {
                numberOfFiles += item.ImageItemList.Count;
            }

            var message = "Number of files  : " + numberOfFiles + "\n";
            await DisplayAlert("Disk Information", message, "Close");
        }
    }
}
