﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Threading;
using System.Windows.Input;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.Controls;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.ViewModels;
using Xamarin.Forms;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;

namespace TVMediaHub.Tizen.Views
{
    using Tizen = Xamarin.Forms.PlatformConfiguration.Tizen;

    /// <summary>
    /// A custom ContentPage for displaying the image viewer
    /// </summary>
    public partial class ImageViewer : ContentPage
    {
        /// <summary>
        /// An enumeration for NaviDirection
        /// </summary>
        enum NaviDirection
        {
            NONE = 0,
            LEFT,
            RIGHT,
        };

        /// <summary>
        /// An enumeration for ControlAreaState
        /// </summary>
        enum ControlAreaState
        {
            SHOW = 0,
            HIDE,
            AUTO,
        };

        /// <summary>
        /// The value for the dim effect
        /// </summary>
        private static int gradientDimEffectTime = 667;

        /// <summary>
        /// The value for the dicrention of the navigation key
        /// </summary>
        NaviDirection naviDirection = default(NaviDirection);

        /// <summary>
        /// A flag whether rotation animation is running or not
        /// </summary>
        private bool isRotationRunning = false;

        /// <summary>
        /// A flag whether option popup is showing
        /// </summary>
        private bool isOptionPopupShowing = false;

        /// <summary>
        /// A flag whether speed option popup is showing
        /// </summary>
        private bool isSpeedOptionPopupShowing = false;

        /// <summary>
        /// A flag whether effect option popup is showing
        /// </summary>
        private bool isEffectOptionPopupShowing = false;

        /// <summary>
        /// A flag whether image viewer is first launching
        /// </summary>
        private bool isFirstLaunching = true;

        /// <summary>
        /// A flag whether navigation is running
        /// </summary>
        private bool isNaviRunning = false;

        /// <summary>
        /// An instance for the option popup
        /// </summary>
        private ContextPopup optionPopup = null;

        /// <summary>
        /// An instance for the speed option popup
        /// </summary>
        private ContextPopup speedOptionPopup = null;

        /// <summary>
        /// An instance for the effect option popup
        /// </summary>
        private ContextPopup effectOptionPopup = null;

        /// <summary>
        /// An instance for the previous content
        /// </summary>
        private ImageViewerInfomation prevContent = null;

        /// <summary>
        /// A key name list of the remote controller
        /// </summary>
        private string[] remoteKeys = new string[] { "Up", "Down", "Left", "Right", "XF86Menu" };

        /// <summary>
        /// The timer to handle the control area
        /// </summary>
        Timer timer;

        /// <summary>
        /// The due time of the timer
        /// 5 second
        /// </summary>
        private int timerDueTime = 5000;

        /// <summary>
        /// Identifies the SavedRotationCommand bindable property
        /// </summary>
        public static readonly BindableProperty SavedRotationCommandProperty = BindableProperty.Create("SavedRotationCommand", typeof(ICommand), typeof(ImageViewer), default(ICommand));

        /// <summary>
        /// Gets or sets SavedRotationCommand
        /// </summary>
        public ICommand SavedRotationCommand
        {
            get { return (ICommand)GetValue(SavedRotationCommandProperty); }
            set { SetValue(SavedRotationCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the NavigationCommand bindable property
        /// </summary>
        public static readonly BindableProperty NavigationCommandProperty = BindableProperty.Create("NavigationCommand", typeof(ICommand), typeof(ImageViewer), default(ICommand));

        /// <summary>
        /// Gets or sets NavigationCommand
        /// </summary>
        public ICommand NavigationCommand
        {
            get { return (ICommand)GetValue(NavigationCommandProperty); }
            set { SetValue(NavigationCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the SetSlideShowSpeedCommand bindable property
        /// </summary>
        public static readonly BindableProperty SetSlideShowSpeedCommandProperty = BindableProperty.Create("SetSlideShowSpeedCommand", typeof(ICommand), typeof(ImageViewer), default(ICommand));

        /// <summary>
        /// Gets or sets SetSlideShowSpeedCommand
        /// </summary>
        public ICommand SetSlideShowSpeedCommand
        {
            get { return (ICommand)GetValue(SetSlideShowSpeedCommandProperty); }
            set { SetValue(SetSlideShowSpeedCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the SetSlideShowEffectCommand bindable property
        /// </summary>
        public static readonly BindableProperty SetSlideShowEffectCommandProperty = BindableProperty.Create("SetSlideShowEffectCommand", typeof(ICommand), typeof(ImageViewer), default(ICommand));

        /// <summary>
        /// Gets or sets SetSlideShowEffectCommand
        /// </summary>
        public ICommand SetSlideShowEffectCommand
        {
            get { return (ICommand)GetValue(SetSlideShowEffectCommandProperty); }
            set { SetValue(SetSlideShowEffectCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the CurrentImage bindable property
        /// </summary>
        public static readonly BindableProperty CurrentImageProperty = BindableProperty.Create("CurrentImage", typeof(ImageViewerInfomation), typeof(ImageViewer), null);

        /// <summary>
        /// Gets or sets CurrentImage
        /// </summary>
        public ImageViewerInfomation CurrentImage
        {
            get { return (ImageViewerInfomation)GetValue(CurrentImageProperty); }
            set { SetValue(CurrentImageProperty, value); }
        }

        /// <summary>
        /// An event handler for BackButton pressed event
        /// </summary>
        private EventHandler OnBackButtonPressedHandler { get; set; }

        /// <summary>
        /// A constructor
        /// </summary>
        public ImageViewer()
        {
            BindingContext = ImageTabViewModelLocator.ViewModel;
            InitializeComponent();
            InitializeSize();

            Xamarin.Forms.NavigationPage.SetHasNavigationBar(this, false);

            SetSlideShowSpeedCommand?.Execute(SlideShowSpeed.Normal);
            SetSlideShowEffectCommand?.Execute(SlideShowEffect.Dissolve);
            SavedRotationCommand?.Execute(Convert.ToDouble(-1));

            InitializeCurrentImage();

            InitializeButtons();

            timer = new Timer(HideControlAreaHandler, null, Timeout.Infinite, Timeout.Infinite);

            PropertyChanged += ImageViewerPropertyChanged;

            OnBackButtonPressedHandler += async (s, e) =>
            {
                if (isOptionPopupShowing)
                {
                    optionPopup.Dismiss();

                    return;
                }

                if (isSpeedOptionPopupShowing)
                {
                    speedOptionPopup.Dismiss();

                    return;
                }

                if (isEffectOptionPopupShowing)
                {
                    effectOptionPopup.Dismiss();

                    return;
                }

                isFirstLaunching = true;

#pragma warning disable CS4014
                GradientDim.FadeTo(0, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
                ControlArea.TranslateTo(0, 93, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
                ControlArea.FadeTo(0, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
                TitleContainer.TranslateTo(0, -50, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
                await TitleContainer.FadeTo(0, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);

                Program.TransitionToMain(1);
#pragma warning restore CS4014
            };
        }

        /// <summary>
        /// Grabs the key of the remote controller
        /// </summary>
        private void GrabRemoteKeys()
        {
            App.KeyDownEvent.On += RemoteKeyDownListener;
        }

        /// <summary>
        /// Ungrabs the key of the remote controller
        /// </summary>
        private void UngrabRemoteKeys()
        {
            App.KeyDownEvent.On -= RemoteKeyDownListener;
        }

        /// <summary>
        /// KeyDown will be triggered when key is pressed down
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">The information about the event</param>
        private void RemoteKeyDownListener(object sender, EventArgs e)
        {
            SetControlAreaState(ControlAreaState.AUTO);
        }

        /// <summary>
        /// A method for changing control area according to ControlAreaState
        /// </summary>
        /// <param name="state">A current ControlAreaState</param>
        private void SetControlAreaState(ControlAreaState state)
        {
            switch (state)
            {
                case ControlAreaState.SHOW:
                    if (ControlArea.Opacity == 0)
                    {
                        RunShowAnimation();
                    }

                    timer.Change(Timeout.Infinite, Timeout.Infinite);

                    break;
                case ControlAreaState.HIDE:
                    if (ControlArea.Opacity == 1)
                    {
                        RunHideAnimation();
                    }

                    timer.Change(Timeout.Infinite, Timeout.Infinite);

                    break;
                case ControlAreaState.AUTO:
                default:
                    if (ControlArea.Opacity == 0)
                    {
                        RunShowAnimation();
                    }

                    timer.Change(timerDueTime, Timeout.Infinite);

                    break;
            }
        }

        /// <summary>
        /// A method for initializing current image's fields
        /// </summary>
        private void InitializeCurrentImage()
        {
            Thumb1.Source = CurrentImage.FilePath;
            Thumb1.WidthRequest = App.MainWindow.Geometry.Width;
            Thumb1.HeightRequest = App.MainWindow.Geometry.Height;
            Thumb1.TranslationX = App.MainWindow.Geometry.X;
            Thumb1.TranslationY = App.MainWindow.Geometry.Y;
            Thumb1.Rotation = CurrentImage.SavedRotation;

            Thumb0.WidthRequest = App.MainWindow.Geometry.Width;
            Thumb0.HeightRequest = App.MainWindow.Geometry.Height;
            Thumb0.TranslationX = -App.MainWindow.Geometry.Width;
            Thumb0.TranslationY = App.MainWindow.Geometry.Y;

            prevContent = new ImageViewerInfomation()
            {
                Title = CurrentImage?.Title,
                FilePath = CurrentImage?.FilePath,
                Date = CurrentImage?.Date,
                SavedRotation = CurrentImage.SavedRotation,
            };
        }

        /// <summary>
        /// A method for initializing several sizes
        /// </summary>
        private void InitializeSize()
        {
            int titleSize = SizeUtils.GetFontSize(44);
            int dateSize = SizeUtils.GetFontSize(26);
            TitleLabel.FontSize = titleSize;
            DateLabel.FontSize = dateSize;
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private async void ImageViewerPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("CurrentImage"))
            {
                int currentAngle = 0;

                Xamarin.Forms.Image visibleImage = GetVisibleImage();
                if (visibleImage == null)
                {
                    return;
                }

                Xamarin.Forms.Image unVisibleImage = GetUnVisibleImage();
                if (unVisibleImage == null)
                {
                    return;
                }

                if (naviDirection.Equals(NaviDirection.NONE))
                {
                    currentAngle = Convert.ToInt32(CurrentImage.SavedRotation);
                    visibleImage.Source = CurrentImage.FilePath;
                    visibleImage.Scale = (currentAngle == 90 || currentAngle == 270) ? (double)((double)App.MainWindow.Geometry.Height / (double)App.MainWindow.Geometry.Width) : 1;
                    visibleImage.Rotation = CurrentImage.SavedRotation;

                    prevContent = CurrentImage;

                    return;
                }

                isNaviRunning = true;

                visibleImage.Source = prevContent.FilePath;
                visibleImage.Scale = (visibleImage.Rotation == 90 || visibleImage.Rotation == 270) ? (double)((double)App.MainWindow.Geometry.Height / (double)App.MainWindow.Geometry.Width) : 1;

                currentAngle = Convert.ToInt32(CurrentImage.SavedRotation);
                unVisibleImage.Source = CurrentImage.FilePath;
                unVisibleImage.Scale = (currentAngle == 90 || currentAngle == 270) ? (double)((double)App.MainWindow.Geometry.Height / (double)App.MainWindow.Geometry.Width) : 1;
                unVisibleImage.Rotation = CurrentImage.SavedRotation;

                if (naviDirection.Equals(NaviDirection.LEFT))
                {
                    unVisibleImage.TranslationX = -App.MainWindow.Geometry.Width;

#pragma warning disable CS4014
                    visibleImage.TranslateTo((App.MainWindow.Geometry.X + App.MainWindow.Geometry.Width), App.MainWindow.Geometry.Y, 500, Easing.CubicInOut);
#pragma warning restore CS4014
                    await unVisibleImage.TranslateTo(App.MainWindow.Geometry.X, App.MainWindow.Geometry.Y, 500, Easing.CubicInOut);
                }
                else if (naviDirection.Equals(NaviDirection.RIGHT))
                {
                    unVisibleImage.TranslationX = (App.MainWindow.Geometry.X + App.MainWindow.Geometry.Width);

#pragma warning disable CS4014
                    visibleImage.TranslateTo((App.MainWindow.Geometry.X - App.MainWindow.Geometry.Width), App.MainWindow.Geometry.Y, 500, Easing.CubicInOut);
#pragma warning restore CS4014
                    await unVisibleImage.TranslateTo(App.MainWindow.Geometry.X, App.MainWindow.Geometry.Y, 500, Easing.CubicInOut);
                }

                naviDirection = NaviDirection.NONE;
                isNaviRunning = false;

                prevContent = CurrentImage;
            }
        }

        /// <summary>
        /// A method for getting visible image
        /// </summary>
        /// <returns>The image to be seen</returns>
        private Xamarin.Forms.Image GetVisibleImage()
        {
            Xamarin.Forms.Image visibleImage = null;

            if (Thumb0.TranslationX == 0)
            {
                visibleImage = Thumb0;
            }
            else if (Thumb1.TranslationX == 0)
            {
                visibleImage = Thumb1;
            }
            else
            {
                DbgPort.E("Failed to get the visible image");

                return null;
            }

            return visibleImage;
        }

        /// <summary>
        /// A method for getting invisible image
        /// </summary>
        /// <returns>The image to be unseen</returns>
        private Xamarin.Forms.Image GetUnVisibleImage()
        {
            Xamarin.Forms.Image unVisibleImage = null;

            if (Thumb0.TranslationX == 0)
            {
                unVisibleImage = Thumb1;
            }
            else if (Thumb1.TranslationX == 0)
            {
                unVisibleImage = Thumb0;
            }
            else
            {
                DbgPort.E("Failed to get the unvisible image");

                return null;
            }

            return unVisibleImage;
        }

        /// <summary>
        /// A method for initializing button of ImageViewer
        /// </summary>
        private void InitializeButtons()
        {
            MediaHubButton[] buttons = new MediaHubButton[6] { RotateBtn, ZoomBtn, LeftBtn, RightBtn, SlideShowBtn, OptionBtn };
            string[] icons = new string[6] { "rotate", "zoom", "left", "right", "video", "option" };
            string iconPath = "btn_viewer_control_rotate_normal.png";

            for (int i = 0; i < buttons.Length; i++)
            {
                buttons[i].NormalImage = iconPath.Replace("rotate", icons[i]);
                buttons[i].PressedImage = buttons[i].NormalImage.Replace("normal", "pressed");
                buttons[i].MediaHubButtonIcon = buttons[i].NormalImage;
            }
        }

        /// <summary>
        /// This method is called when Rotate button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private async void RotateBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            if (isRotationRunning)
            {
                return;
            }

            Xamarin.Forms.Image currentContent = GetVisibleImage();
            if (currentContent == null)
            {
                return;
            }

            int duration = 350;
            int rotateAngle = 90;
            int currentAngle = Convert.ToInt32(currentContent.Rotation);

            isRotationRunning = true;

#pragma warning disable CS4014
            currentContent.ScaleTo((((currentAngle + rotateAngle) == 90 || (currentAngle + rotateAngle) == 270) ? (double)((double)currentContent.Height / (double)currentContent.Width) : 1), Convert.ToUInt32(duration), Easing.CubicInOut);
#pragma warning restore CS4014

            await currentContent.RotateTo(currentContent.Rotation + rotateAngle, Convert.ToUInt32(duration), Easing.CubicInOut);

            currentContent.Rotation = (currentContent.Rotation == 360) ? 0 : currentContent.Rotation;
            SavedRotationCommand?.Execute(currentContent.Rotation);
            isRotationRunning = false;
        }

        /// <summary>
        /// This method is called when Zoom button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void ZoomBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            Program.TransitionTo(new ImageViewerZoom());
        }

        /// <summary>
        /// This method is called when Left button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void LeftBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            if (isNaviRunning)
            {
                return;
            }

            naviDirection = NaviDirection.LEFT;

            NavigationCommand?.Execute("Left");
        }

        /// <summary>
        /// This method is called when Right button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void RightBtnClicked(object sender, EventArgs e)
        {

            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            if (isNaviRunning)
            {
                return;
            }

            naviDirection = NaviDirection.RIGHT;

            NavigationCommand?.Execute("Right");
        }

        /// <summary>
        /// This method is called when Slide show button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void SlideShowBtnClicked(object sender, EventArgs e)
        {

            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            Program.TransitionTo(new ImageViewerSlideShow());
        }

        /// <summary>
        /// This method is called when Option button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void OptionBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);
            CreateOptionPopup();
        }

        /// <summary>
        /// A method for creating option context popup
        /// </summary>
        private void CreateOptionPopup()
        {
            if (isOptionPopupShowing)
            {
                return;
            }

            optionPopup = new ContextPopup
            {
                IsAutoHidingEnabled = true,
                DirectionPriorities = new ContextPopupDirectionPriorities(ContextPopupDirection.Up, ContextPopupDirection.Right, ContextPopupDirection.Left, ContextPopupDirection.Down),
            };

            optionPopup.Items.Add(new ContextPopupItem("  SLIDE SHOW SPEED  "));
            optionPopup.Items.Add(new ContextPopupItem("  SLIDE SHOW EFFECT  "));

            optionPopup.SelectedIndexChanged += (s, args) =>
            {
                var ctxPopup = s as ContextPopup;

                DbgPort.D("[ImageViewer] Selected Item : " + (ctxPopup.SelectedItem as ContextPopupItem).Label + " (" + ctxPopup.SelectedIndex + ")");
                switch (ctxPopup.SelectedIndex)
                {
                    case 0:
                        /// Slide Show Speed
                        CreateSlideShowSpeedOptionPopup();
                        break;
                    case 1:
                        /// Slide Show Effect
                        CreateSlideShowEffectOptionPopup();
                        break;
                    default:
                        break;
                }

                optionPopup.Dismiss();
                optionPopup = null;
            };

            optionPopup.Dismissed += (s, args) =>
            {
                isOptionPopupShowing = false;

                if (!isSpeedOptionPopupShowing && !isEffectOptionPopupShowing)
                {
                    SetControlAreaState(ControlAreaState.AUTO);
                }
            };

            optionPopup.Show(PopupAnchor, (PopupAnchor.Width / 2), PopupAnchor.Height);
            isOptionPopupShowing = true;

            SetControlAreaState(ControlAreaState.SHOW);
        }

        /// <summary>
        /// A method for creating slide show speed option context popup
        /// </summary>
        private void CreateSlideShowSpeedOptionPopup()
        {
            if (isSpeedOptionPopupShowing)
            {
                return;
            }

            speedOptionPopup = new ContextPopup
            {
                IsAutoHidingEnabled = true,
                DirectionPriorities = new ContextPopupDirectionPriorities(ContextPopupDirection.Up, ContextPopupDirection.Right, ContextPopupDirection.Left, ContextPopupDirection.Down),
            };

            speedOptionPopup.Items.Add(new ContextPopupItem("              NORMAL             "));
            speedOptionPopup.Items.Add(new ContextPopupItem("              SLOW              "));
            speedOptionPopup.Items.Add(new ContextPopupItem("              FAST              "));

            speedOptionPopup.SelectedIndexChanged += (s, e) =>
            {
                var ctxPopup = s as ContextPopup;

                DbgPort.D("[SlideShowSpeed] Selected Item : " + (ctxPopup.SelectedItem as ContextPopupItem).Label + "(" + ctxPopup.SelectedIndex + ")");

                switch (ctxPopup.SelectedIndex)
                {
                    case 0:
                        /// Normal
                        SetSlideShowSpeedCommand?.Execute(SlideShowSpeed.Normal);
                        break;
                    case 1:
                        /// Slow
                        SetSlideShowSpeedCommand?.Execute(SlideShowSpeed.Slow);
                        break;
                    case 2:
                        /// Fast
                        SetSlideShowSpeedCommand?.Execute(SlideShowSpeed.Fast);
                        break;
                    default:
                        break;
                }

                speedOptionPopup.Dismiss();
                speedOptionPopup = null;
            };

            speedOptionPopup.Dismissed += (s, args) =>
            {
                isSpeedOptionPopupShowing = false;

                CreateOptionPopup();
            };

            speedOptionPopup.Show(PopupAnchor, (PopupAnchor.Width / 2), PopupAnchor.Height);
            isSpeedOptionPopupShowing = true;
        }

        /// <summary>
        /// A method for creating slide show effect option context popup
        /// </summary>
        private void CreateSlideShowEffectOptionPopup()
        {
            if (isEffectOptionPopupShowing)
            {
                return;
            }

            effectOptionPopup = new ContextPopup
            {
                IsAutoHidingEnabled = true,
                DirectionPriorities = new ContextPopupDirectionPriorities(ContextPopupDirection.Up, ContextPopupDirection.Right, ContextPopupDirection.Left, ContextPopupDirection.Down),
            };

            effectOptionPopup.Items.Add(new ContextPopupItem("            DISSOLVE            "));
            effectOptionPopup.Items.Add(new ContextPopupItem("            FADE OUT            "));

            effectOptionPopup.SelectedIndexChanged += (s, e) =>
            {
                var ctxPopup = s as ContextPopup;

                DbgPort.D("[SlideShowEffect] Selected Item : " + (ctxPopup.SelectedItem as ContextPopupItem).Label + "(" + ctxPopup.SelectedIndex + ")");

                switch (ctxPopup.SelectedIndex)
                {
                    case 0:
                        /// Dissolve
                        SetSlideShowEffectCommand?.Execute(SlideShowEffect.Dissolve);
                        break;
                    case 1:
                        /// Fade out
                        SetSlideShowEffectCommand?.Execute(SlideShowEffect.FadeOut);
                        break;
                    default:
                        break;
                }

                effectOptionPopup.Dismiss();
                effectOptionPopup = null;
            };

            effectOptionPopup.Dismissed += (s, args) =>
            {
                isEffectOptionPopupShowing = false;

                CreateOptionPopup();

            };

            effectOptionPopup.Show(PopupAnchor, (PopupAnchor.Width / 2), PopupAnchor.Height);
            isEffectOptionPopupShowing = true;
        }

        /// <summary>
        /// Positions and sizes the children of a Layout.
        /// </summary>
        /// <param name="x">A value representing the x coordinate of the child region bounding box.</param>
        /// <param name="y">A value representing the y coordinate of the child region bounding box.</param>
        /// <param name="width">A value representing the width of the child region bounding box.</param>
        /// <param name="height">A value representing the height of the child region bounding box.</param>
        protected override void LayoutChildren(double x, double y, double width, double height)
        {
            base.LayoutChildren(x, y, width, height);
            GradientDim.HeightRequest = height;
            GradientDim.WidthRequest = width;
        }

        /// <summary>
        /// This method is called when the page becoming visible.
        /// </summary>
        protected override void OnAppearing()
        {
            base.OnAppearing();

            RotateBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(RotateBtn.MediaHubButtonBtn);
            RotateBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(ZoomBtn.MediaHubButtonBtn);

            ZoomBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(RotateBtn.MediaHubButtonBtn);
            ZoomBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(LeftBtn.MediaHubButtonBtn);

            LeftBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(ZoomBtn.MediaHubButtonBtn);
            LeftBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(RightBtn.MediaHubButtonBtn);

            RightBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(LeftBtn.MediaHubButtonBtn);
            RightBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(SlideShowBtn.MediaHubButtonBtn);

            SlideShowBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(RightBtn.MediaHubButtonBtn);
            SlideShowBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(OptionBtn.MediaHubButtonBtn);

            OptionBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(SlideShowBtn.MediaHubButtonBtn);
            OptionBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(OptionBtn.MediaHubButtonBtn);

            RunShowAnimation();
            timer.Change(timerDueTime, Timeout.Infinite);

            if (isFirstLaunching)
            {
                RotateBtn.MediaHubButtonBtn.Focus();
                isFirstLaunching = false;
            }

            GrabRemoteKeys();
        }

        /// <summary>
        /// This method is callend when the page is disappeared.
        /// </summary>
        protected override void OnDisappearing()
        {
            base.OnDisappearing();

            GradientDim.Opacity = 0;
            TitleContainer.TranslationY = -50;
            TitleContainer.Opacity = 0;
            ControlArea.TranslationY = 93;
            ControlArea.Opacity = 0;

            timer.Change(Timeout.Infinite, Timeout.Infinite);

            UngrabRemoteKeys();
        }

        /// <summary>
        /// A method for hiding control area when timer is over
        /// </summary>
        /// <param name="state">The state to be changed</param>
        private void HideControlAreaHandler(object state)
        {
            SetControlAreaState(ControlAreaState.HIDE);
        }

        /// <summary>
        /// This method is called when Back button is pressed
        /// </summary>
        /// <returns>Always returns true</returns>
        protected override bool OnBackButtonPressed()
        {
            if (OnBackButtonPressedHandler != null)
            {
                OnBackButtonPressedHandler.Invoke(this, EventArgs.Empty);
                return true;
            }

            return true;
        }

        /// <summary>
        /// A method for running hide animation
        /// </summary>
        private void RunHideAnimation()
        {
            GradientDim.FadeTo(0, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
            ControlArea.TranslateTo(0, 93, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
            ControlArea.FadeTo(0, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
            TitleContainer.TranslateTo(0, -50, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
            TitleContainer.FadeTo(0, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
        }

        /// <summary>
        /// A method for running show animation
        /// </summary>
        private void RunShowAnimation()
        {
            GradientDim.Opacity = 0;
            TitleContainer.TranslationY = -50;
            TitleContainer.Opacity = 0;
            ControlArea.TranslationY = 93;
            ControlArea.Opacity = 0;

            GradientDim.FadeTo(1, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
            TitleContainer.TranslateTo(0, 0, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
            TitleContainer.FadeTo(1, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
            ControlArea.TranslateTo(0, 0, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
            ControlArea.FadeTo(1, Convert.ToUInt32(gradientDimEffectTime), Easing.CubicInOut);
        }
    }
}
