﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using TVMediaHub.Tizen.Controls;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.ViewModels;
using Xamarin.Forms;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom ContentPage for displaying ImageViewerSlideShow page
    /// </summary>
    public partial class ImageViewerSlideShow : ContentPageEx
    {
        /// <summary>
        /// A list of the image information to be shown
        /// </summary>
        private List<ImageViewerInfomation> imageList = new List<ImageViewerInfomation>();

        /// <summary>
        /// A flag whether the back key is pressed or not
        /// </summary>
        private static bool isBackKeyPressed = false;


        /// <summary>
        /// Identifies the ItemsSource bindable property
        /// </summary>
        public static readonly BindableProperty ItemsSourceProperty = BindableProperty.Create("ItemsSource", typeof(IEnumerable<ImageViewerInfomation>), typeof(ImageViewerSlideShow), null);

        /// <summary>
        /// Gets or set ItemSource to display slide show
        /// </summary>
        public IEnumerable<ImageViewerInfomation> ItemsSource
        {
            get { return (IEnumerable<ImageViewerInfomation>)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        /// <summary>
        /// Identifies the Speed bindable property
        /// </summary>
        public static readonly BindableProperty SpeedProperty = BindableProperty.Create("Speed", typeof(SlideShowSpeed), typeof(ImageViewerSlideShow), SlideShowSpeed.Normal);

        /// <summary>
        /// Gets or set Speed
        /// </summary>
        public SlideShowSpeed Speed
        {
            get { return (SlideShowSpeed)GetValue(SpeedProperty); }
            set { SetValue(SpeedProperty, value); }
        }

        /// <summary>
        /// Identifies the Effect bindable property
        /// </summary>
        public static readonly BindableProperty EffectProperty = BindableProperty.Create("Effect", typeof(SlideShowEffect), typeof(ImageViewerSlideShow), SlideShowEffect.Dissolve);

        /// <summary>
        /// Gets or set Effect
        /// </summary>
        public SlideShowEffect Effect
        {
            get { return (SlideShowEffect)GetValue(EffectProperty); }
            set { SetValue(EffectProperty, value); }
        }

        /// <summary>
        /// Identifies the CurrentImage bindable property
        /// </summary>
        public static readonly BindableProperty CurrentImageProperty = BindableProperty.Create("CurrentImage", typeof(ImageViewerInfomation), typeof(ImageViewerSlideShow), null);

        /// <summary>
        /// Gets or set CurrentImage
        /// </summary>
        public ImageViewerInfomation CurrentImage
        {
            get { return (ImageViewerInfomation)GetValue(CurrentImageProperty); }
            set { SetValue(CurrentImageProperty, value); }
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public ImageViewerSlideShow()
        {
            BindingContext = ImageTabViewModelLocator.ViewModel;
            InitializeComponent();

            OnBackButtonPressedHandler += (s, e) =>
            {
                this.AbortAnimation("SlideAnimation");
                isBackKeyPressed = true;
                App.AppMainPage.PopAsync();
                //Program.TransitionTo(new SimpleImageViewer());
            };

            ViewSlideShow();
        }

        /// <summary>
        /// A method for initializing page when the page is appeared
        /// </summary>
        protected override void InitializePage()
        {
            isBackKeyPressed = false;
        }

        /// <summary>
        /// A method for finalizing page when the page is disappeared
        /// </summary>
        protected override void FinalizePage()
        {
            ViewExtensions.CancelAnimations(CurrentThumbnail);
            ViewExtensions.CancelAnimations(NextThumbnail);

            CurrentImage = imageList.Find(x => x.FilePath.Equals((CurrentThumbnail.Source as FileImageSource).File));
            imageList.Clear();
        }

        /// <summary>
        /// A method viewing slide show
        /// </summary>
        private void ViewSlideShow()
        {
            foreach (var item in ItemsSource)
            {
                imageList.Add(item);
            }

            if (imageList.Count > 0)
            {
                if (Effect.Equals(SlideShowEffect.Dissolve))
                {
                    DissolveSlideShow();
                }
                else if (Effect.Equals(SlideShowEffect.FadeOut))
                {
                    FadeoutSlideShow();
                }
            }
        }

        /// <summary>
        /// A method for getting delay time according speed option
        /// </summary>
        /// <returns>A delay time</returns>
        private int GetDelayTimeBySpeedOption()
        {
            int delayTime = 2000;

            switch (Speed)
            {
                case SlideShowSpeed.Slow:
                    delayTime *= 2;
                    break;
                case SlideShowSpeed.Fast:
                    delayTime /= 2;
                    break;
                case SlideShowSpeed.Normal:
                default:
                    break;
            }

            return delayTime;
        }

        /// <summary>
        /// A method for setting several fields for slide show on dissolve mode
        /// </summary>
        private async void DissolveSlideShow()
        {
            int delayTime = GetDelayTimeBySpeedOption();
            uint testDelay = Convert.ToUInt32(delayTime / 2);
            int index = imageList.IndexOf(CurrentImage);

            CurrentThumbnail.Source = imageList[index].FilePath;
            await CurrentThumbnail.FadeTo(1.0, 1000);
            DbgPort.D("" + imageList.Count);
            NextThumbnail.Source = imageList[(index + 1) % imageList.Count].FilePath;
            await NextThumbnail.FadeTo(1.0, 0);
            await Task.Delay(delayTime);
            if (isBackKeyPressed)
            {
                return;
            }

            DissolveSlideShowAnimation(index, testDelay);
        }

        /// <summary>
        /// A method for showing slide show on dissolve mode
        /// </summary>
        /// <param name="index">An index of start image</param>
        /// <param name="delay">A delay time</param>
        private void DissolveSlideShowAnimation(int index, uint delay)
        {
            if (index > imageList.Count)
            {
                App.AppMainPage.PopAsync();

                return;
            }

            DbgPort.D("Step 1");
            this.Animate("SlideAnimation",
                (v) =>
                {
                    CurrentThumbnail.Scale = 1.0 + 0.05 * v;
                },
                length: delay,
                finished: (v, IsCanceled) =>
                {
                    if (!IsCanceled)
                    {
                        DbgPort.D("Step 2");
                        this.Animate("SlideAnimation",
                            (v2) =>
                            {
                                CurrentThumbnail.Scale = 1.05 + 0.05 * v2;
                                CurrentThumbnail.Opacity = 1.0 - 1.0 * v2;
                            },
                            length: delay,
                            finished: (v2, IsCanceled2) =>
                            {
                                if (!IsCanceled2)
                                {
                                    DbgPort.D("Step 3");
                                    CurrentThumbnail.Scale = 1.0;
                                    CurrentThumbnail.Source = imageList[(index + 1) % imageList.Count].FilePath;
                                    this.Animate("SlideAnimation",
                                        (v3) =>
                                        {
                                        },
                                        length: delay / 2,
                                        finished: (v3, IsCanceled3) =>
                                        {
                                            if (!IsCanceled3)
                                            {
                                                DbgPort.D("Step 4");
                                                CurrentThumbnail.Opacity = 1.0;
                                                NextThumbnail.Opacity = 0.0;
                                                NextThumbnail.Source = imageList[(index + 2) % imageList.Count].FilePath;
                                                this.Animate("SlideAnimation",
                                                       (v4) =>
                                                       {
                                                       },
                                                       length: delay / 2,
                                                       finished: (v4, IsCanceled4) =>
                                                       {
                                                           if (!IsCanceled4)
                                                           {
                                                               NextThumbnail.Opacity = 1.0;
                                                               DissolveSlideShowAnimation(index + 1, delay);
                                                           }
                                                       });
                                            }
                                        });
                                }
                            });
                    }
                });
        }

        /// <summary>
        /// A method for setting several fields for slide show on fadeout mode
        /// </summary>
        private async void FadeoutSlideShow()
        {
            int delayTime = GetDelayTimeBySpeedOption();
            int index = imageList.IndexOf(CurrentImage);

            CurrentThumbnail.Source = imageList[(index) % imageList.Count].FilePath;

            for (int i = 1; i < imageList.Count + 1; i++)
            {
                await CurrentThumbnail.FadeTo(1.0, Convert.ToUInt32(delayTime));
                await Task.Delay((delayTime / 4));
                await CurrentThumbnail.FadeTo(0.0, Convert.ToUInt32(delayTime));

                if (isBackKeyPressed)
                {
                    return;
                }

                CurrentThumbnail.Source = imageList[(index + i) % imageList.Count].FilePath;
            }

            App.AppMainPage.PopAsync();
        }

        /// <summary>
        /// A method for running hide animation
        /// </summary>
        public override void RunHideAnimation()
        {
        }

        /// <summary>
        /// A method for running show animation
        /// </summary>
        public override void RunShowAnimation()
        {
        }
    }
}
