﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using TVMediaHub.Tizen.ViewModels;
using Xamarin.Forms;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.Utils;
using System.IO;
using TVMediaHub.Tizen.Controls;
using System.Collections.Generic;
using System;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom ContentPage for the image viewer zoom mode
    /// </summary>
    public partial class ImageViewerZoom : ContentPageEx
    {
        /// <summary>
        /// A list of the move direction
        /// </summary>
        private static List<string> MoveDirection = new List<string> { "Up", "Down", "Left", "Right" };

        /// <summary>
        /// The value to move
        /// </summary>
        private static int clipperMoveGap = 10;

        /// <summary>
        /// The value to zoom in
        /// </summary>
        private static double zoomRatio = 1.5;

        /// <summary>
        /// The value to check the orientation of the current image
        /// </summary>
        private ElmSharp.ImageOrientation imgOrientation = ElmSharp.ImageOrientation.None;

        /// <summary>
        /// The background thumbnail for the mini map
        /// </summary>
        private ElmSharp.Rectangle backgroundThumbnail;

        /// <summary>
        /// The thumbnail image for the mini map
        /// </summary>
        private ElmSharp.Image thumbnailImg;

        /// <summary>
        /// The dim image for the mini map
        /// </summary>
        private ElmSharp.Image thumbnailDim;

        /// <summary>
        /// The clipping image for the mini map
        /// </summary>
        private ElmSharp.Image clippingImg;

        /// <summary>
        /// The clipper for the mini map
        /// </summary>
        private ElmSharp.Rectangle clipper;

        /// <summary>
        /// The clipper border image for the mini map
        /// </summary>
        private ElmSharp.Image clipperBorder;

        /// <summary>
        /// Identifies the SavedRotation bindable property
        /// </summary>
        public static readonly BindableProperty SavedRotationProperty = BindableProperty.Create("SavedRotation", typeof(double), typeof(ImageViewerZoom), default(double));

        /// <summary>
        /// Gets or sets SavedRotation
        /// </summary>
        public double SavedRotation
        {
            get { return (double)GetValue(SavedRotationProperty); }
            set { SetValue(SavedRotationProperty, value); }
        }

        /// <summary>
        /// Identifies the ThumbnailPath bindable property
        /// </summary>
        public static readonly BindableProperty ThumbnailPathProperty = BindableProperty.Create("ThumbnailPath", typeof(string), typeof(ImageViewerZoom), default(string));

        /// <summary>
        /// Gets or sets the ThumbnailPath
        /// </summary>
        public string ThumbnailPath
        {
            get { return (string)GetValue(ThumbnailPathProperty); }
            set { SetValue(ThumbnailPathProperty, value); }
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public ImageViewerZoom()
        {
            BindingContext = ImageTabViewModelLocator.ViewModel;
            InitializeComponent();

            int BackKeyInfoLabelSize = SizeUtils.GetFontSize(28);
            BackKeyInfoLabel.FontSize = BackKeyInfoLabelSize;
            Toast.DisplayText("Press [Back] to Escape from Zoom mode.");

            /// Set the current image
            SetCurrentImage();

            /// Create the mini map
            CreateMiniMap();

            OnBackButtonPressedHandler += (s, e) =>
            {
                App.AppMainPage.Navigation.PopAsync();
                //Program.TransitionTo(new SimpleImageViewer());
            };
        }

        /// <summary>
        /// A method for initializing page when the page is appeared
        /// Grabs move keys of the remote controller
        /// </summary>
        protected override void InitializePage()
        {
            /// Grabs the hardware keys to move the given image
            GrabMoveKeys();

        }

        /// <summary>
        /// A method for finalizing page when the page is disappeared
        /// Ungrabs move keys of the remote controller
        /// Releases resources of the mini map
        /// </summary>
        protected override void FinalizePage()
        {
            UngrabMoveKeys();

            if (backgroundThumbnail != null)
            {
                backgroundThumbnail.Unrealize();
                backgroundThumbnail = null;
            }

            if (thumbnailImg != null)
            {
                thumbnailImg.Unrealize();
                thumbnailImg = null;
            }

            if (thumbnailDim != null)
            {
                thumbnailDim.Unrealize();
                thumbnailDim = null;
            }

            if (clippingImg != null)
            {
                clippingImg.Unrealize();
                clippingImg = null;
            }

            if (clipper != null)
            {
                clipper.Unrealize();
                clipper = null;
            }

            if (clipperBorder != null)
            {
                clipperBorder.Unrealize();
                clipperBorder = null;
            }
        }

        /// <summary>
        /// KeyDown will be triggered when key is pressed down
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">The information about the event</param>
        private void EvtDownOn(object sender, EventArgs e)
        {
            ElmSharp.EcoreKeyEventArgs args = e as ElmSharp.EcoreKeyEventArgs;

            DbgPort.D("[ZoomMode] key down : " + args.KeyName);

            if (!MoveDirection.Contains(args.KeyName))
            {
                return;
            }

            int direction = MoveDirection.FindIndex(x => x.Equals(args.KeyName));
            object[] buttons = new object[4] { UpButton, DownButton, LeftButton, RightButton };
            (buttons[direction] as ZoomNaviButton).OnButtonPressed();
        }

        /// <summary>
        /// KeyUp will be triggered when key is loose
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">The information about the event</param>
        private void EvtUpOn(object sender, EventArgs e)
        {
            ElmSharp.EcoreKeyEventArgs args = e as ElmSharp.EcoreKeyEventArgs;

            DbgPort.D("[ZoomMode] key up : " + args.KeyName);

            if (!MoveDirection.Contains(args.KeyName))
            {
                return;
            }

            int direction = MoveDirection.FindIndex(x => x.Equals(args.KeyName));
            object[] buttons = new object[4] { UpButton, DownButton, LeftButton, RightButton };
            (buttons[direction] as ZoomNaviButton).OnButtonReleased();
        }

        /// <summary>
        /// Grabs move keys of the remote controller to move the given image
        /// </summary>
        private void GrabMoveKeys()
        {
            App.KeyDownEvent.On += EvtDownOn;
            App.KeyUpEvent.On += EvtUpOn;
        }

        /// <summary>
        /// Ungrabs move keys of the remote controller
        /// </summary>
        private void UngrabMoveKeys()
        {
            App.KeyDownEvent.On -= EvtDownOn;
            App.KeyUpEvent.On -= EvtUpOn;
        }

        /// <summary>
        /// Sets the navigation buttons to be normal
        /// </summary>
        private void SetNaviButtonNormal()
        {
            if (clipper == null)
            {
                return;
            }

            if (clipper.Geometry.Y != thumbnailDim.Geometry.Y)
            {
                UpButton.Opacity = 1.0;
            }

            if ((clipper.Geometry.Y + clipper.Geometry.Height) != (thumbnailDim.Geometry.Y + thumbnailDim.Geometry.Height))
            {
                DownButton.Opacity = 1.0;
            }

            if (clipper.Geometry.X != thumbnailDim.Geometry.X)
            {
                LeftButton.Opacity = 1.0;
            }

            if ((clipper.Geometry.X + clipper.Geometry.Width) != (thumbnailDim.Geometry.X + thumbnailDim.Geometry.Width))
            {
                RightButton.Opacity = 1.0;
            }
        }

        /// <summary>
        /// Moves the current image and the clipper of the mini map
        /// </summary>
        /// <param name="direction">The direction to move</param>
        private void MoveCurrentImage(string direction)
        {
            int moveDuration = 350;
            if (direction.Equals("Up"))
            {
                int toY = clipper.Geometry.Y - clipperMoveGap;

                if (toY <= thumbnailDim.Geometry.Y)
                {
                    clipper.Move(clipper.Geometry.X, thumbnailDim.Geometry.Y);
                    CurrentImage.TranslateTo(CurrentImage.TranslationX, CurrentImage.Y, Convert.ToUInt32(moveDuration), Easing.CubicOut);

                    UpButton.Opacity = 0.2;

                    return;
                }

                ElmSharp.EcoreAnimator.AddAnimator(delegate ()
                {
                    clipper.Move(clipper.Geometry.X, clipper.Geometry.Y - 2);
                    if (clipper.Geometry.Y <= toY)
                    {
                        clipper.Move(clipper.Geometry.X, toY);

                        SetNaviButtonNormal();

                        return false;
                    }

                    return true;
                });

                CurrentImage.TranslateTo(CurrentImage.TranslationX, CurrentImage.TranslationY + clipperMoveGap * (CurrentImage.Height / thumbnailDim.Geometry.Height), Convert.ToUInt32(moveDuration), Easing.CubicOut);
            }
            else if (direction.Equals("Down"))
            {
                int toY = clipper.Geometry.Y + clipperMoveGap;

                if ((toY + clipper.Geometry.Height) >= (thumbnailDim.Geometry.Y + thumbnailDim.Geometry.Height))
                {
                    clipper.Move(clipper.Geometry.X, (thumbnailDim.Geometry.Y + thumbnailDim.Geometry.Height) - clipper.Geometry.Height);
                    CurrentImage.TranslateTo(CurrentImage.TranslationX, -(CurrentImage.Height - App.MainWindow.Geometry.Height), Convert.ToUInt32(moveDuration), Easing.CubicOut);

                    DownButton.Opacity = 0.2;

                    return;
                }

                ElmSharp.EcoreAnimator.AddAnimator(delegate ()
                {
                    clipper.Move(clipper.Geometry.X, clipper.Geometry.Y + 2);
                    if (clipper.Geometry.Y >= toY)
                    {
                        clipper.Move(clipper.Geometry.X, toY);

                        SetNaviButtonNormal();

                        return false;
                    }

                    return true;
                });

                CurrentImage.TranslateTo(CurrentImage.TranslationX, CurrentImage.TranslationY - clipperMoveGap * (CurrentImage.Height / thumbnailDim.Geometry.Height), Convert.ToUInt32(moveDuration), Easing.CubicOut);
            }
            else if (direction.Equals("Left"))
            {
                int toX = clipper.Geometry.X - clipperMoveGap;

                if (toX <= thumbnailDim.Geometry.X)
                {
                    clipper.Move(thumbnailDim.Geometry.X, clipper.Geometry.Y);
                    CurrentImage.TranslateTo(CurrentImage.X, CurrentImage.TranslationY, Convert.ToUInt32(moveDuration), Easing.CubicOut);

                    LeftButton.Opacity = 0.2;

                    return;
                }

                ElmSharp.EcoreAnimator.AddAnimator(delegate ()
                {
                    clipper.Move(clipper.Geometry.X - 2, clipper.Geometry.Y);
                    if (clipper.Geometry.X <= toX)
                    {
                        clipper.Move(toX, clipper.Geometry.Y);

                        SetNaviButtonNormal();

                        return false;
                    }

                    return true;
                });

                CurrentImage.TranslateTo(CurrentImage.TranslationX + clipperMoveGap * (CurrentImage.Width / thumbnailDim.Geometry.Width), CurrentImage.TranslationY, Convert.ToUInt32(moveDuration), Easing.CubicOut);
            }
            else if (direction.Equals("Right"))
            {
                int toX = clipper.Geometry.X + clipperMoveGap;
                if ((toX + clipper.Geometry.Width) >= (thumbnailDim.Geometry.X + thumbnailDim.Geometry.Width))
                {
                    clipper.Move((thumbnailDim.Geometry.X + thumbnailDim.Geometry.Width) - clipper.Geometry.Width, clipper.Geometry.Y);
                    CurrentImage.TranslateTo(-(CurrentImage.Width - App.MainWindow.Geometry.Width), CurrentImage.TranslationY, Convert.ToUInt32(moveDuration), Easing.CubicOut);

                    RightButton.Opacity = 0.2;

                    return;
                }

                ElmSharp.EcoreAnimator.AddAnimator(delegate ()
                {
                    clipper.Move(clipper.Geometry.X + 2, clipper.Geometry.Y);
                    if (clipper.Geometry.X >= toX)
                    {
                        clipper.Move(toX, clipper.Geometry.Y);

                        SetNaviButtonNormal();

                        return false;
                    }

                    return true;
                });

                CurrentImage.TranslateTo(CurrentImage.TranslationX - clipperMoveGap * (CurrentImage.Width / thumbnailDim.Geometry.Width), CurrentImage.TranslationY, Convert.ToUInt32(moveDuration), Easing.CubicOut);
            }
            else
            {
                DbgPort.D("Wrong direction : " + direction);
                return;
            }
        }

        /// <summary>
        /// Sets the current image
        /// </summary>
        private void SetCurrentImage()
        {
            CurrentImage.WidthRequest = App.MainWindow.Geometry.Width * zoomRatio;
            CurrentImage.HeightRequest = App.MainWindow.Geometry.Height * zoomRatio;
            CurrentImage.TranslationX = -((App.MainWindow.Geometry.Width * zoomRatio) - App.MainWindow.Geometry.Width) / 2;
            CurrentImage.TranslationY = -((App.MainWindow.Geometry.Height * zoomRatio) - App.MainWindow.Geometry.Height) / 2;

            int currentAngle = Convert.ToInt32(SavedRotation);
            imgOrientation = (ElmSharp.ImageOrientation)Enum.ToObject(typeof(ElmSharp.ImageOrientation), currentAngle / 90);

            CurrentImage.Scale = (currentAngle == 90 || currentAngle == 270) ? (double)((double)App.MainWindow.Geometry.Height / (double)App.MainWindow.Geometry.Width) : 1;
            CurrentImage.Aspect = (currentAngle == 90 || currentAngle == 270) ? Aspect.AspectFill : Aspect.AspectFit;
            CurrentImage.Rotation = SavedRotation;
        }

        /// <summary>
        /// Creates the mini map
        /// </summary>
        private void CreateMiniMap()
        {
            Grid miniMap = new Grid()
            {
                WidthRequest = 640,
                HeightRequest = 430,
                Padding = 0,
            };
            miniMap.RowDefinitions.Add(new RowDefinition { Height = new GridLength(430) });
            miniMap.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(640) });
            ZoomLayout.Children.Add(miniMap,
                Constraint.Constant(App.MainWindow.Geometry.Width - 96 - 480 - 80), /// X
                Constraint.Constant(App.MainWindow.Geometry.Height - 96 - 270 - 80), /// Y
                Constraint.Constant(640), /// Width
                Constraint.Constant(430)); /// Height

            /// Create the shadow of the mini map
            var backgroundShadow = new NinePatch
            {
                Source = "img_zoom_minimap_shadow.9.png",
                WidthRequest = miniMap.Width,
                HeightRequest = miniMap.Height,
                BorderTop = 119,
                BorderBottom = 119,
                BorderLeft = 119,
                BorderRight = 119,
                Aspect = Aspect.Fill,
            };
            miniMap.Children.Add(backgroundShadow, 0, 0);

            Grid thumbnailFrame = new Grid();
            thumbnailFrame.RowDefinitions.Add(new RowDefinition { Height = new GridLength(80) });
            thumbnailFrame.RowDefinitions.Add(new RowDefinition { Height = new GridLength(270) });
            thumbnailFrame.RowDefinitions.Add(new RowDefinition { Height = new GridLength(80) });
            thumbnailFrame.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(80) });
            thumbnailFrame.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(480) });
            thumbnailFrame.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(80) });
            miniMap.Children.Add(thumbnailFrame, 0, 0);

            RelativeLayout thumbnailMap = new RelativeLayout();
            thumbnailFrame.Children.Add(thumbnailMap, 1, 1);

            /// Using Elm-Sharp
            backgroundThumbnail = new ElmSharp.Rectangle(App.MainWindow);
            backgroundThumbnail.Color = new ElmSharp.Color(0, 0, 0, 255);
            backgroundThumbnail.Resize(480, 270);
            backgroundThumbnail.Move(App.MainWindow.Geometry.Width - 96 - backgroundThumbnail.Geometry.Width, App.MainWindow.Geometry.Height - 96 - backgroundThumbnail.Geometry.Height);
            backgroundThumbnail.Show();

            thumbnailImg = new ElmSharp.Image(App.MainWindow);
            thumbnailImg.Load((ThumbnailPath != null) ? ThumbnailPath : (CurrentImage.Source as FileImageSource).File);
            thumbnailImg.Orientation = imgOrientation;
            thumbnailImg.Resize(backgroundThumbnail.Geometry.Width, backgroundThumbnail.Geometry.Height);
            thumbnailImg.Move(backgroundThumbnail.Geometry.X, backgroundThumbnail.Geometry.Y);
            thumbnailImg.Show();

            thumbnailDim = new ElmSharp.Image(App.MainWindow);
            thumbnailDim.Load(Path.Combine(Program.ResourceDirectoryPath, "img_zoom_minimap_dimmed.png"));
            thumbnailDim.Resize(thumbnailImg.Geometry.Width, thumbnailImg.Geometry.Height);
            thumbnailDim.Geometry = thumbnailImg.Geometry;
            thumbnailDim.Moved += (s, e) =>
            {
                thumbnailDim.Geometry = ((ElmSharp.Rectangle)s).Geometry;
            };
            thumbnailDim.Show();

            clippingImg = new ElmSharp.Image(App.MainWindow);
            clippingImg.Load((ThumbnailPath != null) ? ThumbnailPath : (CurrentImage.Source as FileImageSource).File);
            clippingImg.Orientation = imgOrientation;
            clippingImg.Resize(thumbnailImg.Geometry.Width, thumbnailImg.Geometry.Height);
            clippingImg.Geometry = thumbnailImg.Geometry;
            clippingImg.Moved += (s, e) =>
            {
                clippingImg.Geometry = ((ElmSharp.Rectangle)s).Geometry;
            };
            clippingImg.Show();

            clipper = new ElmSharp.Rectangle(App.MainWindow);
            clipper.Color = new ElmSharp.Color(255, 255, 255, 255);
            clipper.Resize((int)(backgroundThumbnail.Geometry.Width / zoomRatio), (int)(backgroundThumbnail.Geometry.Height / zoomRatio));
            clipper.Move(thumbnailDim.Geometry.X + (thumbnailDim.Geometry.Width / 2) - (clipper.Geometry.Width / 2), thumbnailDim.Geometry.Y + (thumbnailDim.Geometry.Height / 2) - (clipper.Geometry.Height / 2));
            clippingImg.SetClip(clipper);
            clipper.Show();

            clipperBorder = new ElmSharp.Image(App.MainWindow)
            {
                IsFixedAspect = false,
            };
            clipperBorder.Load(Path.Combine(Program.ResourceDirectoryPath, "img_zoom_minimap_line.9.png"));
            clipperBorder.Resize(clipper.Geometry.Width, clipper.Geometry.Height);
            clipperBorder.Geometry = clipper.Geometry;
            clipperBorder.SetBorder(clipperBorder.Geometry.Width / 2, clipperBorder.Geometry.Width / 2, clipperBorder.Geometry.Height / 2, clipperBorder.Geometry.Height / 2);
            clipperBorder.Show();

            clipper.Moved += (s, e) =>
            {
                clipperBorder.Move(clipper.Geometry.X, clipper.Geometry.Y);
            };

            object[] buttons = new object[4] { UpButton, DownButton, LeftButton, RightButton };
            foreach (object button in buttons)
            {
                (button as ZoomNaviButton).OnButtonDown += (s, e) =>
                {
                    DbgPort.D("[" + e + "] is pressed");

                    MoveCurrentImage(e as string);
                };
            }
        }

        /// <summary>
        /// Positions and sizes the children of a Layout.
        /// </summary>
        /// <param name="x">A value representing the x coordinate of the child region bounding box.</param>
        /// <param name="y">A value representing the y coordinate of the child region bounding box.</param>
        /// <param name="width">A value representing the width of the child region bounding box.</param>
        /// <param name="height">A value representing the height of the child region bounding box.</param>
        protected override void LayoutChildren(double x, double y, double width, double height)
        {
            base.LayoutChildren(x, y, width, height);
            GradientDim.HeightRequest = height;
            GradientDim.WidthRequest = width;
        }

        /// <summary>
        /// A method for running hide animation
        /// </summary>
        public override void RunHideAnimation()
        {
        }

        /// <summary>
        /// A method for running show animation
        /// </summary>
        public override void RunShowAnimation()
        {
        }
    }
}
