﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Threading;
using System.Threading.Tasks;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.Controls;
using TVMediaHub.Tizen.ViewModels;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom page for displaying main page of TV MediaHub
    /// </summary>
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class MediaHubMainPage : TabbedPage
    {
        public static readonly BindableProperty DisableBackKeyHandlerProperty = BindableProperty.Create("DisableBackKeyHandler", typeof(bool), typeof(MediaHubMainPage), false, BindingMode.OneWayToSource);
        public bool DisableBackKeyHandler
        {
            get { return (bool)GetValue(DisableBackKeyHandlerProperty); }
            set { SetValue(DisableBackKeyHandlerProperty, value); }
        }

        /// <summary>
        /// Identifies the SourceName bindable property
        /// </summary>
        public static readonly BindableProperty SourceNameProperty = BindableProperty.Create("SourceName", typeof(string), typeof(MediaHubMainPage), null, BindingMode.TwoWay);

        /// <summary>
        /// Gets or sets the source name
        /// </summary>
        public string SourceName
        {
            get { return (string)GetValue(SourceNameProperty); }
            set { SetValue(SourceNameProperty, value); }
        }

        /// <summary>
        /// A flag that whether app is loaded or not
        /// </summary>
        private bool IsAppLoaded;
        /// <summary>
        /// A constructor
        /// </summary>
        /// <param name="tabIndex">An index of tab to be set as MainPage</param>
        public MediaHubMainPage(int tabIndex)
        {
            IsAppLoaded = false;
            InitializeComponent();
            BindingContext = new MediaHubMainPageViewModel();
            NavigationPage.SetHasBackButton(this, false);
            NavigationPage.SetHasNavigationBar(this, true);
            CurrentPageChanged += MediaHubMainPageCurrentPageChanged;
            PropertyChanged += MediaHubMainPagePropertyChanged;
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void MediaHubMainPagePropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("SourceName"))
            {
                if (SourceName != null)
                {
                    Toast.DisplayText(SourceName + " Disconnected.");
                    SourceName = null;
                }
            }
        }

        /// <summary>
        /// This method is called when the CurrentPage is changed
        /// </summary>
        /// <param name="sender">The source of event</param>
        /// <param name="e">An event argument</param>
        private void MediaHubMainPageCurrentPageChanged(object sender, EventArgs e)
        {
            var page = CurrentPage as ContentPageEx;
            page.RunShowAnimation();
        }

        /// <summary>
        /// This method is called when the page becoming visible
        /// </summary>
        protected override async void OnAppearing()
        {
            base.OnAppearing();
            var page = CurrentPage as ContentPageEx;
            if (IsAppLoaded == false)
            {
                page.RunShowAnimation();
                IsAppLoaded = true;
            }

            await App.AppMainPage.FadeTo(0.99, 667);
            Focus();
        }

        /// <summary>
        /// This method is called when Back button is pressed
        /// </summary>
        /// <returns>Always returns true</returns>
        protected override bool OnBackButtonPressed()
        {
            if (DisableBackKeyHandler == true)
            {
                return true;
            }

            SynchronizationContext.Current.Post((o) =>
            {
#pragma warning disable CS4014
                OnBackKeyPressedAtMain();
#pragma warning restore CS4014
            }, "");
            return true;
        }

        /// <summary>
        /// This method is called when Back button is pressed at MainPage
        /// </summary>
        /// <returns>Always returns true</returns>
        private async Task<bool> OnBackKeyPressedAtMain()
        {
            var answer = await DisplayAlert("QUIT", "Do you want to quit?", "YES", "NO");
            if (answer)
            {
                var page = CurrentPage as ContentPageEx;
                page.RunHideAnimation();
                await App.AppMainPage.FadeTo(0, 667);
                Program.Instance.SelfTerminate();
            }
            else
            {
                Focus();
            }

            return true;
        }
    }
}
