﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Input;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.Utils;
using Xamarin.Forms;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;
using Xamarin.Forms.Xaml;
using static TVMediaHub.Tizen.Views.MusicItem;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    ///  A custom view for displaying Music group
    /// </summary>
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class MusicGroup : RelativeLayout
    {
        /// <summary>
        /// Identifies the ItemsSource bindable property
        /// </summary>
        public static readonly BindableProperty ItemsSourceProperty = BindableProperty.Create("ItemsSource", typeof(IEnumerable<MediaShortcutInfo>), typeof(MusicTab), default(IEnumerable<MediaShortcutInfo>));

        /// <summary>
        /// Gets or set ItemSource to display item
        /// </summary>
        public IEnumerable<MediaShortcutInfo> ItemsSource
        {
            get { return (IEnumerable<MediaShortcutInfo>)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        /// <summary>
        /// A list of music items
        /// </summary>
        public List<MusicItem> MusicItemList { get; set; }

        /// <summary>
        /// A command will be executed when item is clicked
        /// </summary>
        private ICommand ItemClickCommand;

        /// <summary>
        /// A list of children's button objects of group
        /// </summary>
        private List<Button> ChildrenFocusList;

        /// <summary>
        /// Gets TitleFocusArea for focus chain
        /// </summary>
        public Button GetTitleFocusArea()
        {
            return TitleFocusArea;
        }

        /// <summary>
        /// left side items for focus chain
        /// </summary>
        public List<Button> LeftFocusList { get; protected set; }

        /// <summary>
        /// right side items for focus chain
        /// </summary>
        public List<Button> RightFocusList { get; protected set; }
        /// <summary>
        /// @TODO:Comment
        /// </summary>
        public List<KeyValuePair<double, Button>> BottomFocusList { get; protected set; }

        /// <summary>
        /// A delegate will be executed when the context popup item selected
        /// </summary>
        /// <param name="info">A clicked item's MediaInformation</param>
        /// <param name="item">A clicked context popup item</param>
        public delegate void ContextPopupItemSelectedEventHandler(MediaInformationEx info, MusicContextPopupItem item);

        /// <summary>
        /// A ContextPopupItemSelectedEventHandler for select event of context popup
        /// </summary>
        public ContextPopupItemSelectedEventHandler ContextPopupItemSelectedHandler;


        /// <summary>
        /// The focused MusicItem in this group
        /// </summary>
        private MusicItem focusedItem;

        public EventHandler<GroupItemFocusEventArgs> GroupItemFocused;

        /// <summary>
        /// A constructor
        /// </summary>
        public MusicGroup()
        {
            InitializeComponent();
            InitializeSize();
            Init();
            PropertyChanged += MusicGroupPropertyChanged;
        }

        /// <summary>
        /// A method for initializing several lists and properties that are used in this class
        /// </summary>
        private void Init()
        {
            ChildrenFocusList = new List<Button>();
            LeftFocusList = new List<Button>();
            RightFocusList = new List<Button>();
            BottomFocusList = new List<KeyValuePair<double, Button>>();
            MusicItemList = new List<MusicItem>();
            GroupTitle.TextColor = Color.FromRgba(255, 255, 255, 31);
            GroupTitle.FontFamily = "BreezSans";
            GroupTitle.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Normal);
        }

        /// <summary>
        /// A method for initializing sizes
        /// </summary>
        private void InitializeSize()
        {
            GroupTitle.FontSize = Utils.SizeUtils.GetFontSize(28);
            GroupTitle.Margin = new Thickness(SizeUtils.GetWidthSize(4), 0, 0, 0);

            TitleFocusArea.WidthRequest = SizeUtils.GetWidthSize(180);
            GroupContentArea.RowSpacing = SizeUtils.GetHeightSize(-134);
            GroupContentArea.ColumnSpacing = SizeUtils.GetWidthSize(-136);
            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(270) });
            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(270) });
            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(270) });
            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(270) });
            GroupContentArea.Margin = new Thickness(SizeUtils.GetWidthSize(-64), SizeUtils.GetHeightSize(-64), SizeUtils.GetWidthSize(-64), SizeUtils.GetHeightSize(-64));
        }

        /// <summary>
        /// A method for setting the command to ItemClickCommand
        /// </summary>
        /// <param name="command">A command to be set</param>
        public void SetClickCommand(ICommand command)
        {
            ItemClickCommand = command;
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void MusicGroupPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("ItemsSource"))
            {
                var index = 0;
                MusicItemList.Clear();
                foreach (var item in ItemsSource)
                {
                    var itemView = new MusicItem();
                    ChildrenFocusList.Add(itemView.GetFocusArea());
                    itemView.BindingContext = item;
                    MusicItemList.Add(itemView);
                    itemView.OnFocusedEventHandler += (se, ev) =>
                    {
                        GroupItemFocused?.Invoke(itemView, new GroupItemFocusEventArgs(X + itemView.X));
                        GroupContentArea.RaiseChild(itemView);
                        focusedItem = itemView;
                    };
                    itemView.OnUnfocusedEventHandler += (se, ev) =>
                    {
                        focusedItem = null;
                    };
                    itemView.OnItemClickedHandler += (info) =>
                    {
                        ItemClickCommand?.Execute(info);
                    };
                    itemView.ContextPopupItemSelectedHandler += (info, selectedItem) =>
                    {
                        ContextPopupItemSelectedHandler?.Invoke(info, selectedItem);
                    };

                    GroupContentArea.Children.Add(itemView, index / 4, index % 4);
                    index++;
                }

                SetFocusChain();
            }
        }

        /// <summary>
        /// A method for making focus chain among items
        /// </summary>
        private void SetFocusChain()
        {
            if (ChildrenFocusList.Count < 1)
            {
                return;
            }

            var lastColumn = (ChildrenFocusList.Count - 1) / 4;
            SetFocusList();

            TitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusDownView(ChildrenFocusList[0]);
            TitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(TitleFocusArea);
            TitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(TitleFocusArea);

            for (var i = 0; i < ChildrenFocusList.Count; i++)
            {
                var target = ChildrenFocusList[i];
                var row = i % 4;
                var column = i / 4;

                // Left
                if (column != 0)
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(ChildrenFocusList[i - 4]);
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(target);
                }

                // Right
                if (column != lastColumn)
                {
                    if ((i + 4) < ChildrenFocusList.Count)
                    {
                        target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(ChildrenFocusList[i + 4]);
                    }
                    else
                    {
                        target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(ChildrenFocusList[ChildrenFocusList.Count - 1]);
                    }
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(target);
                }

                // Up
                if (row != 0)
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusUpView(ChildrenFocusList[i - 1]);
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusUpView(TitleFocusArea);
                }

                // Down
                if (row != 3 && (i + 1) < ChildrenFocusList.Count)
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusDownView(ChildrenFocusList[i + 1]);
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusDownView(target);
                    var targetX = column * SizeUtils.GetWidthSize(580);
                    BottomFocusList.Add(new KeyValuePair<double, Button>(targetX, target));
                }
            }
        }
        private void SetFocusList()
        {
            var totalCount = ChildrenFocusList.Count - 1;
            var lastItemColumn = totalCount / 4;
            var lastItemRow = totalCount % 4;

            var leftFocusFrom = 0;
            var leftFocusTo = lastItemColumn > 0 ? 3 : lastItemRow;

            var rightFocusFrom = lastItemColumn * 4;
            var rightFocusTo = rightFocusFrom + lastItemRow;

            for (var i = leftFocusFrom; i <= leftFocusTo; i++)
            {
                LeftFocusList.Add(ChildrenFocusList[i]);
            }

            for (var i = rightFocusFrom; i <= rightFocusTo; i++)
            {
                RightFocusList.Add(ChildrenFocusList[i]);
            }
        }

        /// <summary>
        /// If there is focused image item, notify the menu key pressed event to focused music item for show context popup
        /// </summary>
        /// <returns>If group has a focused item, true; otherwise, false</returns>
        public bool NotifyMenuKeyPressedToFocusedItem()
        {
            if (focusedItem != null)
            {
                focusedItem.ShowContextPopup();
                return true;
            }

            return false;
        }

        /// <summary>
        /// This method is called when the group's title is focused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        public void OnTitleFocused(object sender, FocusEventArgs e)
        {
            GroupTitle.AbortAnimation("FocusAnimation");
            GroupTitle.Animate("FocusAnimation",
            (v) => { },
            length: 167,
            finished: (v, IsCanceled) =>
            {
                if (!IsCanceled)
                {

                    GroupTitle.Animate("FocusAnimation",
                        (v2) =>
                        {
                            var R = (int)(255 * (1 - v));
                            var G = (int)(255 - 126 * v);
                            var B = (int)(255 - 57 * v);
                            var A = (int)(31 + 224 * v);
                            GroupTitle.TextColor = Color.FromRgba(R, G, B, A);
                        },
                        length: 167,
                        finished: (v2, IsCanceled2) =>
                        {
                            if (!IsCanceled2)
                            {
                                GroupTitle.TextColor = Color.FromRgba(0, 129, 198, 255);
                            }
                        });
                }
            });
        }

        /// <summary>
        /// This method is called when the group's title is unfocused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        public void OnTitleUnFocused(object sender, FocusEventArgs e)
        {
            GroupTitle.AbortAnimation("FocusAnimation");
            GroupTitle.Animate("FocusAnimation",
            (v) =>
            {
                var R = (int)(255 * v);
                var G = (int)(129 + 126 * v);
                var B = (int)(198 + 57 * v);
                var A = (int)(255 - 224 * v);
                GroupTitle.TextColor = Color.FromRgba(R, G, B, A);
            },
            length: 167,
            finished: (v, IsCanceled) =>
            {
                if (!IsCanceled)
                {
                    GroupTitle.TextColor = Color.FromRgba(255, 255, 255, 31);
                }
            });
        }
    }
}