﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.ComponentModel;
using Tizen.Content.MediaContent;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.ViewModels;
using Xamarin.Forms;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;
using Xamarin.Forms.Xaml;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom view for displaying music item
    /// </summary>
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class MusicItem : RelativeLayout
    {

        /// <summary>
        /// Identifies the MusicInfo bindable property
        /// </summary>
        public static readonly BindableProperty MusicInfoProperty = BindableProperty.Create("MusicInfo", typeof(MediaInformationEx), typeof(MusicItem), null);

        /// <summary>
        /// Gets or sets the MusicInfo of current music
        /// </summary>
        public MediaInformationEx MusicInfo
        {
            set
            {
                SetValue(MusicInfoProperty, value);
            }

            get
            {
                return (MediaInformationEx)GetValue(MusicInfoProperty);
            }
        }

        /// <summary>
        /// An EventHandler for focus event of the item
        /// </summary>
        public EventHandler OnFocusedEventHandler;
        /// <summary>
        /// An EventHandler for unfocus event of the item
        /// </summary>
        public EventHandler OnUnfocusedEventHandler;

        /// <summary>
        /// A delegate will be executed when the item is clicked
        /// </summary>
        /// <param name="info">A clicked item's MediaInformation</param>
        public delegate void ItemEventHandler(MediaInformationEx info);
        /// <summary>
        /// A ClickEventHandler for click event of the item
        /// </summary>
        public ItemEventHandler OnItemClickedHandler;

        /// <summary>
        /// Bounds for TextArea when item is unfocused
        /// </summary>
        private Xamarin.Forms.Rectangle TextAreaNormalBounds;

        /// <summary>
        /// Bounds for TextArea when item is focused
        /// </summary>
        private Xamarin.Forms.Rectangle TextAreaFocusedBounds;

        /// <summary>
        /// An enumeration for context popup selectable items
        /// </summary>
        public enum MusicContextPopupItem
        {
            FileInfo = 0,
            Delete,
        }

        /// <summary>
        /// A delegate will be executed when the context popup item selected
        /// </summary>
        /// <param name="info">A clicked item's MediaInformation</param>
        /// <param name="item">A clicked context popup item</param>
        public delegate void ContextPopupItemSelectedEventHandler(MediaInformationEx info, MusicContextPopupItem item);

        /// <summary>
        /// A ContextPopupItemSelectedEventHandler for select event of context popup
        /// </summary>
        public ContextPopupItemSelectedEventHandler ContextPopupItemSelectedHandler;

        /// <summary>
        /// Gets FocusArea for focus chain
        /// </summary>
        public Button GetFocusArea()
        {
            return FocusArea;
        }

        /// <summary>
        /// A constructor
        /// Initializes the size of the items that are used in this class
        /// </summary>
        public MusicItem()
        {
            InitializeComponent();
            InitializeSize();
            InitializeLabel();
            PropertyChanged += MusicItemPropertyChanged;
        }

        /// <summary>
        /// A method for initializing font and slide effect
        /// </summary>
        public void InitializeLabel()
        {
            SongTitle.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Normal);
            Artist.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Normal);
            AlbumTitle.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Light);

            TextSlideEffect.SetDuration(SongTitle, 15000);
            TextSlideEffect.SetIsAlwaysOn(SongTitle, true);

            TextSlideEffect.SetDuration(Artist, 15000);
            TextSlideEffect.SetIsAlwaysOn(Artist, true);

            TextSlideEffect.SetDuration(SongTitle, 15000);
            TextSlideEffect.SetIsAlwaysOn(SongTitle, true);

            TextSlideEffect.SetDuration(AlbumTitle, 15000);
            TextSlideEffect.SetIsAlwaysOn(AlbumTitle, true);
        }

        /// <summary>
        /// A method for initializing size of the items that are used in this class
        /// </summary>
        private void InitializeSize()
        {
            int h28 = SizeUtils.GetHeightSize(28);
            int h32 = SizeUtils.GetHeightSize(32);
            int w32 = SizeUtils.GetWidthSize(32);
            int h134 = SizeUtils.GetHeightSize(134);
            int w134 = SizeUtils.GetWidthSize(134);
            int w382 = SizeUtils.GetWidthSize(382);
            int w446 = SizeUtils.GetWidthSize(446);

            WidthRequest = SizeUtils.GetWidthSize(716);
            HeightRequest = SizeUtils.GetHeightSize(270);

            Shadow.BorderTop = SizeUtils.GetHeightSize(64);
            Shadow.BorderBottom = SizeUtils.GetHeightSize(64);
            Shadow.BorderLeft = SizeUtils.GetWidthSize(64);
            Shadow.BorderRight = SizeUtils.GetWidthSize(64);

            TextAreaNormalBounds = new Xamarin.Forms.Rectangle(SizeUtils.GetWidthSize(202), SizeUtils.GetHeightSize(68), w446, SizeUtils.GetHeightSize(134));
            TextAreaFocusedBounds = new Xamarin.Forms.Rectangle(SizeUtils.GetWidthSize(206), SizeUtils.GetHeightSize(64), w446, SizeUtils.GetHeightSize(142));

            AlbumCover.WidthRequest = w134;
            AlbumCover.HeightRequest = h134;

            TextBackground.WidthRequest = w446;
            TextBackground.HeightRequest = h134;

            SongTitle.WidthRequest = w382;
            SongTitle.HeightRequest = h32;
            SongTitle.FontSize = SizeUtils.GetFontSize(28);
            SongTitle.Margin = new Thickness(w32, SizeUtils.GetHeightSize(18), w32, 0);

            Artist.WidthRequest = w382;
            Artist.HeightRequest = h28;
            Artist.Margin = new Thickness(w32, SizeUtils.GetHeightSize(12), w32, 0);
            Artist.FontSize = SizeUtils.GetFontSize(24);

            AlbumTitle.WidthRequest = w382;
            AlbumTitle.HeightRequest = h28;
            AlbumTitle.Margin = new Thickness(w32, SizeUtils.GetHeightSize(4), w32, 0);
            AlbumTitle.FontSize = SizeUtils.GetFontSize(24);

        }

        /// <summary>
        /// This method is called when item is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private void OnItemClicked(object sender, EventArgs e)
        {
            OnItemClickedHandler?.Invoke(MusicInfo);
        }

        /// <summary>
        /// This method is called when item is focused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private void OnItemFocused(object sender, FocusEventArgs e)
        {

            Easing easing = new Easing(EasingFunction.EasyIn1);
            TextBackground.Source = "img_music_list_focused.9.png";
            TextBackground.LayoutTo(TextAreaFocusedBounds, 300, easing);

            AlbumCover.ScaleTo(1.0597, 300, easing);
            SongTitle.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Medium);
            SongTitle.TextColor = Color.FromHex("000000");
            TextSlideEffect.SetIsTextSliding(SongTitle, true);


            Artist.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Medium);
            Artist.Opacity = 0.8;
            Artist.TextColor = Color.FromHex("000000");
            TextSlideEffect.SetIsTextSliding(Artist, true);


            AlbumTitle.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Normal);
            AlbumTitle.Opacity = 0.7;
            AlbumTitle.TextColor = Color.FromHex("000000");
            TextSlideEffect.SetIsTextSliding(AlbumTitle, true);

            Shadow.IsVisible = true;
            OnFocusedEventHandler?.Invoke(sender, e);
        }

        /// <summary>
        /// This method is called when item is unfocused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private void OnItemUnfocused(object sender, FocusEventArgs e)
        {
            Easing easing = new Easing(EasingFunction.EasyIn2);
            TextBackground.LayoutTo(TextAreaNormalBounds, 300, easing);

            AlbumCover.ScaleTo(1.0, 300, easing);

            SongTitle.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Normal);
            SongTitle.TextColor = Color.FromHex("ffffff");
            TextSlideEffect.SetIsTextSliding(SongTitle, false);

            Artist.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Normal);
            Artist.Opacity = 0.6;
            Artist.TextColor = Color.FromHex("ffffff");
            TextSlideEffect.SetIsTextSliding(Artist, false);

            AlbumTitle.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Light);
            AlbumTitle.Opacity = 0.5;
            AlbumTitle.TextColor = Color.FromHex("ffffff");
            TextSlideEffect.SetIsTextSliding(AlbumTitle, false);

            TextBackground.Source = "img_music_list_normal.9.png";
            Shadow.IsVisible = false;
            OnUnfocusedEventHandler.Invoke(sender, e);
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void MusicItemPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("MusicInfo"))
            {
                AudioInfo info = MusicInfo.MediaContentInformation as AudioInfo;
                AlbumCover.Source = !(string.IsNullOrEmpty(info.ThumbnailPath)) ? info.ThumbnailPath : "img_music_nocover.png";

                SongTitle.Text = info.Title;
                Artist.Text = info.Artist;
                AlbumTitle.Text = info.Album;
            }
        }
        /// <summary>
        /// A method showing context popup
        /// </summary>
        public void ShowContextPopup()
        {
            if (!FocusArea.IsFocused)
            {
                return;
            }

            ContextPopup popup = new ContextPopup();

            popup.Items.Add(new ContextPopupItem("FILE INFO"));
            popup.Items.Add(new ContextPopupItem("DELETE"));
            popup.SelectedIndexChanged += SelectedIndexChanged;
            popup.Show(this, this.Width / 2 - SizeUtils.GetWidthSize(290 - 104), this.Height + SizeUtils.GetHeightSize(72));

        }

        /// <summary>
        /// This method is called when the context popup item selected
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">Changed event's argument</param>
        private void SelectedIndexChanged(object sender, EventArgs e)
        {
            var ctxPopup = sender as ContextPopup;
            switch (ctxPopup.SelectedIndex)
            {
                case 0:
                    ContextPopupItemSelectedHandler?.Invoke(MusicInfo, MusicContextPopupItem.FileInfo);
                    break;
                case 1:
                    ContextPopupItemSelectedHandler?.Invoke(MusicInfo, MusicContextPopupItem.Delete);
                    break;
            }

            ctxPopup.Dismiss();
        }


    }
}
