﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Windows.Input;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.Controls;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.ViewModels;
using Xamarin.Forms;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;
using static TVMediaHub.Tizen.Views.MusicItem;
using TizenVisualElement = Xamarin.Forms.PlatformConfiguration.TizenSpecific.VisualElement;

namespace TVMediaHub.Tizen.Views
{

    /// <summary>
    /// A custom ContentPage for displaying Music tab
    /// </summary>
    public partial class MusicTab : ContentPageEx
    {
        /// <summary>
        /// The flag whether content is ready
        /// </summary>
        private bool IsContentReady = false;

        /// <summary>
        /// The flag that whether all contents are selected or not
        /// </summary>
        private bool IsSelectedAll { get; set; }

        /// <summary>
        /// Identifies the ItemsSource bindable property
        /// </summary>
        public static readonly BindableProperty ItemsSourceProperty = BindableProperty.Create("ItemsSource", typeof(ObservableCollection<GroupItem>), typeof(MusicTab), default(ObservableCollection<GroupItem>));

        /// <summary>
        /// Gets or set ItemSource to display music tab
        /// </summary>
        public ObservableCollection<GroupItem> ItemsSource
        {
            get { return (ObservableCollection<GroupItem>)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        /// <summary>
        /// Identifies the ChangeSortOptionCommand bindable property
        /// </summary>
        public static readonly BindableProperty ChangeSortOptionCommandProperty = BindableProperty.Create("ChangeSortOptionCommand", typeof(ICommand), typeof(MusicTab), default(ICommand));

        /// <summary>
        /// Gets or sets ChangeTabStatus Command
        /// </summary>
        public ICommand ChangeSortOptionCommand
        {
            get { return (ICommand)GetValue(ChangeSortOptionCommandProperty); }
            set { SetValue(ChangeSortOptionCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the ChangeTabStatusCommand bindable property
        /// </summary>
        public static readonly BindableProperty ChangeTabStatusCommandProperty = BindableProperty.Create("ChangeTabStatusCommand", typeof(ICommand), typeof(MusicTab), default(ICommand));

        /// <summary>
        /// Gets or sets ChangeTabStatus Command
        /// </summary>
        public ICommand ChangeTabStatusCommand
        {
            get { return (ICommand)GetValue(ChangeTabStatusCommandProperty); }
            set { SetValue(ChangeTabStatusCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the MusicTabStatus bindable property
        /// </summary>
        public static readonly BindableProperty MusicTabStatusProperty = BindableProperty.Create("MusicTabStatus", typeof(TabStatus), typeof(MusicTab), TabStatus.Default);

        /// <summary>
        /// Gets or sets MusicTabStatus
        /// </summary>
        public TabStatus MusicTabStatus
        {
            get { return (TabStatus)GetValue(MusicTabStatusProperty); }
            set { SetValue(MusicTabStatusProperty, value); }
        }

        /// <summary>
        /// Identifies the DeleteContentCommand bindable property
        /// </summary>
        public static readonly BindableProperty DeleteContentCommandProperty = BindableProperty.Create("DeleteContentCommand", typeof(ICommand), typeof(MusicTab), default(ICommand));

        /// <summary>
        /// Gets or sets DeleteContent Command
        /// </summary>
        public ICommand DeleteContentCommand
        {
            get { return (ICommand)GetValue(DeleteContentCommandProperty); }
            set { SetValue(DeleteContentCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the DeleteContentCommand bindable property
        /// </summary>
        public static readonly BindableProperty SelectAllContentCommandProperty = BindableProperty.Create("SelectAllContentCommand", typeof(ICommand), typeof(MusicTab), default(ICommand));

        /// <summary>
        /// Gets or sets SelectAllContent Command
        /// </summary>
        public ICommand SelectAllContentCommand
        {
            get { return (ICommand)GetValue(SelectAllContentCommandProperty); }
            set { SetValue(SelectAllContentCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the OnClickCommand bindable property
        /// </summary>
        public static readonly BindableProperty OnClickCommandProperty = BindableProperty.Create("OnClickCommand", typeof(ICommand), typeof(MusicTab), default(ICommand));

        /// <summary>
        /// Gets or sets OnClick Command
        /// </summary>
        public ICommand OnClickCommand
        {
            get
            {
                return (ICommand)GetValue(OnClickCommandProperty);
            }

            set
            {
                SetValue(OnClickCommandProperty, value);
            }
        }

        /// <summary>
        /// Identifies the SelectedCount bindable property
        /// </summary>
        public static readonly BindableProperty SelectedCountProperty = BindableProperty.Create("SelectedCount", typeof(int), typeof(MusicTab), 0);

        /// <summary>
        /// Gets or sets count of selected item
        /// </summary>
        public int SelectedCount
        {
            get { return (int)GetValue(SelectedCountProperty); }
            set { SetValue(SelectedCountProperty, value); }
        }

        /// <summary>
        /// Identifies the GetInformationsCommand bindable property
        /// </summary>
        public static readonly BindableProperty GetInformationsCommandProperty = BindableProperty.Create("GetInformationsCommand", typeof(ICommand), typeof(MusicTab), default(ICommand));

        /// <summary>
        /// Gets or sets GetInformationsCommand
        /// </summary>
        public ICommand GetInformationsCommand
        {
            get { return (ICommand)GetValue(GetInformationsCommandProperty); }
            set { SetValue(GetInformationsCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the ChangeSourceCommand bindable property
        /// </summary>
        public static readonly BindableProperty ChangeSourceCommandProperty = BindableProperty.Create("ChangeSourceCommand", typeof(ICommand), typeof(MusicTab), default(ICommand));

        /// <summary>
        /// Gets or sets ChangeSourceCommand Command
        /// </summary>
        public ICommand ChangeSourceCommand
        {
            get { return (ICommand)GetValue(ChangeSourceCommandProperty); }
            set { SetValue(ChangeSourceCommandProperty, value); }
        }

        /// <summary>
        /// A list of bottom buttons
        /// </summary>
        private List<KeyValuePair<double, Button>> BottomButtonList;

        /// <summary>
        /// A constructor
        /// Initializes the size of the items that are used in Music tab
        /// </summary>
        public MusicTab()
        {
            BindingContext = MusicTabViewModelLocator.ViewModel;
            InitializeComponent();
            InitializeSize();
            InitializeFonts();
            InitializeFooter();
            InitializeData();
            PropertyChanged += MusicTabPropertyChanged;
            ItemsSource.CollectionChanged += ItemsSourceCollectionChanged;
        }

        private void InitializeData()
        {
            BottomButtonList = new List<KeyValuePair<double, Button>>();
        }

        private void MusicTabPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("ImageTabStatus"))
            {
                SetFooterFocusChain(MusicTabScrollView.ScrollX);
            }
        }

        private void SetFooterFocusChain(double scrollX)
        {
            FooterNormal.SetFocusChain();
            FooterDelete.SetFocusChain();

            var Padding = SizeUtils.GetWidthSize(60);
            var ItemWidth = SizeUtils.GetWidthSize(580);

            var FocusableBoundFrom = scrollX;
            var FocusableBoundTo = scrollX + MusicTabScrollView.Width - ItemWidth;

            var list = BottomButtonList.FindAll((pair) =>
            {
                return (pair.Key >= FocusableBoundFrom && pair.Key <= FocusableBoundTo);
            });

            if (MusicTabStatus == TabStatus.Delete)
            {
                TizenVisualElement.SetNextFocusDownView(MusicPlayer.GetFocusArea(), FooterDelete.GetSelectAllButton());
                TizenVisualElement.SetNextFocusUpView(FooterDelete.GetSelectAllButton(), MusicPlayer.GetFocusArea());
            }
            else
            {
                TizenVisualElement.SetNextFocusDownView(MusicPlayer.GetFocusArea(), FooterNormal.DropdownSource);
                TizenVisualElement.SetNextFocusUpView(FooterNormal.DropdownSource, MusicPlayer.GetFocusArea());
            }

            if(list.Count == 1)
            {
                var btn = list[0].Value;
                if (MusicTabStatus == TabStatus.Delete)
                {
                    TizenVisualElement.SetNextFocusDownView(btn, FooterDelete.GetOkButton());
                    TizenVisualElement.SetNextFocusUpView(FooterDelete.GetCancelButton(), btn);
                    TizenVisualElement.SetNextFocusUpView(FooterDelete.GetOkButton(), btn);
                }
                else
                {
                    TizenVisualElement.SetNextFocusDownView(btn, FooterNormal.DropdownSort);
                    TizenVisualElement.SetNextFocusUpView(FooterNormal.ButtonOption, btn);
                    TizenVisualElement.SetNextFocusUpView(FooterNormal.DropdownSort, btn);
                }
            }
            else
            {
                for (var buttonIndex = 0; buttonIndex < list.Count; buttonIndex++)
                {
                    var btn = list[buttonIndex].Value;

                    if (buttonIndex == list.Count - 1)
                    {
                        if (MusicTabStatus == TabStatus.Delete)
                        {
                            TizenVisualElement.SetNextFocusDownView(btn, FooterDelete.GetCancelButton());
                            TizenVisualElement.SetNextFocusUpView(FooterDelete.GetCancelButton(), btn);
                        }
                        else
                        {
                            TizenVisualElement.SetNextFocusDownView(btn, FooterNormal.ButtonOption);
                            TizenVisualElement.SetNextFocusUpView(FooterNormal.ButtonOption, btn);
                        }
                    }
                    else
                    {
                        if (MusicTabStatus == TabStatus.Delete)
                        {
                            TizenVisualElement.SetNextFocusDownView(btn, FooterDelete.GetOkButton());
                            TizenVisualElement.SetNextFocusUpView(FooterDelete.GetOkButton(), btn);
                        }
                        else
                        {
                            TizenVisualElement.SetNextFocusDownView(btn, FooterNormal.DropdownSort);
                            TizenVisualElement.SetNextFocusUpView(FooterNormal.DropdownSort, btn);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// A method for initializing the size of the items that are used in Music tab
        /// </summary>
        private void InitializeSize()
        {
            ImgNoContents.Margin = new Thickness(0, SizeUtils.GetHeightSize(266), 0, SizeUtils.GetHeightSize(28));
            ImgNoContents.HeightRequest = SizeUtils.GetHeightSize(88);
            ImgNoContents.WidthRequest = SizeUtils.GetWidthSize(68);
            LabelNoContents.HeightRequest = SizeUtils.GetHeightSize(32);
            LabelNoContents.FontSize = SizeUtils.GetFontSize(28);
            MusicContentView.Spacing = SizeUtils.GetWidthSize(60);
            MusicTabContents.Padding = new Thickness(0, SizeUtils.GetHeightSize(74), 0, 0);
        }

        /// <summary>
        /// A method for initializing several fonts that are used in this class
        /// </summary>
        private void InitializeFonts()
        {
            LabelNoContents.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Light);
            LabelNoContents.TextColor = Color.FromRgba(255, 255, 255, 20);
        }

        /// <summary>
        /// Initialize elements on footer area
        /// </summary>
        private void InitializeFooter()
        {
            FooterNormal.OnDropdownSourceItemSelected += OnSourceChanged;
            FooterNormal.OnDropdownSortItemSelected += OnSortOptionChanged;
            FooterNormal.OnSelectedOptionIndexChanged += OnOptionSelected;

            FooterDelete.SelecteAllButtonEvent += OnSelectAllClicked;
            FooterDelete.OKButtonEvent += OnOKClicked;
            FooterDelete.CancelButtonEvent += OnCancelClicked;
        }

        /// <summary>
        /// A method for initializing page when the page is appeared
        /// </summary>
        protected override void InitializePage()
        {
            if (IsContentReady == false)
            {
                GetInformationsCommand?.Execute("");
                IsContentReady = true;
            }
        }

        /// <summary>
        /// A method for finalizing page when the page is disappeared
        /// </summary>
        protected override void FinalizePage()
        {
        }

        /// <summary>
        /// A method for running hide animation
        /// </summary>
        public override void RunHideAnimation()
        {
        }

        /// <summary>
        /// A method for running show animation
        /// </summary>
        public override void RunShowAnimation()
        {
        }


        /// <summary>
        /// Occurs when an item is added, removed, changed, moved, or the entire list is refreshed.
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">Information about the event</param>
        private void ItemsSourceCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.Action.ToString().Equals(NotifyCollectionChangedAction.Add.ToString()))
            {
                if (MusicNoContents.IsVisible)
                {
                    MusicNoContents.IsVisible = false;
                    MusicTabScrollView.IsVisible = true;
                    FooterNormal.IsFooterEnabled = true;
                }

                var groupItem = e.NewItems[0];
                MusicGroup groupView = new MusicGroup();
                groupView.PropertyChanged += (se, ev) =>
                {
                    if (ev.PropertyName.Equals("ItemsSource"))
                    {
                        if (groupView.X >= 0)
                        {
                            groupView.BottomFocusList.ForEach((item) =>
                            {
                                var key = groupView.X + item.Key;
                                BottomButtonList.Add(new KeyValuePair<double, Button>(key, item.Value));
                            });

                            SetFooterFocusChain(0);
                        }
                    }
                };

                groupView.SetClickCommand(OnClickCommand);
                groupView.ContextPopupItemSelectedHandler += (info, item) =>
                {
                    ShowPopup(info, item);
                };

                groupView.GetTitleFocusArea().Focused += (se, ev) =>
                {
                    ScrollToTarget(groupView.X, SizeUtils.GetWidthSize(580), SizeUtils.GetWidthSize(100));
                };

                groupView.GroupItemFocused += (se, ev) =>
                {
                    ScrollToTarget(ev.PositionX, SizeUtils.GetWidthSize(580), SizeUtils.GetWidthSize(100));
                };

                MusicContentView.Children.Add(groupView);
                groupView.BindingContext = groupItem;

                if (MusicContentView.Children.Count > 1)
                {
                    var leftGroup = MusicContentView.Children[MusicContentView.Children.Count - 2] as MusicGroup;


                    leftGroup.GetTitleFocusArea().On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(groupView.GetTitleFocusArea());
                    groupView.GetTitleFocusArea().On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(leftGroup.GetTitleFocusArea());

                    leftGroup.GetTitleFocusArea().Clicked += (se, ev) =>
                    {
                        leftGroup.GetTitleFocusArea().On<Xamarin.Forms.PlatformConfiguration.Tizen>().MoveFocusRight();
                    };

                    var leftList = leftGroup.RightFocusList;
                    var rightList = groupView.LeftFocusList;
                    var minCount = leftList.Count > rightList.Count ? rightList.Count : leftList.Count;

                    for (var i = 0; i < minCount; i++)
                    {
                        leftList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(rightList[i]);
                        rightList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(leftList[i]);
                    }
                    if (leftList.Count != rightList.Count)
                    {

                        if (leftList.Count > rightList.Count)
                        {
                            for (var i = rightList.Count; i < leftList.Count; i++)
                            {
                                leftList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(rightList[rightList.Count - 1]);
                            }
                        }
                        else
                        {
                            for (var i = leftList.Count; i < rightList.Count; i++)
                            {
                                rightList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(leftList[leftList.Count - 1]);
                            }
                        }
                    }
                }
                else
                {
                    groupView.GetTitleFocusArea().On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(MusicPlayer.GetFocusArea());
                    MusicPlayer.GetFocusArea().On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(groupView.GetTitleFocusArea());

                    foreach (var item in groupView.LeftFocusList)
                    {
                        item.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(MusicPlayer.GetFocusArea());
                    }

                }
            }
            else if (e.Action.ToString().Equals(NotifyCollectionChangedAction.Reset.ToString()))
            {
                MusicContentView.Children.Clear();
                if (!MusicNoContents.IsVisible)
                {
                    MusicTabScrollView.IsVisible = false;
                    MusicNoContents.IsVisible = true;
                    FooterNormal.IsFooterEnabled = false;
                }
            }
        }

        /// <summary>
        /// This method is called when Source option is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A SelectedItemChanged event's argument</param>
        private void OnSourceChanged(object sender, SelectedItemChangedEventArgs e)
        {
            string storageName = e.SelectedItem as string;

            BottomButtonList.Clear();
            ChangeSourceCommand?.Execute(storageName);
            SetFooterFocusChain(MusicTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when Sort option is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A SelectedItemChanged event's argument</param>
        private void OnSortOptionChanged(object sender, SelectedItemChangedEventArgs e)
        {
            BottomButtonList.Clear();
            ChangeSortOptionCommand?.Execute(e.SelectedItem);
            SetFooterFocusChain(MusicTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when option menu is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A SelectedItemChanged event's argument</param>
        private void OnOptionSelected(object sender, ContextPopupSelectedEventArgs e)
        {
            var label = e.Item.Label.Trim().ToLower();
            if (label.Equals("delete"))
            {
                ChangeTabStatusCommand?.Execute("");
                FooterNormal.IsVisible = false;
                FooterDelete.IsVisible = true;
                SetFooterFocusChain(MusicTabScrollView.ScrollX);
            }
            else if (label.Equals("disk info"))
            {
                ShowDiskInfoPopup();
            }
        }

        /// <summary>
        /// This method is called when cancel button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void OnCancelClicked(object sender, EventArgs e)
        {
            ChangeTabStatusCommand?.Execute("");
            FooterNormal.IsVisible = true;
            FooterDelete.IsVisible = false;
            SetFooterFocusChain(MusicTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when SelectAll button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void OnSelectAllClicked(object sender, EventArgs e)
        {
            IsSelectedAll = FooterDelete.IsSelectedAll;
            SelectAllContentCommand?.Execute(IsSelectedAll);
        }

        /// <summary>
        /// This method is called when OK button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private async void OnOKClicked(object sender, EventArgs e)
        {
            string title = "Delete";

            if (SelectedCount < 1)
            {
                return;
            }
            else if (SelectedCount > 1)
            {
                title = "Delete Selected";

                if (IsSelectedAll)
                {
                    title = "Delete All";
                }
            }

            bool answer = await DisplayAlert(title, "Do you want to delete?", "YES", "NO");

            if (answer)
            {
                DeleteContentCommand?.Execute(null);
                GetInformationsCommand?.Execute("");
            }

            ChangeTabStatusCommand?.Execute("");
            FooterNormal.IsVisible = true;
            FooterDelete.IsVisible = false;
            SetFooterFocusChain(MusicTabScrollView.ScrollX);
        }

        /// <summary>
        /// this method is for customizing behavior when the Page becoming visible
        /// </summary>
        protected override void OnAppearing()
        {
            base.OnAppearing();

            App.KeyUpEvent.On += KeyUpEventOn;
        }

        /// <summary>
        /// KeyDown will be triggered when key is menu
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">The information about the event</param>
        private void KeyUpEventOn(object sender, ElmSharp.EcoreKeyEventArgs e)
        {
            if (e.KeyName.Contains(ElmSharp.EvasKeyEventArgs.PlatformMenuButtonName))
            {
                NotifyMenuKeyPressed();
            }
        }

        /// <summary>
        /// this method is for customizing behavior when the Page becoming invisible
        /// </summary>
        protected override void OnDisappearing()
        {
            base.OnDisappearing();
            App.KeyUpEvent.On -= KeyUpEventOn;
        }

        /// <summary>
        /// A method notifies menu key pressed event
        /// </summary>
        private void NotifyMenuKeyPressed()
        {
            if (MusicContentView.Children.Count > 0)
            {
                foreach (MusicGroup group in MusicContentView.Children)
                {
                    if (group.NotifyMenuKeyPressedToFocusedItem())
                    {
                        break;
                    }
                }

            }
        }

        /// <summary>
        /// A method displaying pop-ups
        /// </summary>
        /// <param name="info">A focused item's MediaInformation</param>
        /// <param name="item">A clicked context popup item</param>
        private async void ShowPopup(MediaInformationEx info, MusicContextPopupItem item)
        {
            switch (item)
            {
                case MusicContextPopupItem.FileInfo:
                    string message = "File name  : " + info.MediaContentInformation.Title + "\n";
                    message += "File size : " + info.MediaContentInformation.FileSize + "\n";
                    message += "File format : " + info.MediaContentInformation.MimeType + "\n";
                    message += "File size : " + info.MediaContentInformation.FileSize + "\n";
                    message += "File path : " + info.MediaContentInformation.Path;

                    await DisplayAlert("File Information", message, "Close");
                    Dialog dialog = new Dialog();
                    break;
                case MusicContextPopupItem.Delete:
                    bool answer = await DisplayAlert("Delete", "Delete '" + info.MediaContentInformation.Title + "'?", "Yes", "No");
                    if (answer)
                    {
                        DeleteContentCommand?.Execute(info);
                        GetInformationsCommand?.Execute("");
                    }

                    break;
            }

        }

        /// <summary>
        /// A method for showing Disk Information Popup
        /// </summary>
        private async void ShowDiskInfoPopup()
        {
            int numberOfFiles = 0;

            foreach (MusicGroup group in MusicContentView.Children)
            {
                numberOfFiles += group.MusicItemList.Count;
            }

            var message = "Number of files  : " + numberOfFiles + "\n";
            await DisplayAlert("Disk Information", message, "Close");
        }

        /// <summary>
        /// Moves the scroller to the given target
        /// </summary>
        /// <param name="targetX">The given target</param>
        /// <param name="targetWidth">The width of the given target</param>
        /// <param name="padding">The padding of the given target</param>
        private void ScrollToTarget(double targetX, int targetWidth, int padding)
        {
            var ScrollX = MusicTabScrollView.ScrollX;

            var ScrollBoundFrom = targetX - (MusicTabScrollView.Width - targetWidth - padding);
            ScrollBoundFrom = ScrollBoundFrom > 0 ? ScrollBoundFrom : 0;
            var ScrollBoundTo = targetX - padding;
            ScrollBoundTo = ScrollBoundTo > 0 ? ScrollBoundTo : 0;
            DbgPort.D("TEST");
            // Is Focusable.
            if (ScrollX >= ScrollBoundFrom && ScrollX <= ScrollBoundTo)
            {
                return;
            }

            if (Math.Abs(ScrollX - ScrollBoundFrom) > Math.Abs(ScrollX - ScrollBoundTo))
            {
                Device.BeginInvokeOnMainThread(() => MusicTabScrollView.ScrollToAsync(ScrollBoundTo, 0, true));
                SetFooterFocusChain(ScrollBoundTo);
            }
            else
            {
                Device.BeginInvokeOnMainThread(() => MusicTabScrollView.ScrollToAsync(ScrollBoundFrom, 0, true));
                SetFooterFocusChain(ScrollBoundFrom);
            }
        }
    }
}
