﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Threading;
using System.Windows.Input;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.Controls;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.ViewModels;
using Xamarin.Forms;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;

namespace TVMediaHub.Tizen.Views
{
    using Tizen = Xamarin.Forms.PlatformConfiguration.Tizen;

    /// <summary>
    /// A custom ContentPage for displaying the simple image viewer
    /// </summary>
    public partial class SimpleImageViewer : ContentPageEx
    {
        /// <summary>
        /// An enumeration for ControlAreaState
        /// </summary>
        enum ControlAreaState
        {
            SHOW = 0,
            HIDE,
            AUTO,
        };

        /// <summary>
        /// A flag whether rotation animation is running or not
        /// </summary>
        private bool isRotationRunning;

        /// <summary>
        /// A flag whether option popup is showing
        /// </summary>
        private bool isOptionPopupShowing;

        /// <summary>
        /// A flag whether speed option popup is showing
        /// </summary>
        private bool isSpeedOptionPopupShowing;

        /// <summary>
        /// A flag whether effect option popup is showing
        /// </summary>
        private bool isEffectOptionPopupShowing;

        /// <summary>
        /// An instance for the option popup
        /// </summary>
        private ContextPopup optionPopup;

        /// <summary>
        /// An instance for the speed option popup
        /// </summary>
        private ContextPopup speedOptionPopup;

        /// <summary>
        /// An instance for the effect option popup
        /// </summary>
        private ContextPopup effectOptionPopup;

        /// <summary>
        /// A key name list of the remote controller
        /// </summary>
        private string[] remoteKeys = new string[] { "Up", "Down", "Left", "Right", "XF86Menu" };

        /// <summary>
        /// The timer to handle the control area
        /// </summary>
        private Timer timer;

        /// <summary>
        /// The due time of the timer
        /// 5 second
        /// </summary>
        private int timerDueTime = 5000;

        /// <summary>
        /// Identifies the SavedRotationCommand bindable property
        /// </summary>
        public static readonly BindableProperty SavedRotationCommandProperty = BindableProperty.Create("SavedRotationCommand", typeof(ICommand), typeof(SimpleImageViewer), default(ICommand));

        /// <summary>
        /// Gets or sets SavedRotationCommand
        /// </summary>
        public ICommand SavedRotationCommand
        {
            get { return (ICommand)GetValue(SavedRotationCommandProperty); }
            set { SetValue(SavedRotationCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the NavigationCommand bindable property
        /// </summary>
        public static readonly BindableProperty NavigationCommandProperty = BindableProperty.Create("NavigationCommand", typeof(ICommand), typeof(SimpleImageViewer), default(ICommand));

        /// <summary>
        /// Gets or sets NavigationCommand
        /// </summary>
        public ICommand NavigationCommand
        {
            get { return (ICommand)GetValue(NavigationCommandProperty); }
            set { SetValue(NavigationCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the SetSlideShowSpeedCommand bindable property
        /// </summary>
        public static readonly BindableProperty SetSlideShowSpeedCommandProperty = BindableProperty.Create("SetSlideShowSpeedCommand", typeof(ICommand), typeof(SimpleImageViewer), default(ICommand));

        /// <summary>
        /// Gets or sets SetSlideShowSpeedCommand
        /// </summary>
        public ICommand SetSlideShowSpeedCommand
        {
            get { return (ICommand)GetValue(SetSlideShowSpeedCommandProperty); }
            set { SetValue(SetSlideShowSpeedCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the SetSlideShowEffectCommand bindable property
        /// </summary>
        public static readonly BindableProperty SetSlideShowEffectCommandProperty = BindableProperty.Create("SetSlideShowEffectCommand", typeof(ICommand), typeof(SimpleImageViewer), default(ICommand));

        /// <summary>
        /// Gets or sets SetSlideShowEffectCommand
        /// </summary>
        public ICommand SetSlideShowEffectCommand
        {
            get { return (ICommand)GetValue(SetSlideShowEffectCommandProperty); }
            set { SetValue(SetSlideShowEffectCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the CurrentImage bindable property
        /// </summary>
        public static readonly BindableProperty CurrentImageProperty = BindableProperty.Create("CurrentImage", typeof(ImageViewerInfomation), typeof(SimpleImageViewer), null);

        /// <summary>
        /// Gets or sets CurrentImage
        /// </summary>
        public ImageViewerInfomation CurrentImage
        {
            get { return (ImageViewerInfomation)GetValue(CurrentImageProperty); }
            set { SetValue(CurrentImageProperty, value); }
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public SimpleImageViewer()
        {
            BindingContext = ImageTabViewModelLocator.ViewModel;
            InitializeComponent();
            InitializeSize();

            SetSlideShowSpeedCommand?.Execute(SlideShowSpeed.Normal);
            SetSlideShowEffectCommand?.Execute(SlideShowEffect.Dissolve);
            SavedRotationCommand?.Execute(Convert.ToDouble(-1));

            InitializeCurrentImage();

            InitializeButtons();

            timer = new Timer(HideControlAreaHandler, null, Timeout.Infinite, Timeout.Infinite);

            PropertyChanged += ImageViewerPropertyChanged;
            OnBackButtonPressedHandler += async (s, e) =>
            {
                if (isOptionPopupShowing)
                {
                    optionPopup.Dismiss();

                    return;
                }

                if (isSpeedOptionPopupShowing)
                {
                    speedOptionPopup.Dismiss();

                    return;
                }

                if (isEffectOptionPopupShowing)
                {
                    effectOptionPopup.Dismiss();

                    return;
                }

#pragma warning disable CS4014
                GradientDim.FadeTo(0, 667, Easing.CubicInOut);
                ControlArea.TranslateTo(0, 93, 667, Easing.CubicInOut);
                ControlArea.FadeTo(0, 667, Easing.CubicInOut);
                TitleContainer.TranslateTo(0, -50, 667, Easing.CubicInOut);
                await TitleContainer.FadeTo(0, 667, Easing.CubicInOut);

                App.AppMainPage.Navigation.PopAsync();
                //Program.TransitionToMain(1);
#pragma warning restore CS4014
            };
        }

        /// <summary>
        /// Grabs the key of the remote controller
        /// </summary>
        private void GrabRemoteKeys()
        {
            App.KeyDownEvent.On += EvtDownOn;
        }

        private void EvtDownOn(object sender, ElmSharp.EcoreKeyEventArgs e)
        {
            SetControlAreaState(ControlAreaState.AUTO);
        }

        /// <summary>
        /// Ungrabs the key of the remote controller
        /// </summary>
        private void UngrabRemoteKeys()
        {
            App.KeyDownEvent.On -= EvtDownOn;
        }

        /// <summary>
        /// KeyDown will be triggered when key is pressed down
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">The information about the event</param>
        private void RemoteKeyDownListener(object sender, ElmSharp.EvasKeyEventArgs e)
        {
            SetControlAreaState(ControlAreaState.AUTO);
        }

        /// <summary>
        /// A method for changing control area according to ControlAreaState
        /// </summary>
        /// <param name="state">A current ControlAreaState</param>
        private void SetControlAreaState(ControlAreaState state)
        {
            switch (state)
            {
                case ControlAreaState.SHOW:
                    if (ControlArea.Opacity == 0)
                    {
                        RunShowAnimation();
                    }

                    timer.Change(Timeout.Infinite, Timeout.Infinite);

                    break;
                case ControlAreaState.HIDE:
                    if (ControlArea.Opacity == 1)
                    {
                        RunHideAnimation();
                    }

                    timer.Change(Timeout.Infinite, Timeout.Infinite);

                    break;
                case ControlAreaState.AUTO:
                default:
                    if (ControlArea.Opacity == 0)
                    {
                        RunShowAnimation();
                    }

                    timer.Change(timerDueTime, Timeout.Infinite);

                    break;
            }
        }

        /// <summary>
        /// A method for initializing current image's fields
        /// </summary>
        private void InitializeCurrentImage()
        {
            Thumb1.Source = CurrentImage.FilePath;
            Thumb1.WidthRequest = App.MainWindow.Geometry.Width;
            Thumb1.HeightRequest = App.MainWindow.Geometry.Height;
            Thumb1.TranslationX = App.MainWindow.Geometry.X;
            Thumb1.TranslationY = App.MainWindow.Geometry.Y;
            Thumb1.Rotation = CurrentImage.SavedRotation;
        }

        /// <summary>
        /// A method for initializing several sizes
        /// </summary>
        private void InitializeSize()
        {
            int titleSize = SizeUtils.GetFontSize(44);
            int dateSize = SizeUtils.GetFontSize(26);
            TitleLabel.FontSize = titleSize;
            DateLabel.FontSize = dateSize;
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void ImageViewerPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("CurrentImage"))
            {
                int currentAngle = Convert.ToInt32(CurrentImage.SavedRotation);
                Thumb1.Source = CurrentImage.FilePath;
                Thumb1.Scale = (currentAngle == 90 || currentAngle == 270) ? (double)((double)App.MainWindow.Geometry.Height / (double)App.MainWindow.Geometry.Width) : 1;
                Thumb1.Rotation = CurrentImage.SavedRotation;
                return;
            }
        }

        /// <summary>
        /// A method for initializing button of ImageViewer
        /// </summary>
        private void InitializeButtons()
        {
            MediaHubButton[] buttons = new MediaHubButton[6] { RotateBtn, ZoomBtn, LeftBtn, RightBtn, SlideShowBtn, OptionBtn };
            string[] icons = new string[6] { "rotate", "zoom", "left", "right", "video", "option" };
            string iconPath = "btn_viewer_control_rotate_normal.png";

            for (int i = 0; i < buttons.Length; i++)
            {
                buttons[i].NormalImage = iconPath.Replace("rotate", icons[i]);
                buttons[i].PressedImage = buttons[i].NormalImage.Replace("normal", "pressed");
                buttons[i].MediaHubButtonIcon = buttons[i].NormalImage;
            }
        }

        /// <summary>
        /// This method is called when Rotate button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private async void RotateBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            if (isRotationRunning)
            {
                return;
            }

            int duration = 350;
            int rotateAngle = 90;
            int currentAngle = Convert.ToInt32(Thumb1.Rotation);

            isRotationRunning = true;

#pragma warning disable CS4014
            Thumb1.ScaleTo((((currentAngle + rotateAngle) == 90 || (currentAngle + rotateAngle) == 270) ? (double)((double)Thumb1.Height / (double)Thumb1.Width) : 1), Convert.ToUInt32(duration), Easing.CubicInOut);
#pragma warning restore CS4014

            await Thumb1.RotateTo(Thumb1.Rotation + rotateAngle, Convert.ToUInt32(duration), Easing.CubicInOut);

            Thumb1.Rotation = (Thumb1.Rotation == 360) ? 0 : Thumb1.Rotation;
            SavedRotationCommand?.Execute(Thumb1.Rotation);
            isRotationRunning = false;
        }

        /// <summary>
        /// This method is called when Zoom button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void ZoomBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            Program.TransitionTo(new ImageViewerZoom());
        }

        /// <summary>
        /// This method is called when Left button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void LeftBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            NavigationCommand?.Execute("Left");
        }

        /// <summary>
        /// This method is called when Right button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void RightBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            NavigationCommand?.Execute("Right");
        }

        /// <summary>
        /// This method is called when Slide show button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void SlideShowBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            Program.TransitionTo(new ImageViewerSlideShow());
        }

        /// <summary>
        /// This method is called when Option button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void OptionBtnClicked(object sender, EventArgs e)
        {
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            CreateOptionPopup();
        }

        /// <summary>
        /// A method for creating option context popup
        /// </summary>
        private void CreateOptionPopup()
        {
            if (isOptionPopupShowing)
            {
                return;
            }

            optionPopup = new ContextPopup
            {
                IsAutoHidingEnabled = true,
                DirectionPriorities = new ContextPopupDirectionPriorities(ContextPopupDirection.Up, ContextPopupDirection.Right, ContextPopupDirection.Left, ContextPopupDirection.Down),
            };

            optionPopup.Items.Add(new ContextPopupItem("  SLIDE SHOW SPEED  "));
            optionPopup.Items.Add(new ContextPopupItem("  SLIDE SHOW EFFECT  "));

            optionPopup.SelectedIndexChanged += (s, args) =>
            {
                var ctxPopup = s as ContextPopup;

                DbgPort.D("[SimpleImageViewer] Selected Item : " + (ctxPopup.SelectedItem as ContextPopupItem).Label + " (" + ctxPopup.SelectedIndex + ")");
                switch (ctxPopup.SelectedIndex)
                {
                    case 0:
                        /// Slide Show Speed
                        CreateSlideShowSpeedOptionPopup();
                        break;
                    case 1:
                        /// Slide Show Effect
                        CreateSlideShowEffectOptionPopup();
                        break;
                    default:
                        break;
                }

                optionPopup.Dismiss();
                optionPopup = null;
            };

            optionPopup.Dismissed += (s, args) =>
            {
                isOptionPopupShowing = false;

                if (!isSpeedOptionPopupShowing && !isEffectOptionPopupShowing)
                {
                    SetControlAreaState(ControlAreaState.AUTO);
                }
            };

            optionPopup.Show(PopupAnchor, (PopupAnchor.Width / 2), PopupAnchor.Height);
            isOptionPopupShowing = true;

            SetControlAreaState(ControlAreaState.SHOW);
        }

        /// <summary>
        /// A method for creating slide show speed option context popup
        /// </summary>
        private void CreateSlideShowSpeedOptionPopup()
        {
            if (isSpeedOptionPopupShowing)
            {
                return;
            }

            speedOptionPopup = new ContextPopup
            {
                IsAutoHidingEnabled = true,
                DirectionPriorities = new ContextPopupDirectionPriorities(ContextPopupDirection.Up, ContextPopupDirection.Right, ContextPopupDirection.Left, ContextPopupDirection.Down),
            };

            speedOptionPopup.Items.Add(new ContextPopupItem("              NORMAL             "));
            speedOptionPopup.Items.Add(new ContextPopupItem("              SLOW              "));
            speedOptionPopup.Items.Add(new ContextPopupItem("              FAST              "));

            speedOptionPopup.SelectedIndexChanged += (s, e) =>
            {
                var ctxPopup = s as ContextPopup;

                DbgPort.D("[SlideShowSpeed] Selected Item : " + (ctxPopup.SelectedItem as ContextPopupItem).Label + "(" + ctxPopup.SelectedIndex + ")");

                switch (ctxPopup.SelectedIndex)
                {
                    case 0:
                        /// Normal
                        SetSlideShowSpeedCommand?.Execute(SlideShowSpeed.Normal);
                        break;
                    case 1:
                        /// Slow
                        SetSlideShowSpeedCommand?.Execute(SlideShowSpeed.Slow);
                        break;
                    case 2:
                        /// Fast
                        SetSlideShowSpeedCommand?.Execute(SlideShowSpeed.Fast);
                        break;
                    default:
                        break;
                }

                speedOptionPopup.Dismiss();
                speedOptionPopup = null;
            };

            speedOptionPopup.Dismissed += (s, args) =>
            {
                isSpeedOptionPopupShowing = false;

                CreateOptionPopup();
            };

            speedOptionPopup.Show(PopupAnchor, (PopupAnchor.Width / 2), PopupAnchor.Height);
            isSpeedOptionPopupShowing = true;
        }

        /// <summary>
        /// A method for creating slide show effect option context popup
        /// </summary>
        private void CreateSlideShowEffectOptionPopup()
        {
            if (isEffectOptionPopupShowing)
            {
                return;
            }

            effectOptionPopup = new ContextPopup
            {
                IsAutoHidingEnabled = true,
                DirectionPriorities = new ContextPopupDirectionPriorities(ContextPopupDirection.Up, ContextPopupDirection.Right, ContextPopupDirection.Left, ContextPopupDirection.Down),
            };

            effectOptionPopup.Items.Add(new ContextPopupItem("            DISSOLVE            "));
            effectOptionPopup.Items.Add(new ContextPopupItem("            FADE OUT            "));

            effectOptionPopup.SelectedIndexChanged += (s, e) =>
            {
                var ctxPopup = s as ContextPopup;

                DbgPort.D("[SlideShowEffect] Selected Item : " + (ctxPopup.SelectedItem as ContextPopupItem).Label + "(" + ctxPopup.SelectedIndex + ")");

                switch (ctxPopup.SelectedIndex)
                {
                    case 0:
                        /// Dissolve
                        SetSlideShowEffectCommand?.Execute(SlideShowEffect.Dissolve);
                        break;
                    case 1:
                        /// Fade out
                        SetSlideShowEffectCommand?.Execute(SlideShowEffect.FadeOut);
                        break;
                    default:
                        break;
                }

                effectOptionPopup.Dismiss();
                effectOptionPopup = null;
            };

            effectOptionPopup.Dismissed += (s, args) =>
            {
                isEffectOptionPopupShowing = false;

                CreateOptionPopup();

            };

            effectOptionPopup.Show(PopupAnchor, (PopupAnchor.Width / 2), PopupAnchor.Height);
            isEffectOptionPopupShowing = true;
        }

        /// <summary>
        /// Positions and sizes the children of a Layout.
        /// </summary>
        /// <param name="x">A value representing the x coordinate of the child region bounding box.</param>
        /// <param name="y">A value representing the y coordinate of the child region bounding box.</param>
        /// <param name="width">A value representing the width of the child region bounding box.</param>
        /// <param name="height">A value representing the height of the child region bounding box.</param>
        protected override void LayoutChildren(double x, double y, double width, double height)
        {
            base.LayoutChildren(x, y, width, height);
            GradientDim.HeightRequest = height;
            GradientDim.WidthRequest = width;
        }

        /// <summary>
        /// A method for initializing page when the page is appeared
        /// Grabs move keys of the remote controller
        /// </summary>
        protected override void InitializePage()
        {
            RotateBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(RotateBtn.MediaHubButtonBtn);
            RotateBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(ZoomBtn.MediaHubButtonBtn);

            ZoomBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(RotateBtn.MediaHubButtonBtn);
            ZoomBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(LeftBtn.MediaHubButtonBtn);

            LeftBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(ZoomBtn.MediaHubButtonBtn);
            LeftBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(RightBtn.MediaHubButtonBtn);

            RightBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(LeftBtn.MediaHubButtonBtn);
            RightBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(SlideShowBtn.MediaHubButtonBtn);

            SlideShowBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(RightBtn.MediaHubButtonBtn);
            SlideShowBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(OptionBtn.MediaHubButtonBtn);

            OptionBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(SlideShowBtn.MediaHubButtonBtn);
            OptionBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(OptionBtn.MediaHubButtonBtn);

            RunShowAnimation();
            timer.Change(timerDueTime, Timeout.Infinite);

            GrabRemoteKeys();
        }

        /// <summary>e
        /// A method for hiding control area when timer is over
        /// </summary>
        /// <param name="state">The status to be set</param>
        private void HideControlAreaHandler(object state)
        {
            SetControlAreaState(ControlAreaState.HIDE);
        }

        /// <summary>
        /// A method for finalizing page when the page is disappeared
        /// Ungrabs keys of the remote controller
        /// </summary>
        protected override void FinalizePage()
        {
            UngrabRemoteKeys();

            timer.Change(Timeout.Infinite, Timeout.Infinite);
        }

        /// <summary>
        /// A method for running hide animation
        /// </summary>
        public override void RunHideAnimation()
        {
            GradientDim.FadeTo(0, 667, Easing.CubicInOut);
            ControlArea.TranslateTo(0, 93, 667, Easing.CubicInOut);
            ControlArea.FadeTo(0, 667, Easing.CubicInOut);
            TitleContainer.TranslateTo(0, -50, 667, Easing.CubicInOut);
            TitleContainer.FadeTo(0, 667, Easing.CubicInOut);
        }

        /// <summary>
        /// A method for running show animation
        /// </summary>
        public override void RunShowAnimation()
        {
            GradientDim.Opacity = 0;
            TitleContainer.TranslationY = -50;
            TitleContainer.Opacity = 0;
            ControlArea.TranslationY = 93;
            ControlArea.Opacity = 0;

            GradientDim.FadeTo(1, 667, Easing.CubicInOut);
            TitleContainer.TranslateTo(0, 0, 667, Easing.CubicInOut);
            TitleContainer.FadeTo(1, 667, Easing.CubicInOut);
            ControlArea.TranslateTo(0, 0, 667, Easing.CubicInOut);
            ControlArea.FadeTo(1, 667, Easing.CubicInOut);
        }
    }
}
