﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using TVMediaHub.Tizen.DataModels;
using System.Windows.Input;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.Utils;
using Xamarin.Forms;
using Tizen.Content.MediaContent;
using static TVMediaHub.Tizen.Views.VideoItem;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom view for displaying Video group
    /// </summary>
    public partial class VideoGroup : RelativeLayout
    {
        /// <summary>
        /// A label of group title
        /// </summary>
        private Xamarin.Forms.Label GroupTitle;
        /// <summary>
        /// A grid view of group content area
        /// </summary>
        private Grid GroupContentArea;
        /// <summary>
        /// Gets or sets the focus area of group title
        /// </summary>
        public Button GroupTitleFocusArea { get; protected set; }

        /// <summary>
        /// An event EventHandler for group item's focus event
        /// </summary>
        public EventHandler<GroupItemFocusEventArgs> GroupItemFocused;

        /// <summary>
        /// Identifies the ItemsSource bindable property
        /// </summary>
        public static readonly BindableProperty ItemsSourceProperty = BindableProperty.Create("ItemsSource", typeof(IEnumerable<MediaShortcutInfo>), typeof(VideoTab), default(IEnumerable<MediaShortcutInfo>));

        /// <summary>
        /// A command will be executed when item is clicked
        /// </summary>
        private ICommand ItemClickCommand;

        public delegate void ContextPopupItemSelelectedEventHandler(MediaInformationEx info, VideoContextPopupItem item);

        public ContextPopupItemSelelectedEventHandler ContextPopupItemSelectedHandler;
        /// <summary>
        /// Gets or sets source of list items
        /// </summary>
        public IEnumerable<MediaShortcutInfo> ItemsSource
        {
            get { return (IEnumerable<MediaShortcutInfo>)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        /// <summary>
        /// A list of children's button objects of group
        /// </summary>
        private List<Button> ChildrenFocusList;
        /// <summary>
        /// @TODO:Comment
        /// </summary>
        public List<Button> LeftFocusList { get; protected set; }
        /// <summary>
        /// @TODO:Comment
        /// </summary>
        public List<Button> RightFocusList { get; protected set; }
        /// <summary>
        /// @TODO:Comment
        /// </summary>
        public List<KeyValuePair<double, Button>> BottomFocusList { get; protected set; }

        private VideoItem focusedItem;


        /// <summary>
        /// A list of video items
        /// </summary>
        public List<VideoItem> VideoItemList { get; set; }

        /// <summary>
        /// A constructor
        /// Initializes several lists and properties that are used in this class
        /// Initializes several size of the items that are used in this class
        /// </summary>
        public VideoGroup()
        {
            InitializeComponent();
            Init();
            InitializeSize();
            PropertyChanged += VideoGroupPropertyChanged;
        }

        /// <summary>
        /// A method for initializing several lists and properties that are used in this class
        /// </summary>
        private void Init()
        {
            ChildrenFocusList = new List<Button>();
            LeftFocusList = new List<Button>();
            RightFocusList = new List<Button>();
            BottomFocusList = new List<KeyValuePair<double, Button>>();
            VideoItemList = new List<VideoItem>();

            GroupTitle = new Xamarin.Forms.Label();
            GroupTitleFocusArea = new Button();
            GroupContentArea = new Grid();

            GroupTitle.FontFamily = "BreezSans";
            GroupTitle.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Normal);
            GroupTitle.SetBinding(Xamarin.Forms.Label.TextProperty, new Binding("Title"));
            GroupTitle.TextColor = Color.FromRgba(255, 255, 255, 31);

            GroupTitleFocusArea.Opacity = 0;
            GroupTitleFocusArea.Focused += TitleFocused;
            GroupTitleFocusArea.Unfocused += TitleUnfocused;
        }

        /// <summary>
        /// This method is called when the group's title is unfocused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private void TitleUnfocused(object sender, FocusEventArgs e)
        {
            GroupTitle.AbortAnimation("FocusAnimation");
            GroupTitle.Animate("FocusAnimation",
                (v) =>
                {
                    var R = (int)(255 * v);
                    var G = (int)(129 + 126 * v);
                    var B = (int)(198 + 57 * v);
                    var A = (int)(255 - 224 * v);
                    GroupTitle.TextColor = Color.FromRgba(R, G, B, A);
                },
                length: 167,
                finished: (v, IsCanceled) =>
                {
                    if (!IsCanceled)
                    {
                        GroupTitle.TextColor = Color.FromRgba(255, 255, 255, 31);
                    }
                });
        }

        /// <summary>
        /// This method is called when the group's title is focused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private void TitleFocused(object sender, FocusEventArgs e)
        {
            GroupTitle.AbortAnimation("FocusAnimation");
            GroupTitle.Animate("FocusAnimation",
                (v) => { },
                length: 167,
                finished: (v, IsCanceled) =>
                {
                    if (!IsCanceled)
                    {

                        GroupTitle.Animate("FocusAnimation",
                            (v2) =>
                            {
                                var R = (int)(255 * (1 - v));
                                var G = (int)(255 - 126 * v);
                                var B = (int)(255 - 57 * v);
                                var A = (int)(31 + 224 * v);
                                GroupTitle.TextColor = Color.FromRgba(R, G, B, A);
                            },
                            length: 167,
                            finished: (v2, IsCanceled2) =>
                            {
                                if (!IsCanceled2)
                                {
                                    GroupTitle.TextColor = Color.FromRgba(0, 129, 198, 255);
                                }
                            });
                    }
                });
        }

        /// <summary>
        /// A method for initializing several size of the items that are used in this class
        /// </summary>
        private void InitializeSize()
        {

            Children.Add(GroupTitleFocusArea,
                xConstraint: Constraint.Constant(SizeUtils.GetWidthSize(12)),
                yConstraint: Constraint.Constant(SizeUtils.GetHeightSize(74)),
                widthConstraint: Constraint.Constant(SizeUtils.GetWidthSize(320)),
                heightConstraint: Constraint.Constant(SizeUtils.GetHeightSize(50)));
            Children.Add(GroupTitle,
                xConstraint: Constraint.Constant(SizeUtils.GetWidthSize(12)),
                yConstraint: Constraint.Constant(SizeUtils.GetHeightSize(74)),
                widthConstraint: Constraint.Constant(SizeUtils.GetWidthSize(320)),
                heightConstraint: Constraint.Constant(SizeUtils.GetHeightSize(50)));
            Children.Add(GroupContentArea,
                yConstraint: Constraint.Constant(SizeUtils.GetHeightSize(124)));

            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(332) });
            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(332) });
            GroupContentArea.RowDefinitions.Add(new RowDefinition { Height = SizeUtils.GetHeightSize(332) });

            GroupContentArea.Margin = new Thickness(SizeUtils.GetWidthSize(-64), SizeUtils.GetHeightSize(-64), SizeUtils.GetWidthSize(-64), SizeUtils.GetHeightSize(-64));

            GroupContentArea.RowSpacing = SizeUtils.GetWidthSize(-150);
            GroupContentArea.ColumnSpacing = SizeUtils.GetWidthSize(-150);

            GroupTitle.FontSize = SizeUtils.GetFontSize(28);
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void VideoGroupPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("ItemsSource"))
            {
                var index = 0;
                ChildrenFocusList.Clear();
                VideoItemList.Clear();
                foreach (var item in ItemsSource)
                {
                    var view = new VideoItem();
                    ChildrenFocusList.Add(view.GetFocusArea());
                    view.BindingContext = item;
                    VideoItemList.Add(view);
                    view.OnFocusedEventHandler += (se, ev) =>
                    {
                        GroupItemFocused?.Invoke(view, new GroupItemFocusEventArgs(X + view.X));
                        GroupContentArea.RaiseChild(view);
                        focusedItem = view;
                    };
                    view.OnUnfocusedEventHandler += (se, ev) =>
                    {
                        if (focusedItem != null && focusedItem.Equals(view as VideoItem))
                        {
                            focusedItem = null;
                        }
                    };
                    view.OnItemClickedHandler += (info) =>
                    {
                        ItemClickCommand?.Execute(info);
                    };
                    view.ContextPopupItemSelectedHandler += (info, selItem) =>
                    {
                        ContextPopupItemSelectedHandler?.Invoke(info, selItem);
                    };

                    GroupContentArea.Children.Add(view, index / 3, index % 3);
                    index++;
                }

                SetFocusChain();
            }
        }

        /// <summary>
        /// A method for making focus chain among items
        /// </summary>
        private void SetFocusChain()
        {
            if (ChildrenFocusList.Count < 1)
            {
                return;
            }

            var lastColumn = (ChildrenFocusList.Count - 1) / 3;

            SetFocusList();

            GroupTitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusDownView(ChildrenFocusList[0]);
            GroupTitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(GroupTitleFocusArea);
            GroupTitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(GroupTitleFocusArea);

            for (var i = 0; i < ChildrenFocusList.Count; i++)
            {
                var target = ChildrenFocusList[i];
                var row = i % 3;
                var column = i / 3;

                // Left
                if (column != 0)
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(ChildrenFocusList[i - 3]);
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(target);
                }

                // Right
                if (column != lastColumn)
                {
                    if ((i + 3) < ChildrenFocusList.Count)
                    {
                        target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(ChildrenFocusList[i + 3]);
                    }
                    else
                    {
                        target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(ChildrenFocusList[ChildrenFocusList.Count - 1]);
                    }
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(target);
                }

                // Up
                if (row != 0)
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusUpView(ChildrenFocusList[i - 1]);
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusUpView(GroupTitleFocusArea);
                }

                // Down
                if (row != 2 && (i + 1) < ChildrenFocusList.Count)
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusDownView(ChildrenFocusList[i + 1]);
                }
                else
                {
                    target.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusDownView(target);
                    var targetX = column * SizeUtils.GetWidthSize(322);
                    BottomFocusList.Add(new KeyValuePair<double, Button>(targetX, target));
                }
            }
        }

        /// <summary>
        /// @TODO:comment
        /// </summary>
        private void SetFocusList()
        {
            var totalCount = ChildrenFocusList.Count - 1;
            var lastItemColumn = totalCount / 3;
            var lastItemRow = totalCount % 3;

            var leftFocusFrom = 0;
            var leftFocusTo = lastItemColumn > 0 ? 2 : lastItemRow;

            var rightFocusFrom = lastItemColumn * 3;
            var rightFocusTo = rightFocusFrom + lastItemRow;

            for (var i = leftFocusFrom; i <= leftFocusTo; i++)
            {
                LeftFocusList.Add(ChildrenFocusList[i]);
            }

            for (var i = rightFocusFrom; i <= rightFocusTo; i++)
            {
                RightFocusList.Add(ChildrenFocusList[i]);
            }
        }

        /// <summary>
        /// A method for setting the command to ItemClickCommand
        /// </summary>
        /// <param name="command">A command to be set</param>
        public void SetClickCommand(ICommand command)
        {
            ItemClickCommand = command;
        }

        /// <summary>
        /// If there is focused video item, notify the menu key pressed event to focused video item for show context popup
        /// </summary>
        /// <returns>If group has a focused video item, true; otherwise, false</returns>
        public bool NotifyMenuKeyPressedToFocusedItem()
        {
            if (focusedItem != null)
            {
                focusedItem.ShowContextPopup();
                return true;
            }

            return false;
        }
    }
}
