﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.ComponentModel;
using Tizen.Content.MediaContent;
using Tizen.Xamarin.Forms.Extension;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.DataModels;
using Xamarin.Forms;
using TVMediaHub.Tizen.ViewModels;

namespace TVMediaHub.Tizen.Views
{
    /// <summary>
    /// A custom view for displaying video item
    /// </summary>
    public partial class VideoItem : RelativeLayout
    {
        /// <summary>
        /// A flag that whether the item is loaded or not
        /// </summary>
        private bool IsLoaded = false;

        /// <summary>
        /// An enumeration for ItemStatus
        /// </summary>
        public enum ItemStatus
        {
            Normal = 0,
            Selected,
        };

        public enum VideoContextPopupItem
        {
            FileInfo = 0,
            Delete,
        }

        /// <summary>
        /// A value of current ItemStatus
        /// </summary>
        private ItemStatus CurStatus;

        /// <summary>
        /// Bounds for ImageArea(Image Container) when item is unfocused
        /// </summary>
        private Xamarin.Forms.Rectangle NormalBounds;

        /// <summary>
        /// Bounds for ImageArea(Image Container) when item is focused
        /// </summary>
        private Xamarin.Forms.Rectangle FocusedBounds;

        /// <summary>
        /// Bounds for Shadow Image when item is unfocused
        /// </summary>
        private Xamarin.Forms.Rectangle NormalShadowBounds;

        /// <summary>
        /// Bounds for Shadow Image when item is focused
        /// </summary>
        private Xamarin.Forms.Rectangle FocusedShadowBounds;

        /// <summary>
        /// Bounds for Content Image when item is unfocused
        /// </summary>
        private Xamarin.Forms.Rectangle NormalContentImageBounds;

        /// <summary>
        /// Bounds for Content Image when item is focused
        /// </summary>
        private Xamarin.Forms.Rectangle FocusedContentImageBounds;

        /// <summary>
        /// Bounds for Play Icon
        /// </summary>
        private Xamarin.Forms.Rectangle PlayIconBounds;

        /// <summary>
        /// Bounds for Title
        /// </summary>
        private Xamarin.Forms.Rectangle TitleContainerBounds;

        /// <summary>
        /// An EventHandler for focus event of item
        /// </summary>
        public EventHandler OnFocusedEventHandler;
        /// <summary>
        /// An EventHandler for unfocus event of item
        /// </summary>
        public EventHandler OnUnfocusedEventHandler;
        /// <summary>
        /// A delegate will be executed when the item is clicked
        /// </summary>
        /// <param name="info">A clicked item's MediaInformation</param>
        public delegate void ItemEventHandler(MediaInformationEx info);
        /// <summary>
        /// A ClickEventHandler for click event of the item
        /// </summary>
        public ItemEventHandler OnItemClickedHandler;

        public delegate void ContextPopupItemSelectedEventHandler(MediaInformationEx info, VideoContextPopupItem item);

        public ContextPopupItemSelectedEventHandler ContextPopupItemSelectedHandler;

        /// <summary>
        /// Identifies the VideoTabStatus bindable property
        /// </summary>
        public static readonly BindableProperty VideoTabStatusProperty = BindableProperty.Create("VideoTabStatus", typeof(TabStatus), typeof(VideoItem), TabStatus.Default);

        /// <summary>
        /// Gets or sets VideoTabStatus
        /// </summary>
        public TabStatus VideoTabStatus
        {
            set
            {
                SetValue(VideoTabStatusProperty, value);
            }

            get
            {
                return (TabStatus)GetValue(VideoTabStatusProperty);
            }
        }

        /// <summary>
        /// Identifies the VideoInfo bindable property
        /// </summary>
        public static readonly BindableProperty VideoInfoProperty = BindableProperty.Create("VideoInfo", typeof(MediaInformationEx), typeof(VideoItem), null);

        /// <summary>
        /// Gets or sets the VideoInfo of current video
        /// </summary>
        public MediaInformationEx VideoInfo
        {
            set
            {
                SetValue(VideoInfoProperty, value);
            }

            get
            {
                return (MediaInformationEx)GetValue(VideoInfoProperty);
            }
        }

        /// <summary>
        /// A constructor
        /// Initializes the size of the items that are used in this class
        /// </summary>
        public VideoItem()
        {
            InitializeComponent();
            InitializeSize();
            InitializeLabelSlide();
            PropertyChanged += VideoItemPropertyChanged;
        }

        /// <summary>
        /// A method for initializing label slide
        /// </summary>
        private void InitializeLabelSlide()
        {
            TextSlideEffect.SetIsAlwaysOn(ContentTitle, true);
            TextSlideEffect.SetMode(ContentTitle, TextSlideMode.Long); ;
            TextSlideEffect.SetIsTextSliding(ContentTitle, false);

            ContentTitle.PropertyChanged += SetSlideDuration;
        }

        /// <summary>
        /// This method is called when the ContentTitle property is changed
        /// Sets label slide duration
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void SetSlideDuration(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("Width"))
            {
                double slideDuration = ContentTitle.Width * 12000 / 480;
                TextSlideEffect.SetDuration(ContentTitle, (int)slideDuration);

                ContentTitle.PropertyChanged -= SetSlideDuration;
            }
        }

        /// <summary>
        /// A method for initializing size of the items that are used in this class
        /// </summary>
        private void InitializeSize()
        {
            int w0 = SizeUtils.GetWidthSize(0);
            int h0 = SizeUtils.GetHeightSize(0);
            int w12 = SizeUtils.GetWidthSize(12);
            int h12 = SizeUtils.GetHeightSize(12);
            int w344 = SizeUtils.GetWidthSize(344);
            int h204 = SizeUtils.GetHeightSize(204);

            int h5 = SizeUtils.GetHeightSize(5);
            int w5 = SizeUtils.GetWidthSize(5);

            NormalShadowBounds = new Xamarin.Forms.Rectangle(w12, h12, SizeUtils.GetWidthSize(448), SizeUtils.GetHeightSize(308));
            FocusedShadowBounds = new Xamarin.Forms.Rectangle(w0, h0, SizeUtils.GetWidthSize(472), SizeUtils.GetHeightSize(332));
            FocusedBounds = new Xamarin.Forms.Rectangle(SizeUtils.GetWidthSize(64), SizeUtils.GetHeightSize(64), w344, h204);
            NormalBounds = new Xamarin.Forms.Rectangle(SizeUtils.GetWidthSize(76), SizeUtils.GetHeightSize(76), SizeUtils.GetWidthSize(320), SizeUtils.GetHeightSize(180));

            NormalContentImageBounds = new Xamarin.Forms.Rectangle(-w12, -h12, w344, h204);
            FocusedContentImageBounds = new Xamarin.Forms.Rectangle(w0, h0, w344, h204);

            PlayIconBounds = new Xamarin.Forms.Rectangle(SizeUtils.GetWidthSize(208), SizeUtils.GetHeightSize(126), SizeUtils.GetWidthSize(80), SizeUtils.GetHeightSize(80));

            TitleContainerBounds = new Xamarin.Forms.Rectangle(SizeUtils.GetWidthSize(94), SizeUtils.GetHeightSize(214), SizeUtils.GetWidthSize(284), SizeUtils.GetHeightSize(30) * 1.3);

            WidthRequest = SizeUtils.GetWidthSize(472);
            HeightRequest = SizeUtils.GetHeightSize(332);

            BgDimImage.BorderTop = SizeUtils.GetHeightSize(124);
            BgDimImage.BorderBottom = SizeUtils.GetHeightSize(124);
            BgDimImage.BorderLeft = SizeUtils.GetWidthSize(124);
            BgDimImage.BorderRight = SizeUtils.GetWidthSize(124);
            BgDimImage.Opacity = 0;

            DimImage.BorderTop = SizeUtils.GetHeightSize(20);
            DimImage.BorderBottom = SizeUtils.GetHeightSize(20);
            DimImage.BorderLeft = SizeUtils.GetWidthSize(20);
            DimImage.BorderRight = SizeUtils.GetWidthSize(20);

            CheckDimImage.BorderTop = h5;
            CheckDimImage.BorderBottom = h5;
            CheckDimImage.BorderLeft = w5;
            CheckDimImage.BorderRight = w5;

            CheckImage.BorderTop = h5;
            CheckImage.BorderBottom = h5;
            CheckImage.BorderLeft = w5;
            CheckImage.BorderRight = w5;

            FocusArea.HeightRequest = SizeUtils.GetHeightSize(50);
            FocusArea.WidthRequest = SizeUtils.GetWidthSize(50);

            ContentTitle.FontSize = SizeUtils.GetFontSize(26);

            CheckImage.ImageColor = Color.White;
        }

        /// <summary>
        /// Positions and sizes the children of a Layout.
        /// </summary>
        /// <param name="x">A value representing the x coordinate of the child region bounding box.</param>
        /// <param name="y">A value representing the y coordinate of the child region bounding box.</param>
        /// <param name="width">A value representing the width of the child region bounding box.</param>
        /// <param name="height">A value representing the height of the child region bounding box.</param>
        protected override void LayoutChildren(double x, double y, double width, double height)
        {
            if (IsLoaded == false)
            {
                FocusArea.Layout(NormalBounds);
                BgDimImage.Layout(NormalShadowBounds);
                ImageArea.Layout(NormalBounds);
                ContentImage.Layout(NormalContentImageBounds);
                DimImage.Layout(NormalBounds);
                TitleContainer.Layout(TitleContainerBounds);
                PlayImage.Layout(PlayIconBounds);
                CheckDimImage.Layout(NormalBounds);
                CheckImage.Layout(NormalBounds);
                UnavailableIcon.Layout(NormalBounds);
                IsLoaded = true;
            }
        }

        /// <summary>
        /// A method for getting item's button object
        /// </summary>
        /// <returns>A button object</returns>
        public Button GetFocusArea()
        {
            return FocusArea;
        }

        /// <summary>
        /// This method is called when item is unfocused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        public void OnUnfocused(object sender, FocusEventArgs e)
        {
            OnUnfocusedEventHandler?.Invoke(sender, e);
            Easing easing = new Easing(EasingFunction.EasyIn2);
            this.AbortAnimation("TextSliding");
            this.AbortAnimation("FocusAnimation");
            TextSlideEffect.SetIsTextSliding(ContentTitle, false);

            BgDimImage.Opacity = 0;
            ContentImage.ScaleTo(1.0, 167, easing);
            DimImage.LayoutTo(NormalBounds, 167, easing);
            ContentImage.LayoutTo(NormalContentImageBounds, 167, easing);
            BgDimImage.LayoutTo(NormalShadowBounds, 167, easing);

            if (VideoInfo.IsAvailable)
            {
                PlayImage.FadeTo(0.0, 167);
            }

            if (VideoTabStatus == TabStatus.Delete)
            {
                CheckDimImage.LayoutTo(NormalBounds, 167, easing);
                CheckImage.LayoutTo(NormalBounds, 167, easing);
            }

            ImageArea.LayoutTo(NormalBounds, 167, easing);
        }

        /// <summary>
        /// This method is called when item is focused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        public void OnFocused(object sender, FocusEventArgs e)
        {
            Easing easing = new Easing(EasingFunction.EasyIn1);
            this.Animate("TextSliding",
                (v) => { },
                length: 2000,
                finished: (v, IsCanceled) =>
                {
                    if (!IsCanceled)
                    {
                        TextSlideEffect.SetIsTextSliding(ContentTitle, true);
                    }
                });

            this.Animate("FocusAnimation",
                (v) => { },
                length: 167,
                finished: (v, IsCanceled) =>
                {
                    if (!IsCanceled)
                    {
                        BgDimImage.Opacity = 0.99;
                        OnFocusedEventHandler?.Invoke(sender, e);
#pragma warning disable CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
                        ContentImage.ScaleTo(1.3, 167, easing);
                        DimImage.LayoutTo(FocusedBounds, 500, easing);
                        ContentImage.LayoutTo(FocusedContentImageBounds, 500, easing);
                        BgDimImage.LayoutTo(FocusedShadowBounds, 500, easing);

                        if (VideoInfo.IsAvailable)
                        {
                            PlayImage.FadeTo(0.99, 167);
                        }

                        if (VideoTabStatus == TabStatus.Delete)
                        {
                            CheckDimImage.LayoutTo(FocusedBounds, 167, easing);
                            CheckImage.LayoutTo(FocusedBounds, 167, easing);
                        }
#pragma warning restore CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
                        ImageArea.LayoutTo(FocusedBounds, 500, easing);
                    }
                });
        }

        /// <summary>
        /// This method is called when item is focused
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A Focus event's argument</param>
        private async void OnItemClicked(object sender, EventArgs e)
        {
            if (DimImage.AnimationIsRunning("LayoutTo"))
            {
                return;
            }
#pragma warning disable CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
            DimImage.LayoutTo(NormalBounds, 167);
            ContentImage.LayoutTo(NormalContentImageBounds, 167);
            BgDimImage.LayoutTo(NormalShadowBounds, 167);
            CheckDimImage.LayoutTo(NormalBounds, 167);
            CheckImage.LayoutTo(NormalBounds, 167);
            await ImageArea.LayoutTo(NormalBounds, 167);

            DimImage.LayoutTo(FocusedBounds, 167);
            ContentImage.LayoutTo(FocusedContentImageBounds, 167);
            BgDimImage.LayoutTo(FocusedShadowBounds, 167);
            CheckDimImage.LayoutTo(FocusedBounds, 167);
            CheckImage.LayoutTo(FocusedBounds, 167);
            ImageArea.LayoutTo(FocusedBounds, 167);
            OnItemClickedHandler?.Invoke(VideoInfo);

            if (VideoTabStatus == TabStatus.Delete)
            {
                if (CurStatus == ItemStatus.Normal)
                {
                    CurStatus = ItemStatus.Selected;
                    PlayImage.FadeTo(0, 167);
                }
                else
                {
                    CurStatus = ItemStatus.Normal;
                    PlayImage.FadeTo(0.99, 167);
                }
#pragma warning restore CS4014
                UpdateView();
            }
            else
            {
                PushNavigationPage();
            }
        }

        public void UpdateViewSelectAll(bool isSelectedAll)
        {
            if (!isSelectedAll)
            {
                CheckDimImage.FadeTo(0.0, 167);
                CheckImage.FadeTo(0.0, 167);
                CurStatus = ItemStatus.Normal;
            }
            else
            {
                CheckDimImage.FadeTo(0.75, 167);
                CheckImage.FadeTo(0.99, 167);
                CurStatus = ItemStatus.Selected;
            }
        }

        /// <summary>
        /// A method for updating view according to current status
        /// </summary>
        private void UpdateView()
        {
            switch (CurStatus)
            {
                case ItemStatus.Normal:
                    CheckDimImage.FadeTo(0.0, 167);
                    CheckImage.FadeTo(0.0, 167);
                    break;
                case ItemStatus.Selected:
                    CheckDimImage.FadeTo(0.75, 167);
                    CheckImage.FadeTo(0.99, 167);
                    break;
            }
        }

        /// <summary>
        /// A method for pushing VideoPlayer page at MainPage of TV MediaHub
        /// </summary>
        private void PushNavigationPage()
        {
            Program.TransitionTo(new VideoPlayer());
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void VideoItemPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName.CompareTo("VideoTabStatus") == 0)
            {
                if (VideoTabStatus == TabStatus.Default)
                {
                    CurStatus = ItemStatus.Normal;
                    UpdateView();
                }
            }
            else if (e.PropertyName.CompareTo("VideoInfo") == 0)
            {
                if (VideoInfo.IsAvailable == false)
                {
                    ContentImage.IsVisible = false;
                    UnavailableIcon.IsVisible = true;
                }
            }

        }

        public void ShowContextPopup()
        {
            if (!FocusArea.IsFocused)
            {
                return;
            }

            ContextPopup popup = new ContextPopup();
            popup.Items.Add(new ContextPopupItem("          FILE INFO           "));
            popup.Items.Add(new ContextPopupItem("DELETE"));
            popup.SelectedIndexChanged += SelectedIndexChanged;
            popup.Show(this, this.Width / 2, this.Height + SizeUtils.GetHeightSize(72));

        }

        private void SelectedIndexChanged(object sender, EventArgs e)
        {
            var ctxPopup = sender as ContextPopup;
            switch (ctxPopup.SelectedIndex)
            {
                case 0:
                    ContextPopupItemSelectedHandler.Invoke(VideoInfo, VideoContextPopupItem.FileInfo);
                    break;
                case 1:
                    ContextPopupItemSelectedHandler?.Invoke(VideoInfo, VideoContextPopupItem.Delete);
                    break;
            }

            ctxPopup.Dismiss();
        }
    }
}
