﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using Tizen.Content.MediaContent;
using Tizen.Multimedia;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.ViewModels;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Controls;
using Xamarin.Forms;
using Extension = Tizen.Xamarin.Forms.Extension;
using System.Threading;
using System.Windows.Input;
using Xamarin.Forms.PlatformConfiguration.TizenSpecific;

namespace TVMediaHub.Tizen.Views
{
    using Tizen = Xamarin.Forms.PlatformConfiguration.Tizen;

    /// <summary>
    /// A custom ContentPage for displaying the video player
    /// </summary>
    public partial class VideoPlayer : ContentPageEx
    {
        /// <summary>
        /// Enumeration for the state of the control area
        /// </summary>
        enum ControlAreaState
        {
            SHOW = 0,
            HIDE,
            AUTO,
        };

        enum PlayStatus
        {
            Play = 0,
            Rewind,
            FastFoward,
            Pause
        }

        /// <summary>
        /// The media duration in Milliseconds.
        /// </summary>
        private int sourceDuration;

        /// <summary>
        /// The instance of the player
        /// </summary>
        private Player playerInstance;

        /// <summary>
        /// The instance of the player display
        /// </summary>
        private Display playerDisplayInstance;

        /// <summary>
        /// The instance of the media view
        /// </summary>
        private Extension.MediaView mediaView;

        /// <summary>
        /// The media source
        /// </summary>
        private MediaUriSource videoMediaSource;

        /// <summary>
        /// The height size(72)
        /// </summary>
        private double height72 = SizeUtils.GetHeightSize(72);

        /// <summary>
        /// The key name list to handle key event
        /// </summary>
        private string[] remoteKeys = new string[] { "Up", "Down", "Left", "Right", "XF86Menu" };

        /// <summary>
        /// The timer to handle the control area
        /// </summary>
        private Timer timer;

        /// <summary>
        /// The due time of the timer
        /// 5 second
        /// </summary>
        private int timerDueTime = 5000;

        /// <summary>
        /// Gets or sets current play status
        /// </summary>
        private PlayStatus CurrentPlayStatus { get; set; }

        /// <summary>
        /// The value for play back forward index
        /// </summary>
        private int playBackForwardIndex;

        /// <summary>
        /// Gets or sets play back forward index
        /// </summary>
        private int PlayBackForwardIndex
        {
            set
            {
                playBackForwardIndex = value;
            }

            get
            {
                return playBackForwardIndex % PlayBackRate.Length;
            }
        }

        /// <summary>
        /// The value for play back rewind index
        /// </summary>
        private int playBackRewindIndex;

        /// <summary>
        /// Gets or sets play back rewind index
        /// </summary>
        private int PlayBackRewindIndex
        {
            set
            {
                playBackRewindIndex = value;
            }

            get
            {
                return playBackRewindIndex % PlayBackRate.Length;
            }
        }

        /// <summary>
        /// The value for the playback rate
        /// </summary>
        private float[] PlayBackRate = { 1, 2, 4 };

        /// <summary>
        /// Identifies the CurrenVideo bindable property
        /// </summary>
        public static readonly BindableProperty CurrentVideoProperty = BindableProperty.Create("CurrentVideo", typeof(MediaInformationEx), typeof(VideoPlayer), null);

        /// <summary>
        /// Gets or sets CurrentVideo
        /// </summary>
        public MediaInformationEx CurrentVideo
        {
            get { return (MediaInformationEx)GetValue(CurrentVideoProperty); }
            set { SetValue(CurrentVideoProperty, value); }
        }

        /// <summary>
        /// Identifies the SetNextVideoCommand bindable property
        /// </summary>
        public static readonly BindableProperty SetNextVideoCommandProperty = BindableProperty.Create("SetNextVideoCommand", typeof(ICommand), typeof(VideoPlayer), null);

        /// <summary>
        /// A Command for setting current video to next video
        /// </summary>
        public ICommand SetNextVideoCommand
        {
            get { return (ICommand)GetValue(SetNextVideoCommandProperty); }
            set { SetValue(SetNextVideoCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the SetPrevVideoCommand bindable property
        /// </summary>
        public static readonly BindableProperty SetPrevVideoCommandProperty = BindableProperty.Create("SetPrevVideoCommand", typeof(ICommand), typeof(VideoPlayer), null);

        /// <summary>
        /// A Command for setting current video to previous video
        /// </summary>
        public ICommand SetPrevVideoCommand
        {
            get { return (ICommand)GetValue(SetPrevVideoCommandProperty); }
            set { SetValue(SetPrevVideoCommandProperty, value); }
        }

        /// <summary>
        /// A constructor
        /// </summary>
        public VideoPlayer()
        {
            BindingContext = VideoTabViewModelLocator.ViewModel;
            InitializeComponent();
            InitializeFontSize();
            PropertyChanged += VideoPlayePropertyChanged;
            mediaView = new Extension.MediaView();

            mediaView.NativeViewCreated += (s, e) =>
            {
                InitializePlayerDisplay();
                InitializePlayer();
                PrepareAsync();
            };

            mediaView.IsVisible = true;
            mediaView.WidthRequest = 1920;
            mediaView.HeightRequest = 1080;
            MainLayout.Children.Insert(3, mediaView);

            InitializeButtons();

            timer = new Timer(HideControlAreaHandler, null, Timeout.Infinite, Timeout.Infinite);

            OnBackButtonPressedHandler += async (s, e) =>
           {
               try
               {
                   playerInstance?.Stop();
                   playerInstance?.Unprepare();
               }
               catch (Exception ex)
               {
                   DbgPort.E("Stop Failed : " + ex.Message);
               }

#pragma warning disable CS4014
               GradientDim.FadeTo(0, 667, Easing.CubicInOut);
               ControlArea.TranslateTo(0, 93, 667, Easing.CubicInOut);
               ControlArea.FadeTo(0, 667, Easing.CubicInOut);
               TitleLabel.TranslateTo(0, -50, 667, Easing.CubicInOut);
               await TitleLabel.FadeTo(0, 667, Easing.CubicInOut);

               App.AppMainPage.Navigation.PopAsync();
#pragma warning restore CS4014
           };
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void VideoPlayePropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("CurrentVideo"))
            {
                if (playerInstance.State == PlayerState.Playing || playerInstance.State == PlayerState.Paused)
                {
                    try
                    {
                        playerInstance.Stop();
                        playerInstance.Unprepare();
                        playerDisplayInstance = new Display((MediaView)mediaView.NativeView);
                        playerInstance.Display = playerDisplayInstance;
                        videoMediaSource = new MediaUriSource(CurrentVideo.MediaContentInformation.Path);
                        playerInstance.SetSource(videoMediaSource);
                    }
                    catch (Exception ex)
                    {
                        DbgPort.D(ex.Message);
                        return;
                    }

                    PrepareAsync();
                }
            }
        }

        /// <summary>
        /// Grabs the key of the remote controller
        /// </summary>
        private void GrabRemoteKeys()
        {
            App.KeyDownEvent.On += RemoteKeyDownListener;
        }

        /// <summary>
        /// Ungrabs the key of the remote controller
        /// </summary>
        private void UngrabRemoteKeys()
        {
            App.KeyDownEvent.On -= RemoteKeyDownListener;
        }

        /// <summary>
        /// KeyDown will be triggered when key is preesd down
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">The information about the event</param>
        private void RemoteKeyDownListener(object sender, EventArgs e)
        {
            SetControlAreaState(ControlAreaState.AUTO);
        }

        /// <summary>
        /// Sets the status of the control area
        /// </summary>
        /// <param name="state">The status to be set</param>
        private void SetControlAreaState(ControlAreaState state)
        {
            switch (state)
            {
                case ControlAreaState.SHOW:
                    if (ControlArea.Opacity == 0)
                    {
                        RunShowAnimation();
                    }

                    timer.Change(Timeout.Infinite, Timeout.Infinite);

                    break;
                case ControlAreaState.HIDE:
                    if (ControlArea.Opacity == 1)
                    {
                        RunHideAnimation();
                    }

                    timer.Change(Timeout.Infinite, Timeout.Infinite);

                    break;
                case ControlAreaState.AUTO:
                default:
                    if (ControlArea.Opacity == 0)
                    {
                        RunShowAnimation();
                    }

                    timer.Change(timerDueTime, Timeout.Infinite);

                    break;
            }
        }

        /// <summary>
        /// Initialize the timer of the player
        /// </summary>
        private void InitializeTimer()
        {
            Device.StartTimer(new TimeSpan(0, 0, 0, 0, 100), TimerElapsed);
        }

        /// <summary>
        /// The previous status of the player
        /// </summary>
        PlayerState prevPlayerStatus = PlayerState.Idle;

        /// <summary>
        /// Display elapsed time of the media source
        /// </summary>
        /// <returns>If the player state is playing or paused, false; otherwise, true</returns>
        private bool TimerElapsed()
        {
            Device.BeginInvokeOnMainThread(() =>
            {
                if (playerInstance.State != PlayerState.Ready &&
                    playerInstance.State != PlayerState.Paused &&
                    playerInstance.State != PlayerState.Playing)
                {
                    return;
                }

                int playPosition = playerInstance.GetPlayPosition();
                SetPlayTime(playPosition);
                progress.Progress = (double)playPosition / sourceDuration;
                //DbgPort.D("Status : " + playerInstance.State + "/" + prevPlayerStatus + ", temp : " + playPosition + ", progress : " + progress.Progress.ToString());

                if (prevPlayerStatus != playerInstance.State)
                {
                    if (playerInstance.State == PlayerState.Paused || playerInstance.State == PlayerState.Ready)
                    {
                        if (CurrentPlayStatus == PlayStatus.Rewind || CurrentPlayStatus == PlayStatus.FastFoward)
                        {
                            playerInstance.SetPlaybackRate((PlayBackRate[0]));
                        }

                        PausePlayBtn.PressedImage = PausePlayBtn.PressedImage.Replace("pause", "play");
                        PausePlayBtn.NormalImage = PausePlayBtn.NormalImage.Replace("pause", "play");
                        PausePlayBtn.MediaHubButtonIcon = PausePlayBtn.MediaHubButtonIcon.Replace("pause", "play");
                    }
                    else if (playerInstance.State == PlayerState.Playing)
                    {
                        PausePlayBtn.PressedImage = PausePlayBtn.PressedImage.Replace("play", "pause");
                        PausePlayBtn.NormalImage = PausePlayBtn.NormalImage.Replace("play", "pause");
                        PausePlayBtn.MediaHubButtonIcon = PausePlayBtn.MediaHubButtonIcon.Replace("play", "pause");
                    }

                    prevPlayerStatus = playerInstance.State;
                }
            });

            if (playerInstance.State != PlayerState.Playing &&
                playerInstance.State != PlayerState.Paused)
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Initialize the player display
        /// </summary>
        private void InitializePlayerDisplay()
        {
            playerDisplayInstance = new Display((MediaView)mediaView.NativeView);
            DbgPort.D("Player Display has been created");
        }

        /// <summary>
        /// Initialize the player
        /// </summary>
        private void InitializePlayer()
        {
            // Create Player Instance
            playerInstance = new Player();
            playerInstance.Display = playerDisplayInstance;
            playerInstance.PlaybackCompleted += (obj, eventArg) =>
            {
                PlayBackRewindIndex = 0;
                PlayBackForwardIndex = 0;
                DbgPort.D("PlaybackCompleted");

                if (CurrentPlayStatus == PlayStatus.Rewind)
                {
                    try
                    {
                        CurrentPlayStatus = PlayStatus.Play;
                        playerInstance?.SetPlaybackRate(1);
                        playerInstance?.Start();
                    }
                    catch (Exception ex)
                    {
                        DbgPort.E("Start Failed : " + ex.Message);
                        return;
                    }
                }
                else if (VideoTabViewModelLocator.ViewModel.IsLastVideo)
                {
                    App.AppMainPage.Navigation.PopAsync();
                }
                else
                {
                    try
                    {
                        CurrentPlayStatus = PlayStatus.Play;
                        SetNextVideoCommand?.Execute("");
                    }
                    catch (Exception ex)
                    {
                        DbgPort.E("Stop Failed : " + ex.Message);
                    }
                }
            };

            videoMediaSource = new MediaUriSource(CurrentVideo.MediaContentInformation.Path);
            playerInstance.SetSource(videoMediaSource);
        }

        /// <summary>
        /// Prepares the media player for playback, asynchronously
        /// </summary>
        private async void PrepareAsync()
        {
            DbgPort.D("Prepare player");
            await playerInstance.PrepareAsync();
            DbgPort.D("Player prepared, state : " + playerInstance.State.ToString());

            if (playerInstance.State == PlayerState.Ready)
            {
                sourceDuration = playerInstance.StreamInfo.GetDuration();
                SetRemainTime();
                // If you don't want to play automatically whenever VideoPlayer is opened, remove below line.
                // TODO: Remove this later, only for TDC
                PlayVideo();
                InitializeTimer();
                RunningTimeArea.IsVisible = true;
            }
        }

        /// <summary>
        /// Sets the play time on the control area
        /// </summary>
        /// <param name="time">A current playing time</param>
        private void SetPlayTime(int time)
        {
            int second = (time / 1000) % 60;
            int minute = (time / (1000 * 60)) % 60;
            int hour = (time / (1000 * 60 * 60)) % 24;

            PlayTimeHr.Text = string.Format("{0:D2}", hour);
            PlayTimeMin.Text = string.Format("{0:D2}", minute);
            PlayTimeSec.Text = string.Format("{0:D2}", second);
        }

        /// <summary>
        /// Sets the remain time on the control area
        /// </summary>
        private void SetRemainTime()
        {
            int second = (sourceDuration / 1000) % 60;
            int minute = (sourceDuration / (1000 * 60)) % 60;
            int hour = (sourceDuration / (1000 * 60 * 60)) % 24;

            RemainTimeHr.Text = string.Format("{0:D2}", hour);
            RemainTimeMin.Text = string.Format("{0:D2}", minute);
            RemainTimeSec.Text = string.Format("{0:D2}", second);
        }

        /// <summary>
        /// A method for initializing page when the page is appeared
        /// </summary>
        protected override void InitializePage()
        {
            DbgPort.D("Initialize Page");

            PreviousBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(PreviousBtn.MediaHubButtonBtn);
            PreviousBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(RewindBtn.MediaHubButtonBtn);

            RewindBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(PreviousBtn.MediaHubButtonBtn);
            RewindBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(PausePlayBtn.MediaHubButtonBtn);

            PausePlayBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(RewindBtn.MediaHubButtonBtn);
            PausePlayBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(ForwardBtn.MediaHubButtonBtn);

            ForwardBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(PausePlayBtn.MediaHubButtonBtn);
            ForwardBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(NextBtn.MediaHubButtonBtn);

            NextBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusLeftView(ForwardBtn.MediaHubButtonBtn);
            NextBtn.MediaHubButtonBtn.On<Tizen>().SetNextFocusRightView(NextBtn.MediaHubButtonBtn);

            RunShowAnimation();
            timer.Change(timerDueTime, Timeout.Infinite);

            PausePlayBtn.MediaHubButtonBtn.Focus();

            GrabRemoteKeys();
        }

        /// <summary>e
        /// A method for hiding control area when timer is over
        /// </summary>
        /// <param name="state">The status to be set</param>
        private void HideControlAreaHandler(object state)
        {
            SetControlAreaState(ControlAreaState.HIDE);
        }

        /// <summary>
        /// A method for finalizing page when the page is disappeared
        /// </summary>
        protected override void FinalizePage()
        {
            UngrabRemoteKeys();

            timer.Change(Timeout.Infinite, Timeout.Infinite);
        }

        /// <summary>
        /// Starts or resumes playback
        /// </summary>
        private void PlayVideo()
        {
            try
            {
                playerInstance.Start();
            }
            catch (Exception ex)
            {
                DbgPort.E("Start Failed : " + ex.Message);
                return;
            }

            DbgPort.D("Play!!");
        }

        /// <summary>
        /// Initialize the font size for the player
        /// </summary>
        private void InitializeFontSize()
        {
            int fontSize44 = SizeUtils.GetFontSize(44);
            int fontSize40 = SizeUtils.GetFontSize(40);
            int fontSize28 = SizeUtils.GetFontSize(28);

            TitleLabel.FontSize = fontSize44;

            PlayTimeHr.FontSize = fontSize28;
            PlayTimeColon.FontSize = fontSize28;
            PlayTimeMin.FontSize = fontSize28;
            PlayTimeColon2.FontSize = fontSize28;
            PlayTimeSec.FontSize = fontSize28;

            RemainTimeHr.FontSize = fontSize28;
            RemainTimeColon.FontSize = fontSize28;
            RemainTimeMin.FontSize = fontSize28;
            RemainTimeColon2.FontSize = fontSize28;
            RemainTimeSec.FontSize = fontSize28;

            XText.FontSize = fontSize44;
            NumText.FontSize = fontSize40;

            TitleLabel.On<Tizen>().SetFontWeight(FontWeight.Bold);
            PlayTimeHr.On<Tizen>().SetFontWeight(FontWeight.Normal);
            PlayTimeColon.On<Tizen>().SetFontWeight(FontWeight.Normal);
            PlayTimeMin.On<Tizen>().SetFontWeight(FontWeight.Normal);
            PlayTimeColon2.On<Tizen>().SetFontWeight(FontWeight.Normal);
            PlayTimeSec.On<Tizen>().SetFontWeight(FontWeight.Normal);
            RemainTimeHr.On<Tizen>().SetFontWeight(FontWeight.Light);
            RemainTimeColon.On<Tizen>().SetFontWeight(FontWeight.Light);
            RemainTimeMin.On<Tizen>().SetFontWeight(FontWeight.Light);
            RemainTimeColon2.On<Tizen>().SetFontWeight(FontWeight.Light);
            RemainTimeSec.On<Tizen>().SetFontWeight(FontWeight.Light);
            XText.On<Tizen>().SetFontWeight(FontWeight.Light);
            NumText.On<Tizen>().SetFontWeight(FontWeight.Light);
        }

        /// <summary>
        /// Initialize control buttons for the player
        /// </summary>
        private void InitializeButtons()
        {
            MediaHubButton[] buttons = new MediaHubButton[5] { PreviousBtn, RewindBtn, PausePlayBtn, ForwardBtn, NextBtn };
            string[] icons = new string[5] { "previous", "rewind", "play", "forward", "next" };
            string iconPath = "btn_viewer_control_previous_normal.png";

            for (int i = 0; i < buttons.Length; i++)
            {
                buttons[i].NormalImage = iconPath.Replace("previous", icons[i]);
                buttons[i].PressedImage = buttons[i].NormalImage.Replace("normal", "pressed");
                buttons[i].MediaHubButtonIcon = buttons[i].NormalImage;
            }
        }

        /// <summary>
        /// This method is called when previous button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private async void PreviousBtnClicked(object sender, EventArgs e)
        {
            if (playerInstance.State == PlayerState.Idle || playerInstance.State == PlayerState.Preparing)
            {
                return;
            }

            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            if (playerInstance.GetPlayPosition() < 1000)
            {
                SetPrevVideoCommand?.Execute("");
                return;
            }
            else
            {
                try
                {
                    await playerInstance.SetPlayPositionAsync(0, true);
                }
                catch (Exception ex)
                {
                    DbgPort.E("SetPlayPositionAsync Failed : " + ex.Message);
                    return;
                }

                progress.Progress = 0.0;
            }
        }

        /// <summary>
        /// This method is called when rewind button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void RewindBtnClicked(object sender, EventArgs e)
        {
            PlayBackForwardIndex = 0;
            if (SpeedInfo.Opacity == 1)
            {
#pragma warning disable CS4014
                SpeedInfo.FadeTo(0.0, 100);
#pragma warning restore CS4014
            }

            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);
            /*
            int playPosition = playerInstance.GetPlayPosition();
            playPosition /= 2;
            SetPlayTime(playPosition);
            progress.Progress = (double)playPosition / duration;

            try
            {
                await playerInstance.SetPlayPositionAsync(playPosition, true);
            }
            catch (Exception ex)
            {
                DbgPort.E("SetPlayPositionAsync Failed : " + ex.Message);
            }
            */
            try
            {
                CurrentPlayStatus = PlayStatus.Rewind;
                PlayBackRewindIndex++;
                playerInstance.SetPlaybackRate((PlayBackRate[PlayBackRewindIndex]) * -1);
                // TODO : check later, added as hotfix for VD GBM player
                if (playerInstance.State == PlayerState.Paused || playerInstance.State == PlayerState.Ready)
                {
                    PlayVideo();
                }

                SpeedInfo.IsVisible = true;
                SpeedInfo.FadeTo(1.0, 100);
                NumText.Text = PlayBackRate[PlayBackRewindIndex].ToString();
            }
            catch (Exception ex)
            {
                DbgPort.E("Rewind Set PlayBackRate Failed : " + ex.Message);
            }
        }

        /// <summary>
        /// This method is called when pause/play button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void PausePlayBtnClicked(object sender, EventArgs e)
        {
            if (SpeedInfo.Opacity == 1)
            {
#pragma warning disable CS4014
                SpeedInfo.FadeTo(0.0, 100);
#pragma warning restore CS4014
            }

            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            if (playerInstance.State == PlayerState.Paused || playerInstance.State == PlayerState.Ready)
            {
                SetControlAreaState(ControlAreaState.SHOW);
                CurrentPlayStatus = PlayStatus.Play;
                PlayVideo();
            }
            else if (playerInstance.State == PlayerState.Playing)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                try
                {
                    CurrentPlayStatus = PlayStatus.Pause;
                    playerInstance.Pause();
                }
                catch (Exception ex)
                {
                    DbgPort.E("Pause Failed : " + ex.Message);
                }
            }
        }

        /// <summary>
        /// This method is called when forward button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void ForwardBtnClicked(object sender, EventArgs e)
        {
            PlayBackRewindIndex = 0;
            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);

                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            try
            {
                CurrentPlayStatus = PlayStatus.FastFoward;
                PlayBackForwardIndex++;
                playerInstance.SetPlaybackRate(PlayBackRate[PlayBackForwardIndex]);
                // TODO : check later, added as hotfix for VD GBM player
                if (playerInstance.State == PlayerState.Paused || playerInstance.State == PlayerState.Ready)
                {
                    PlayVideo();
                }

                SpeedInfo.IsVisible = true;
                SpeedInfo.FadeTo(1.0, 100);
                NumText.Text = PlayBackRate[PlayBackForwardIndex].ToString();
            }
            catch (Exception err)
            {
                DbgPort.E(err.Message);
            }
        }

        /// <summary>
        /// This method is called when next button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private async void NextBtnClicked(object sender, EventArgs e)
        {
            if (playerInstance.State == PlayerState.Idle || playerInstance.State == PlayerState.Preparing)
            {
                return;
            }

            if (ControlArea.Opacity == 0)
            {
                SetControlAreaState(ControlAreaState.AUTO);
                return;
            }

            SetControlAreaState(ControlAreaState.AUTO);

            if (playerInstance.GetPlayPosition() > sourceDuration - 1000)
            {
                SetNextVideoCommand?.Execute("");
            }
            else
            {
                try
                {
                    await playerInstance.SetPlayPositionAsync(sourceDuration - 1000, true);
                }
                catch (Exception err)
                {
                    DbgPort.E("SetPlayPositionAsync Failed : " + err.Message);
                    return;
                }

            }

            progress.Progress = 1.0;
        }

        /// <summary>
        /// A method for running hide animation
        /// </summary>
        public override void RunHideAnimation()
        {
            if (SpeedInfo.Opacity == 1)
            {
                SpeedInfo.FadeTo(0.0, 100);
            }

            GradientDim.FadeTo(0, 667, Easing.CubicInOut);
            ControlArea.TranslateTo(0, 93, 667, Easing.CubicInOut);
            ControlArea.FadeTo(0, 667, Easing.CubicInOut);
            TitleLabel.TranslateTo(0, -50, 667, Easing.CubicInOut);
            TitleLabel.FadeTo(0, 667, Easing.CubicInOut);
        }

        /// <summary>
        /// A method for running show animation
        /// </summary>
        public override void RunShowAnimation()
        {
            GradientDim.Opacity = 0;
            TitleLabel.TranslationY = -50;
            TitleLabel.Opacity = 0;
            ControlArea.TranslationY = 93;
            ControlArea.Opacity = 0;

            GradientDim.FadeTo(1, 667, Easing.CubicInOut);
            ControlArea.TranslateTo(0, 0, 667, Easing.CubicInOut);
            ControlArea.FadeTo(1, 667, Easing.CubicInOut);
            TitleLabel.TranslateTo(0, 0, 667, Easing.CubicInOut);
            TitleLabel.FadeTo(1, 667, Easing.CubicInOut);

            if (CurrentPlayStatus == PlayStatus.Rewind || CurrentPlayStatus == PlayStatus.FastFoward)
            {
                SpeedInfo.FadeTo(1.0, 100);
            }
        }
    }
}
