﻿/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Input;
using TVMediaHub.Tizen.Controls;
using TVMediaHub.Tizen.DataModels;
using TVMediaHub.Tizen.Models;
using TVMediaHub.Tizen.Utils;
using TVMediaHub.Tizen.ViewModels;
using Xamarin.Forms;
using TizenVisualElement = Xamarin.Forms.PlatformConfiguration.TizenSpecific.VisualElement;

namespace TVMediaHub.Tizen.Views
{
    using Xamarin.Forms.PlatformConfiguration.TizenSpecific;
    using static TVMediaHub.Tizen.Views.VideoItem;
    /// <summary>
    /// A custom ContentPage for displaying the video tab
    /// </summary>
    public partial class VideoTab : ContentPageEx
    {
        /// <summary>
        /// The flag whether content is ready
        /// </summary>
        private bool IsContentReady = false;

        /// <summary>
        /// The flag that whether all contents are selected or not
        /// </summary>
        private bool IsSelectedAll { get; set; }

        public List<VideoGroup> VideoGroupList { get; set; }

        /// <summary>
        /// Identifies the ItemsSource bindable property
        /// </summary>
        public static readonly BindableProperty ItemsSourceProperty = BindableProperty.Create("ItemsSource", typeof(ObservableCollection<GroupItem>), typeof(VideoTab), default(ObservableCollection<GroupItem>));

        /// <summary>
        /// Gets or sets source of list items
        /// </summary>
        public ObservableCollection<GroupItem> ItemsSource
        {
            get { return (ObservableCollection<GroupItem>)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        /// <summary>
        /// Identifies the GetInformationCommand bindable property
        /// </summary>
        public static readonly BindableProperty GetInformationsCommandProperty = BindableProperty.Create("GetInformationsCommand", typeof(ICommand), typeof(VideoTab), default(ICommand));

        /// <summary>
        /// Gets or sets GetInformation Command
        /// </summary>
        public ICommand GetInformationsCommand
        {
            get { return (ICommand)GetValue(GetInformationsCommandProperty); }
            set { SetValue(GetInformationsCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the VideoTabStatus bindable property
        /// </summary>
        public static readonly BindableProperty VideoTabStatusProperty = BindableProperty.Create("VideoTabStatus", typeof(TabStatus), typeof(VideoTab), TabStatus.Default);

        /// <summary>
        /// Gets or sets VideoTabStatus
        /// </summary>
        public TabStatus VideoTabStatus
        {
            get { return (TabStatus)GetValue(VideoTabStatusProperty); }
            set { SetValue(VideoTabStatusProperty, value); }
        }

        /// <summary>
        /// Identifies the IsNeededClearList bindable property
        /// </summary>
        public static readonly BindableProperty IsNeededClearListProperty = BindableProperty.Create("IsNeededClearList", typeof(bool), typeof(ImageTab), false);

        /// <summary>
        /// Gets or sets IsNeededClearList
        /// </summary>
        public bool IsNeededClearList
        {
            get { return (bool)GetValue(IsNeededClearListProperty); }
            set { SetValue(IsNeededClearListProperty, value); }
        }

        /// <summary>
        /// Identifies the ChangeTabStatusCommand bindable property
        /// </summary>
        public static readonly BindableProperty ChangeTabStatusCommandProperty = BindableProperty.Create("ChangeTabStatusCommand", typeof(ICommand), typeof(VideoTab), default(ICommand));

        /// <summary>
        /// Gets or sets ChangeTabStatus Command
        /// </summary>
        public ICommand ChangeTabStatusCommand
        {
            get { return (ICommand)GetValue(ChangeTabStatusCommandProperty); }
            set { SetValue(ChangeTabStatusCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the ChangeSortOptionCommand bindable property
        /// </summary>
        public static readonly BindableProperty ChangeSortOptionCommandProperty = BindableProperty.Create("ChangeSortOptionCommand", typeof(ICommand), typeof(VideoTab), default(ICommand));

        /// <summary>
        /// Gets or sets ChangeSortOption Command
        /// </summary>
        public ICommand ChangeSortOptionCommand
        {
            get { return (ICommand)GetValue(ChangeSortOptionCommandProperty); }
            set { SetValue(ChangeSortOptionCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the ChangeSourceCommand bindable property
        /// </summary>
        public static readonly BindableProperty ChangeSourceCommandProperty = BindableProperty.Create("ChangeSourceCommand", typeof(ICommand), typeof(VideoTab), default(ICommand));

        /// <summary>
        /// Gets or sets ChangeSourceCommand Command
        /// </summary>
        public ICommand ChangeSourceCommand
        {
            get { return (ICommand)GetValue(ChangeSourceCommandProperty); }
            set { SetValue(ChangeSourceCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the DeleteContentCommand bindable property
        /// </summary>
        public static readonly BindableProperty DeleteContentCommandProperty = BindableProperty.Create("DeleteContentCommand", typeof(ICommand), typeof(VideoTab), default(ICommand));

        /// <summary>
        /// Gets or sets DeleteContent Command
        /// </summary>
        public ICommand DeleteContentCommand
        {
            get { return (ICommand)GetValue(DeleteContentCommandProperty); }
            set { SetValue(DeleteContentCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the DeleteContentCommand bindable property
        /// </summary>
        public static readonly BindableProperty SelectAllContentCommandProperty = BindableProperty.Create("SelectAllContentCommand", typeof(ICommand), typeof(VideoTab), default(ICommand));

        /// <summary>
        /// Gets or sets SelectAllContent Command
        /// </summary>
        public ICommand SelectAllContentCommand
        {
            get { return (ICommand)GetValue(SelectAllContentCommandProperty); }
            set { SetValue(SelectAllContentCommandProperty, value); }
        }

        /// <summary>
        /// Identifies the OnClickCommand bindable property
        /// </summary>
        public static readonly BindableProperty OnClickCommandProperty = BindableProperty.Create("OnClickCommand", typeof(ICommand), typeof(VideoTab), default(ICommand));

        /// <summary>
        /// Gets or sets OnClick Command
        /// </summary>
        public ICommand OnClickCommand
        {
            get
            {
                return (ICommand)GetValue(OnClickCommandProperty);
            }

            set
            {
                SetValue(OnClickCommandProperty, value);
            }
        }

        /// <summary>
        /// Identifies the SelectedCount bindable property
        /// </summary>
        public static readonly BindableProperty SelectedCountProperty = BindableProperty.Create("SelectedCount", typeof(int), typeof(VideoTab), 0);

        /// <summary>
        /// Gets or sets count of selected item
        /// </summary>
        public int SelectedCount
        {
            get { return (int)GetValue(SelectedCountProperty); }
            set { SetValue(SelectedCountProperty, value); }
        }

        /// <summary>
        /// A list of bottom buttons
        /// </summary>
        private List<KeyValuePair<double, Button>> BottomButtonList;

        /// <summary>
        /// A constructor
        /// </summary>
        public VideoTab()
        {
            BindingContext = VideoTabViewModelLocator.ViewModel;
            InitializeComponent();
            InitializeData();
            InitializeSize();
            InitializeFonts();
            InitializeFooter();
            ItemsSource.CollectionChanged += ItemsSourceCollectionChanged;
            PropertyChanged += VideoTabPropertyChanged;
        }

        /// <summary>
        /// This method is called when the properties is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A propertyChanged event argument</param>
        private void VideoTabPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("IsNeededClearList"))
            {
                if (IsNeededClearList)
                {
                    BottomButtonList.Clear();
                }
            }
            else if (e.PropertyName.Equals("VideoTabStatus"))
            {
                SetFooterFocusChain(VideoTabScrollView.ScrollX);
            }
        }

        /// <summary>
        /// Initialize the data that is used in this class
        /// </summary>
        private void InitializeData()
        {
            BottomButtonList = new List<KeyValuePair<double, Button>>();
            VideoGroupList = new List<VideoGroup>();
        }

        /// <summary>
        /// A method for initializing several fonts that are used in this class
        /// </summary>
        private void InitializeFonts()
        {
            LabelNoContents.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetFontWeight(FontWeight.Light);
            LabelNoContents.TextColor = Color.FromRgba(255, 255, 255, 20);
        }

        /// <summary>
        /// Initialize the size that is used in this class
        /// </summary>
        private void InitializeSize()
        {
            VideoTabList.Padding = new Thickness(SizeUtils.GetWidthSize(96), SizeUtils.GetWidthSize(0));
            VideoTabList.Spacing = SizeUtils.GetWidthSize(36);
            ImgNoContents.Margin = new Thickness(0, SizeUtils.GetHeightSize(266), 0, SizeUtils.GetHeightSize(28));
            ImgNoContents.HeightRequest = SizeUtils.GetHeightSize(88);
            ImgNoContents.WidthRequest = SizeUtils.GetWidthSize(68);
            LabelNoContents.HeightRequest = SizeUtils.GetHeightSize(32);
            LabelNoContents.FontSize = SizeUtils.GetFontSize(28);
        }

        /// <summary>
        /// Initialize elements on footer area
        /// </summary>
        private void InitializeFooter()
        {
            FooterNormal.OnDropdownSourceItemSelected += OnSourceChanged;
            FooterNormal.OnDropdownSortItemSelected += OnSortOptionChanged;
            FooterNormal.OnSelectedOptionIndexChanged += OnOptionSelected;

            FooterDelete.SelecteAllButtonEvent += OnSelectAllClicked;
            FooterDelete.OKButtonEvent += OnOKClicked;
            FooterDelete.CancelButtonEvent += OnCancelClicked;
        }

        /// <summary>
        /// A method for initializing page when the page is appeared
        /// </summary>
        protected override void InitializePage()
        {
            if (IsContentReady == false)
            {
                GetInformationsCommand?.Execute("");
                IsContentReady = true;
            }
        }

        /// <summary>
        /// A method for finalizing page when the page is disappeared
        /// </summary>
        protected override void FinalizePage()
        {
            /*
            ItemsSource.Clear();
            VideoTabList.Children.Clear();
            BottomButtonList.Clear();
            if (VideoTabStatus == TabStatus.Delete)
            {
                ChangeTabStatusCommand?.Execute("");
                FooterNormal.IsVisible = true;
                FooterDelete.IsVisible = false;
            }
            IsContentReady = false;
            */
        }

        /// <summary>
        /// Occurs when an item is added, removed, changed, moved, or the entire list is refreshed.
        /// </summary>
        /// <param name="sender">The object that raised the event</param>
        /// <param name="e">Information about the event</param>
        private void ItemsSourceCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action.ToString().Equals("Add"))
            {
                if (VideoNoContents.IsVisible)
                {
                    VideoNoContents.IsVisible = false;
                    VideoTabScrollView.IsVisible = true;
                    FooterNormal.IsFooterEnabled = true;
                }

                VideoGroup groupView = new VideoGroup();

                groupView.PropertyChanged += (se, ev) =>
                {
                    if (ev.PropertyName.Equals("ItemsSource"))
                    {
                        groupView.BottomFocusList.ForEach((item) =>
                        {
                            var key = groupView.X + item.Key;
                            BottomButtonList.Add(new KeyValuePair<double, Button>(key, item.Value));
                        });

                        SetFooterFocusChain(0);
                    }
                };

                var GroupItem = e.NewItems[0];
                groupView.SetClickCommand(OnClickCommand);
                groupView.ContextPopupItemSelectedHandler += (info, item) =>
                {
                    ShowPopup(info, item);
                };

                VideoTabList.Children.Add(groupView);
                groupView.BindingContext = GroupItem;
                VideoGroupList.Add(groupView);

                groupView.GroupTitleFocusArea.Focused += (se, ev) =>
                {
                    ScrollToTarget(groupView.X, SizeUtils.GetWidthSize(300), SizeUtils.GetWidthSize(96));
                };
                groupView.GroupItemFocused += (se, ev) =>
                {
                    ScrollToTarget(ev.PositionX, SizeUtils.GetWidthSize(320), SizeUtils.GetWidthSize(96));
                };

                if (VideoTabList.Children.Count > 1)
                {
                    VideoGroup leftGroup = VideoTabList.Children[VideoTabList.Children.Count - 2] as VideoGroup;

                    leftGroup.GroupTitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(groupView.GroupTitleFocusArea);
                    groupView.GroupTitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(leftGroup.GroupTitleFocusArea);

                    leftGroup.GroupTitleFocusArea.Clicked += (se, ev) =>
                    {
                        leftGroup.GroupTitleFocusArea.On<Xamarin.Forms.PlatformConfiguration.Tizen>().MoveFocusRight();
                    };

                    var leftList = leftGroup.RightFocusList;
                    var rightList = groupView.LeftFocusList;
                    var minCount = leftList.Count > rightList.Count ? rightList.Count : leftList.Count;

                    for (var i = 0; i < minCount; i++)
                    {
                        leftList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(rightList[i]);
                        rightList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(leftList[i]);
                    }
                    if (leftList.Count != rightList.Count)
                    {

                        if (leftList.Count > rightList.Count)
                        {
                            for (var i = rightList.Count; i < leftList.Count; i++)
                            {
                                leftList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusRightView(rightList[rightList.Count - 1]);
                            }
                        }
                        else
                        {
                            for (var i = leftList.Count; i < rightList.Count; i++)
                            {
                                rightList[i].On<Xamarin.Forms.PlatformConfiguration.Tizen>().SetNextFocusLeftView(leftList[leftList.Count - 1]);
                            }
                        }
                    }
                }
            }
            else if (e.Action.ToString().Equals("Reset"))
            {
                VideoTabList.Children.Clear();
                VideoGroupList.Clear();
                if (!VideoNoContents.IsVisible)
                {
                    VideoTabScrollView.IsVisible = false;
                    VideoNoContents.IsVisible = true;
                    FooterNormal.IsFooterEnabled = false;
                }
            }
        }

        /// <summary>
        /// Sets the focust chain on footer area
        /// </summary>
        /// <param name="scrollX">The current X position of the scroll</param>
        private void SetFooterFocusChain(double scrollX)
        {
            FooterNormal.SetFocusChain();
            FooterDelete.SetFocusChain();

            var Padding = SizeUtils.GetWidthSize(96);
            var ItemWidth = SizeUtils.GetWidthSize(320);

            var FocusableBoundFrom = scrollX;
            var FocusableBoundTo = scrollX + SizeUtils.BaseScreenWidth - ItemWidth;
            var list = BottomButtonList.FindAll((pair) =>
            {
                return (pair.Key >= FocusableBoundFrom && pair.Key <= FocusableBoundTo);
            });

            for (var buttonIndex = 0; buttonIndex < list.Count; buttonIndex++)
            {
                var btn = list[buttonIndex].Value;
                if (buttonIndex == 0)
                {
                    if (VideoTabStatus == TabStatus.Delete)
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterDelete.GetSelectAllButton());
                        TizenVisualElement.SetNextFocusUpView(FooterDelete.GetSelectAllButton(), btn);
                    }
                    else
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterNormal.DropdownSource);
                        TizenVisualElement.SetNextFocusUpView(FooterNormal.DropdownSource, btn);
                    }
                }
                else if (buttonIndex == list.Count - 1)
                {
                    if (VideoTabStatus == TabStatus.Delete)
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterDelete.GetCancelButton());
                        TizenVisualElement.SetNextFocusUpView(FooterDelete.GetCancelButton(), btn);
                    }
                    else
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterNormal.ButtonOption);
                        TizenVisualElement.SetNextFocusUpView(FooterNormal.ButtonOption, btn);
                    }
                }
                else
                {
                    if (VideoTabStatus == TabStatus.Delete)
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterDelete.GetOkButton());
                        TizenVisualElement.SetNextFocusUpView(FooterDelete.GetOkButton(), btn);
                    }
                    else
                    {
                        TizenVisualElement.SetNextFocusDownView(btn, FooterNormal.DropdownSort);
                        TizenVisualElement.SetNextFocusUpView(FooterNormal.DropdownSort, btn);
                    }
                }
            }
        }

        /// <summary>
        /// Moves the scroller to the given target
        /// </summary>
        /// <param name="targetX">The given target</param>
        /// <param name="targetWidth">The width of the given target</param>
        /// <param name="padding">The padding of the given target</param>
        private void ScrollToTarget(double targetX, int targetWidth, int padding)
        {
            var ScrollX = VideoTabScrollView.ScrollX;

            var ScrollBoundFrom = targetX - (SizeUtils.BaseScreenWidth - targetWidth - padding);
            ScrollBoundFrom = ScrollBoundFrom > 0 ? ScrollBoundFrom : 0;
            var ScrollBoundTo = targetX - padding;
            ScrollBoundTo = ScrollBoundTo > 0 ? ScrollBoundTo : 0;

            // Is Focusable.
            if (ScrollX >= ScrollBoundFrom && ScrollX <= ScrollBoundTo)
            {
                return;
            }

            if (Math.Abs(ScrollX - ScrollBoundFrom) > Math.Abs(ScrollX - ScrollBoundTo))
            {
                VideoTabScrollView.ScrollToAsync(ScrollBoundTo, 0, true);
                SetFooterFocusChain(ScrollBoundTo);
            }
            else
            {
                VideoTabScrollView.ScrollToAsync(ScrollBoundFrom, 0, true);
                SetFooterFocusChain(ScrollBoundFrom);
            }
        }

        /// <summary>
        /// This method is called when cancel button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void OnCancelClicked(object sender, EventArgs e)
        {
            ChangeTabStatusCommand?.Execute("");
            FooterNormal.IsVisible = true;
            FooterDelete.IsVisible = false;
            SetFooterFocusChain(VideoTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when SelectAll button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private void OnSelectAllClicked(object sender, EventArgs e)
        {
            IsSelectedAll = FooterDelete.IsSelectedAll;
            SelectAllContentCommand?.Execute(IsSelectedAll);

            foreach (var group in VideoGroupList)
            {
                foreach (var item in group.VideoItemList)
                {
                    item.UpdateViewSelectAll(IsSelectedAll);
                }
            }
        }

        /// <summary>
        /// This method is called when OK button is clicked
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">An event's argument</param>
        private async void OnOKClicked(object sender, EventArgs e)
        {
            string title = "Delete";

            if (SelectedCount < 1)
            {
                return;
            }
            else if (SelectedCount > 1)
            {
                title = "Delete Selected";

                if (IsSelectedAll)
                {
                    title = "Delete All";
                }
            }

            bool answer = await DisplayAlert(title, "Do you want to delete?", "YES", "NO");

            if (answer)
            {
                DeleteContentCommand?.Execute(null);
                GetInformationsCommand?.Execute("");
            }

            ChangeTabStatusCommand?.Execute("");
            FooterNormal.IsVisible = true;
            FooterDelete.IsVisible = false;
            SetFooterFocusChain(VideoTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when Source option is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A SelectedItemChanged event's argument</param>
        private void OnSourceChanged(object sender, SelectedItemChangedEventArgs e)
        {
            String storageName = e.SelectedItem as String;

            ChangeSourceCommand?.Execute(storageName);
            SetFooterFocusChain(VideoTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when Sort option is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A SelectedItemChanged event's argument</param>
        private void OnSortOptionChanged(object sender, SelectedItemChangedEventArgs e)
        {
            ChangeSortOptionCommand?.Execute(e.SelectedItem);
            SetFooterFocusChain(VideoTabScrollView.ScrollX);
        }

        /// <summary>
        /// This method is called when option menu is changed
        /// </summary>
        /// <param name="sender">The source of the event</param>
        /// <param name="e">A SelectedItemChanged event's argument</param>
        private void OnOptionSelected(object sender, ContextPopupSelectedEventArgs e)
        {
            var label = e.Item.Label.Trim().ToLower();
            if (label.Equals("delete"))
            {
                ChangeTabStatusCommand?.Execute("");
                FooterNormal.IsVisible = false;
                FooterDelete.IsVisible = true;
                SetFooterFocusChain(VideoTabScrollView.ScrollX);
            }
            else if (label.Equals("disk info"))
            {
                ShowDiskInfoPopup();
            }
        }

        /// <summary>
        /// A method for running hide animation
        /// </summary>
#pragma warning disable CS1998 // Async method lacks 'await' operators and will run synchronously
        public override async void RunHideAnimation()
#pragma warning restore CS1998 // Async method lacks 'await' operators and will run synchronously
        {
            var h12 = SizeUtils.GetHeightSize(12);
#pragma warning disable CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
            VideoTabList.TranslateTo(0, h12, 667);
#pragma warning restore CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
        }

        /// <summary>
        /// A method for running show animation
        /// </summary>
#pragma warning disable CS1998 // Async method lacks 'await' operators and will run synchronously
        public override async void RunShowAnimation()
#pragma warning restore CS1998 // Async method lacks 'await' operators and will run synchronously
        {
            var h0 = SizeUtils.GetHeightSize(0);
            var h12 = SizeUtils.GetHeightSize(12);

            VideoTabList.TranslationY = h12;
            VideoTabList.Opacity = 0;
#pragma warning disable CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
            VideoTabList.FadeTo(0.99, 667);
            VideoTabList.TranslateTo(0, h0, 667);
#pragma warning restore CS4014 // Because this call is not awaited, execution of the current method continues before the call is completed
        }

        protected override void OnAppearing()
        {
            base.OnAppearing();
            // add Listener
            App.KeyUpEvent.On += MenuKeyListener;
        }

        protected override void OnDisappearing()
        {
            base.OnDisappearing();
            // remove listener
            App.KeyUpEvent.On -= MenuKeyListener;
        }

        private void MenuKeyListener(object sender, EventArgs e)
        {
            ElmSharp.EcoreKeyEventArgs args = e as ElmSharp.EcoreKeyEventArgs;

            if (args.KeyName.Contains(ElmSharp.EvasKeyEventArgs.PlatformMenuButtonName))
            {
                NotifyMenuKeyPressed();
            }
        }

        private void NotifyMenuKeyPressed()
        {
            if (VideoTabList.Children.Count > 0)
            {
                foreach (VideoGroup group in VideoTabList.Children)
                {
                    if (group.NotifyMenuKeyPressedToFocusedItem())
                    {
                        break;
                    }
                }

            }
        }

        private async void ShowPopup(MediaInformationEx info, VideoContextPopupItem item)
        {
            switch (item)
            {
                case VideoContextPopupItem.FileInfo:
                    string message = "File name  : " + info.MediaContentInformation.Title + "\n";
                    message += "File size : " + info.MediaContentInformation.FileSize + "\n";
                    message += "File format : " + info.MediaContentInformation.MimeType + "\n";
                    message += "File size : " + info.MediaContentInformation.FileSize + "\n";
                    message += "File path : " + info.MediaContentInformation.Path;

                    await DisplayAlert("File Information", message, "Close");
                    break;
                case VideoContextPopupItem.Delete:
                    bool answer = await DisplayAlert("Delete", "Delete '" + info.MediaContentInformation.Title + "'?", "Yes", "No");
                    if (answer)
                    {
                        DeleteContentCommand?.Execute(info);
                        GetInformationsCommand?.Execute("");
                    }

                    break;
            }
        }

        /// <summary>
        /// A method for showing Disk Information Popup
        /// </summary>
        private async void ShowDiskInfoPopup()
        {
            int numberOfFiles = 0;

            foreach (var item in VideoGroupList)
            {
                numberOfFiles += item.VideoItemList.Count;
            }

            var message = "Number of files  : " + numberOfFiles + "\n";
            await DisplayAlert("Disk Information", message, "Close");
        }
    }
}
